@@ -75,8 +75,12 @@
 
 #define  HAVE_FLOCK
 
+#define  HAVE_KILLPG
+
 #undef  HAVE_POSIX_OPENPT
 
+#define  HAVE_GETTIMEOFDAY
+
 #undef  USE_WIN32API
 
 #define  HAVE_STDINT_H
