@@ -28,6 +28,26 @@
 #endif
 
 
+#ifdef  KIK_DEBUG
+
+#define  KIK_TESTIT(func, args)  TEST_ ## func args
+#define  KIK_TESTIT_ONCE(func, args) \
+	{ \
+		static int  func ## _tested ; \
+		if( ! func ## _tested) \
+		{ \
+			func ## _tested = 1 ; \
+			TEST_ ## func args ; \
+		} \
+	}
+#else
+
+#define  KIK_TESTIT(func, args)
+#define  KIK_TESTIT_ONCE(func, args)
+
+#endif
+
+
 #define  kik_debug_printf(...) \
 	(__android_log_print( ANDROID_LOG_INFO , "" , __VA_ARGS__) >= 0)
 
