@@ -9,6 +9,7 @@
 #include  <linux/keyboard.h>
 
 #include  <kiklib/kik_debug.h>
+#include  <kiklib/kik_conf_io.h>
 
 #include  "../x_window.h"
 #include  "../../common/c_animgif.c"
@@ -24,11 +25,6 @@ static Display  _display ;
 static int  rotate_display ;
 static int  locked ;
 
-static int  visible_frame_changed ;
-static int  new_yoffset ;
-static u_int  new_width ;
-static u_int  new_height ;
-
 
 /* --- static functions --- */
 
@@ -208,7 +204,24 @@ kcode_to_ksym(
 			return  '\x1b' ;
 
 		default:
-			return  kcode + 0x100 ;
+			if( kcode == -0x3ed)
+			{
+				/* XXX for Nihongo Full Keyboard */
+				return  XK_Henkan_Mode ;
+			}
+			else if( kcode == -0x3ec)
+			{
+				/* XXX for Nihongo Full Keyboard */
+				return  XK_Muhenkan ;
+			}
+			else if( kcode < 0)
+			{
+				return  0 ;
+			}
+			else
+			{
+				return  kcode + 0x100 ;
+			}
 		}
 	}
 }
@@ -990,22 +1003,6 @@ x_display_process_event(
 	int  ident
 	)
 {
-	if( visible_frame_changed)
-	{
-		/* XXX should synchronize */
-		_display.yoffset = new_yoffset ;
-		_disp.width = new_width ;
-		_disp.height = new_height ;
-
-		visible_frame_changed = 0 ;
-
-		if( _disp.num_of_roots > 0)
-		{
-			x_window_resize_with_margin( _disp.roots[0] ,
-				_disp.width , _disp.height , NOTIFY_TO_MYSELF) ;
-		}
-	}
-
 	/* Process this event. */
 	if( source)
 	{
@@ -1384,17 +1381,71 @@ x_display_get_bitmap(
 	return  image ;
 }
 
-
-/* Called in the main thread (not in the native activity thread) */
 void
-Java_mlterm_native_1activity_MLActivity_visibleFrameChanged(
-	JNIEnv *  env ,
-	jobject  this ,
-	jint  yoffset ,
-	jint  width ,
-	jint  height
+x_display_request_text_selection(void)
+{
+	JNIEnv *  env ;
+	JavaVM *  vm ;
+	jobject  this ;
+
+	vm = _display.app->activity->vm ;
+
+	(*vm)->AttachCurrentThread( vm , &env , NULL) ;
+
+	this = _display.app->activity->clazz ;
+	(*env)->CallVoidMethod( env , this ,
+		(*env)->GetMethodID( env , (*env)->GetObjectClass( env , this) ,
+			"getTextFromClipboard" , "()V")) ;
+
+	(*vm)->DetachCurrentThread(vm) ;
+}
+
+void
+x_display_send_text_selection(
+	u_char *  sel_data ,
+	size_t  sel_len
 	)
 {
+	u_char *  p ;
+	JNIEnv *  env ;
+	JavaVM *  vm ;
+	jobject  this ;
+	jstring  str ;
+
+	if( ! ( p = alloca( sel_len + 1)))
+	{
+		return ;
+	}
+
+	sel_data = memcpy( p , sel_data , sel_len) ;
+	sel_data[sel_len] = '\0' ;
+
+	vm = _display.app->activity->vm ;
+
+	(*vm)->AttachCurrentThread( vm , &env , NULL) ;
+
+	this = _display.app->activity->clazz ;
+	(*env)->CallVoidMethod( env , this ,
+		(*env)->GetMethodID( env , (*env)->GetObjectClass( env , this) ,
+			"setTextToClipboard" , "(Ljava/lang/String;)V") ,
+		(*env)->NewStringUTF( env , sel_data)) ;
+
+	(*vm)->DetachCurrentThread(vm) ;
+}
+
+/* Called in the main thread (not in the native activity thread) */
+
+void
+x_display_resize(
+	int  yoffset ,
+	int  width ,
+	int  height ,
+	int  (*need_resize)( u_int , u_int , u_int , u_int)
+	)
+{
+	u_int  new_width ;
+	u_int  new_height ;
+
 #ifdef  DEBUG
 	kik_debug_printf( "Visible frame changed yoff %d w %d h %d => yoff %d w %d h %d\n" ,
 		_display.yoffset , _disp.width , _disp.height , yoffset , width , height) ;
@@ -1406,29 +1457,10 @@ Java_mlterm_native_1activity_MLActivity_visibleFrameChanged(
 		return ;
 	}
 
-	if( _disp.roots)
+	_display.yoffset = yoffset ;
+
+	if( _disp.num_of_roots == 0)
 	{
-		/* XXX should synchronize */
-
-		new_yoffset = yoffset ;
-
-		if( rotate_display)
-		{
-			new_width = height ;
-			new_height = width ;
-		}
-		else
-		{
-			new_width = width ;
-			new_height = height ;
-		}
-
-		visible_frame_changed = 1 ;
-	}
-	else
-	{
-		_display.yoffset = yoffset ;
-
 		if( rotate_display)
 		{
 			_disp.width = height ;
@@ -1439,24 +1471,69 @@ Java_mlterm_native_1activity_MLActivity_visibleFrameChanged(
 			_disp.width = width ;
 			_disp.height = height ;
 		}
+
+		return ;
 	}
+
+	if( rotate_display)
+	{
+		new_width = height ;
+		new_height = width ;
+	}
+	else
+	{
+		new_width = width ;
+		new_height = height ;
+	}
+
+	if( need_resize &&
+	    ! (*need_resize)( _disp.width , _disp.height , new_width , new_height))
+	{
+		return ;
+	}
+
+	_disp.width = new_width ;
+	_disp.height = new_height ;
+
+	x_window_resize_with_margin( _disp.roots[0] ,
+			_disp.width , _disp.height , NOTIFY_TO_MYSELF) ;
 }
 
-jint
-Java_mlterm_native_1activity_MLActivity_hashPath(
+jstring
+Java_mlterm_native_1activity_MLActivity_convertToTmpPath(
 	JNIEnv *  env ,
 	jobject  this ,
 	jstring  jstr	/* must be original URL. (Don't specify /sdcard/.mlterm/anim*) */
 	)
 {
-	const char *  path ;
-	jint  hash ;
+	char *  dst_path ;
+	char *  dir ;
 
-	path = (*env)->GetStringUTFChars( env , jstr , NULL) ;
-	hash = hash_path( path) ;
-	(*env)->ReleaseStringUTFChars( env , jstr , path) ;
+	if( ! ( dir = kik_get_user_rc_path( "mlterm/")))
+	{
+		return  NULL ;
+	}
 
-	return  hash ;
+	if( ! ( dst_path = alloca( strlen(dir) + 8 + 5 /* hash <= 65535 */ + 1)))
+	{
+		jstr = NULL ;
+	}
+	else
+	{
+		const char *  src_path ;
+		jint  hash ;
+
+		src_path = (*env)->GetStringUTFChars( env , jstr , NULL) ;
+		hash = hash_path( src_path) ;
+		(*env)->ReleaseStringUTFChars( env , jstr , src_path) ;
+
+		sprintf( dst_path , "%sanim%d.gif" , dir , hash) ;
+		jstr = ((*env)->NewStringUTF)( env , dst_path) ;
+	}
+
+	free( dir) ;
+
+	return  jstr ;
 }
 
 void
@@ -1468,19 +1545,28 @@ Java_mlterm_native_1activity_MLActivity_splitAnimationGif(
 {
 	const char *  str ;
 	const char *  path ;
-	char tmp[24 + 5 + 1] ;
+	char *  dir ;
+	char *  tmp ;
 	int  hash ;
 
+	if( ! ( dir = kik_get_user_rc_path( "mlterm/")))
+	{
+		return ;
+	}
+
 	path = str = (*env)->GetStringUTFChars( env , jstr , NULL) ;
 	hash = hash_path( path) ;
 
-	if( strstr( path , "://"))
+	if( strstr( path , "://") &&
+	    ( tmp = alloca( strlen( dir) + 8 + 5 /* hash <= 65535 */ + 1)))
 	{
-		sprintf( tmp , "/sdcard/.mlterm/anim%d.gif" , hash) ;
+		sprintf( tmp , "%sanim%d.gif" , dir , hash) ;
 		path = tmp ;
 	}
 
-	split_animation_gif( path , "/sdcard/.mlterm/" , hash) ;
+	split_animation_gif( path , dir , hash) ;
+
+	free( dir) ;
 
 	(*env)->ReleaseStringUTFChars( env , jstr , str) ;
 }
