@@ -50,5 +50,12 @@ int  x_cmap_get_pixel_rgb( u_int8_t *  red , u_int8_t *  green ,
 
 u_char *  x_display_get_bitmap( char *  path , u_int *  width , u_int *  height) ;
 
+void  x_display_request_text_selection(void) ;
+
+void  x_display_send_text_selection( u_char *  sel_data , size_t  sel_len) ;
+
+void  x_display_resize( int  yoffset , int  width , int  height ,
+	int (*need_resize)( u_int , u_int , u_int , u_int)) ;
+
 
 #endif
