@@ -16,7 +16,11 @@ import  android.view.View ;
 import  android.view.ViewGroup ;
 import  android.text.InputType ;
 import  android.content.Context ;
+import  android.content.ClipboardManager ;
+import  android.content.ClipData ;
+import  android.content.ClipDescription ;
 import  android.os.Bundle ;
+import  android.os.Handler ;
 import  android.graphics.Rect ;
 import  android.graphics.BitmapFactory ;
 import  android.graphics.Bitmap ;
@@ -34,13 +38,16 @@ public class MLActivity extends NativeActivity
 
 	private native void  visibleFrameChanged( int  yoffset , int  width , int  height) ;
 	private native void  commitText( String  str) ;
+	private native void  commitTextNoLock( String  str) ;
 	private native void  preeditText( String  str) ;
-	private native int  hashPath( String  path) ;
+	private native String  convertToTmpPath( String  path) ;
 	private native void  splitAnimationGif( String  path) ;
 
-	private boolean  isPreediting ;
 	private String  keyString ;
 	private View  contentView ;
+	private int  inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE ;
+	private int  imeOptions = EditorInfo.IME_FLAG_NO_ENTER_ACTION | EditorInfo.IME_ACTION_DONE ;
+	private ClipboardManager  clipMan ;
 
 	private class TextInputConnection extends BaseInputConnection
 	{
@@ -54,7 +61,6 @@ public class MLActivity extends NativeActivity
 		{
 			super.commitText( text , newCursorPosition) ;
 
-			isPreediting = false ;
 			commitText( text.toString()) ;
 
 			return  true ;
@@ -65,7 +71,6 @@ public class MLActivity extends NativeActivity
 		{
 			super.setComposingText( text , newCursorPosition) ;
 
-			isPreediting = true ;
 			preeditText( text.toString()) ;
 
 			return  true ;
@@ -76,7 +81,7 @@ public class MLActivity extends NativeActivity
 		{
 			super.finishComposingText() ;
 
-			isPreediting = false ;
+			preeditText( "") ;
 
 			return  true ;
 		}
@@ -103,20 +108,57 @@ public class MLActivity extends NativeActivity
 		@Override
 		public InputConnection onCreateInputConnection( EditorInfo  outAttrs)
 		{
-			outAttrs.inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE ;
-			outAttrs.imeOptions = EditorInfo.IME_FLAG_NO_ENTER_ACTION |
-			                      EditorInfo.IME_ACTION_DONE ;
+			outAttrs.inputType = inputType ;
+			outAttrs.imeOptions = imeOptions ;
 
 			return  new TextInputConnection( this , true) ;
 		}
 	}
 
+	private void forceAsciiInput()
+	{
+		if( true)
+		{
+			inputType |= InputType.TYPE_TEXT_VARIATION_URI ;
+		}
+		else
+		{
+			inputType |= InputType.TYPE_TEXT_VARIATION_VISIBLE_PASSWORD ;
+		}
+
+		imeOptions |= 0x80000000 ; /* EditorInfo.IME_FLAG_FORCE_ASCII ; (API level 16) */
+
+		((InputMethodManager)getSystemService( Context.INPUT_METHOD_SERVICE)).restartInput(
+				contentView) ;
+	}
+
 	private void showSoftInput()
 	{
 		((InputMethodManager)getSystemService( Context.INPUT_METHOD_SERVICE)).showSoftInput(
 			contentView , InputMethodManager.SHOW_FORCED) ;
 	}
 
+	private void setTextToClipboard( String  str)
+	{
+		ClipData.Item  item = new ClipData.Item( str) ;
+		String[]  mimeType = new String[1] ;
+		mimeType[0] = ClipDescription.MIMETYPE_TEXT_PLAIN ;
+		ClipData  cd = new ClipData(
+						new ClipDescription( "text_data" , mimeType) , item) ;
+		clipMan.setPrimaryClip( cd) ;
+	}
+
+	private void getTextFromClipboard()
+	{
+		ClipData  cd = clipMan.getPrimaryClip() ;
+		if( cd != null)
+		{
+			ClipData.Item  item = cd.getItemAt( 0) ;
+
+			commitTextNoLock( item.getText().toString()) ;
+		}
+	}
+
 	@Override
 	public boolean dispatchKeyEvent( KeyEvent  event)
 	{
@@ -163,12 +205,14 @@ public class MLActivity extends NativeActivity
 					Rect  r = new Rect() ;
 					v.getWindowVisibleDisplayFrame(r) ;
 					visibleFrameChanged( r.top , r.right , r.bottom - r.top) ;
-					if( isPreediting)
-					{
-						commitText( "") ;	/* Stop preediting. */
-					}
 				}
 			}) ;
+
+		/*
+		 * It is prohibited to call getSystemService(CLIPBOARD_SERVICE)
+		 * in native activity thread.
+		 */
+		clipMan = (ClipboardManager)getSystemService( Context.CLIPBOARD_SERVICE) ;
 	}
 
 	@Override
@@ -189,7 +233,7 @@ public class MLActivity extends NativeActivity
 		showSoftInput() ;
 	}
 
-	private String saveUnifont()
+	private String saveDefaultFont()
 	{
 		File  file = getFileStreamPath( "unifont.pcf") ;
 		if( ! file.exists() ||
@@ -234,8 +278,7 @@ public class MLActivity extends NativeActivity
 
 				if( path.indexOf( ".gif") != -1)
 				{
-					String  tmp = "/sdcard/.mlterm/anim" +
-									Integer.toString( hashPath( path)) + ".gif" ;
+					String  tmp = convertToTmpPath( path) ;
 					OutputStream  os = new FileOutputStream( tmp) ;
 
 					int  len ;
@@ -255,7 +298,7 @@ public class MLActivity extends NativeActivity
 			}
 			else
 			{
-				if( path.indexOf( ".gif") != -1)
+				if( path.indexOf( "mlterm/anim") == -1 && path.indexOf( ".gif") != -1)
 				{
 					splitAnimationGif( path) ;
 				}
