@@ -5,7 +5,7 @@
 #include  <stdio.h>
 #include  <ibus.h>
 
-#include  <kiklib/kik_list.h>
+#include  <kiklib/kik_slist.h>
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>
 
@@ -53,16 +53,17 @@ typedef struct im_ibus
 	u_int  prev_num_of_cands ;
 #endif
 
-}  im_ibus_t ;
+	struct im_ibus *  next ;
 
-KIK_LIST_TYPEDEF( im_ibus_t) ;
+}  im_ibus_t ;
 
 
 /* --- static variables --- */
 
 static int  is_init ;
 static IBusBus *  ibus_bus ;
-static KIK_LIST( im_ibus_t)  ibus_list = NULL ;
+static int  ibus_bus_fd = -1 ;
+static im_ibus_t *  ibus_list = NULL ;
 static int  ref_count = 0 ;
 static mkf_parser_t *  parser_utf8 = NULL ;
 static x_im_export_syms_t *  syms = NULL ; /* mlterm internal symbols */
@@ -147,8 +148,13 @@ update_preedit_text(
 			if( (*ibus->im.listener->get_spot)(
 					ibus->im.listener->self , NULL , 0 , &x , &y))
 			{
+				u_int  line_height ;
+
+				line_height = (*ibus->im.listener->get_line_height)(
+						ibus->im.listener->self) ;
 				ibus_input_context_set_cursor_location(
-					ibus->context , x , y , 10 , 10) ;
+					ibus->context ,
+					x , y - line_height , 0 , line_height) ;
 			}
 		}
 		
@@ -547,6 +553,70 @@ update_lookup_table(
 #endif	/* USE_FRAMEBUFFER */
 
 
+static void
+connection_handler(void)
+{
+#ifdef  DBUS_H
+	DBusConnection *  connection ;
+
+	connection = ibus_connection_get_connection( ibus_bus_get_connection( ibus_bus)) ;
+
+	dbus_connection_read_write( connection , 0) ;
+
+	while( dbus_connection_dispatch( connection) == DBUS_DISPATCH_DATA_REMAINS) ;
+#else
+#if  0
+	g_dbus_connection_flush_sync( ibus_bus_get_connection( ibus_bus) , NULL , NULL) ;
+#endif
+	g_main_context_iteration( g_main_context_default() , FALSE) ;
+#endif
+}
+
+static int
+add_event_source(void)
+{
+#ifdef  DBUS_H
+	if( ! dbus_connection_get_unix_fd( ibus_connection_get_connection(
+			ibus_bus_get_connection( ibus_bus)) , &ibus_bus_fd))
+	{
+		return  0 ;
+	}
+#else
+	/*
+	 * GIOStream returned by g_dbus_connection_get_stream() is forcibly
+	 * regarded as GSocketConnection.
+	 */
+	if( ( ibus_bus_fd = g_socket_get_fd( g_socket_connection_get_socket(
+				g_dbus_connection_get_stream(
+					ibus_bus_get_connection( ibus_bus))))) == -1)
+	{
+		return  0 ;
+	}
+#endif
+	(*syms->x_event_source_add_fd)( ibus_bus_fd , connection_handler) ;
+	(*syms->x_event_source_add_fd)( IBUS_ID , connection_handler) ;
+
+	return  1 ;
+}
+
+static void
+remove_event_source(
+	int  complete
+	)
+{
+	if( ibus_bus_fd >= 0)
+	{
+		(*syms->x_event_source_remove_fd)( ibus_bus_fd) ;
+		ibus_bus_fd = -1 ;
+	}
+
+	if( complete)
+	{
+		(*syms->x_event_source_remove_fd)( IBUS_ID) ;
+	}
+}
+
+
 /*
  * methods of x_im_t
  */
@@ -560,13 +630,16 @@ delete(
 
 	ibus = (im_ibus_t*) im ;
 
-#ifdef  DBUS_H
-	ibus_object_destroy( (IBusObject*)ibus->context) ;
-#else
-	ibus_proxy_destroy( (IBusProxy*)ibus->context) ;
-#endif
+	if( ibus->context)
+	{
+	#ifdef  DBUS_H
+		ibus_object_destroy( (IBusObject*)ibus->context) ;
+	#else
+		ibus_proxy_destroy( (IBusProxy*)ibus->context) ;
+	#endif
+	}
 
-	kik_list_search_and_remove( im_ibus_t , ibus_list , ibus) ;
+	kik_slist_remove( ibus_list , ibus) ;
 
 	if( ibus->conv)
 	{
@@ -586,39 +659,11 @@ delete(
 
 	if( -- ref_count == 0)
 	{
-		int  fd ;
-
-	#ifdef  DBUS_H
-		if( dbus_connection_get_unix_fd( ibus_connection_get_connection(
-				ibus_bus_get_connection( ibus_bus)) , &fd))
-	#else
-		/*
-		 * GIOStream returned by g_dbus_connection_get_stream() is forcibly
-		 * regarded as GSocketConnection.
-		 */
-		if( ( fd = g_socket_get_fd( g_socket_connection_get_socket(
-				g_dbus_connection_get_stream(
-					ibus_bus_get_connection( ibus_bus))))) != -1)
-	#endif
-		{
-			(*syms->x_event_source_remove_fd)( fd) ;
-		}
-
-		(*syms->x_event_source_remove_fd)( IBUS_ID) ;
+		remove_event_source(1) ;
 
 		ibus_object_destroy( (IBusObject*)ibus_bus) ;
 		ibus_bus = NULL ;
 
-		if( ! kik_list_is_empty( ibus_list))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " ibus list is not empty.\n") ;
-		#endif
-		}
-
-		kik_list_delete( im_ibus_t , ibus_list) ;
-		ibus_list = NULL ;
-
 		if( parser_utf8)
 		{
 			(*parser_utf8->delete)( parser_utf8) ;
@@ -745,6 +790,11 @@ key_event(
 
 	ibus = (im_ibus_t*) im ;
 
+	if( ! ibus->context)
+	{
+		return  1 ;
+	}
+
 	if( event->state & IBUS_IGNORED_MASK)
 	{
 		/* Is put back in forward_key_event */
@@ -814,24 +864,39 @@ set_engine(
 #if  IBUS_CHECK_VERSION(1,5,0)
 static void
 next_engine(
-	im_ibus_t *  ibus
+	IBusInputContext *  context
 	)
 {
 	IBusConfig *  config ;
 	GVariant *  var ;
 
-	config = ibus_bus_get_config( ibus_bus) ;
-	if( ( var = ibus_config_get_value( config , "general" , "preload-engines")))
+	if( ( config = ibus_bus_get_config( ibus_bus)) &&
+	    ( var = ibus_config_get_value( config , "general" , "preload-engines")))
 	{
-		gchar *  cur_name ;
+		static int  show_engines = 1 ;
+		const gchar *  cur_name ;
 		GVariantIter *  iter ;
 		gchar *  name ;
 
 		cur_name = ibus_engine_desc_get_name(
-				ibus_input_context_get_engine( ibus->context)) ;
+				ibus_input_context_get_engine( context)) ;
 
 		g_variant_get( var , "as" , &iter) ;
 
+		if( show_engines)
+		{
+			kik_msg_printf( "iBus engines: ") ;
+			while( g_variant_iter_loop( iter , "s" , &name))
+			{
+				kik_msg_printf( name) ;
+				kik_msg_printf( ",") ;
+			}
+			kik_msg_printf( "\n") ;
+
+			g_variant_iter_init( iter , var) ;
+			show_engines = 0 ;
+		}
+
 		if( g_variant_iter_loop( iter , "s" , &name))
 		{
 			gchar *  first_name ;
@@ -855,7 +920,7 @@ next_engine(
 			}
 			while( loop) ;
 
-			set_engine( ibus->context , name) ;
+			set_engine( context , name) ;
 
 			free( first_name) ;
 		}
@@ -875,8 +940,14 @@ switch_mode(
 
 	ibus =  (im_ibus_t*)  im ;
 
+	if( ! ibus->context)
+	{
+		return  0 ;
+	}
+
 #if  IBUS_CHECK_VERSION(1,5,0)
-	next_engine( ibus) ;
+	next_engine( ibus->context) ;
+	ibus->is_enabled = ! ibus->is_enabled ;
 #else
 	if( ibus->is_enabled)
 	{
@@ -910,6 +981,11 @@ focused(
 
 	ibus =  (im_ibus_t*)  im ;
 
+	if( ! ibus->context)
+	{
+		return ;
+	}
+
 	ibus_input_context_focus_in( ibus->context) ;
 
 	if( ibus->im.cand_screen)
@@ -927,6 +1003,11 @@ unfocused(
 
 	ibus = (im_ibus_t*)  im ;
 
+	if( ! ibus->context)
+	{
+		return ;
+	}
+
 	ibus_input_context_focus_out( ibus->context) ;
 
 	if( ibus->im.cand_screen)
@@ -936,23 +1017,101 @@ unfocused(
 }
 
 
-static void
-connection_handler(void)
+static IBusInputContext *
+context_new(
+	im_ibus_t *  ibus ,
+	char *  engine
+	)
 {
-#ifdef  DBUS_H
-	DBusConnection *  connection ;
+	IBusInputContext *  context ;
 
-	connection = ibus_connection_get_connection( ibus_bus_get_connection( ibus_bus)) ;
+	if( ! ( context = ibus_bus_create_input_context( ibus_bus , "mlterm")))
+	{
+		return  NULL ;
+	}
 
-	dbus_connection_read_write( connection , 0) ;
+	ibus_input_context_set_capabilities( context ,
+	#ifdef  USE_FRAMEBUFFER
+		IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_LOOKUP_TABLE
+	#else
+		IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS | IBUS_CAP_SURROUNDING_TEXT
+	#endif
+		) ;
 
-	while( dbus_connection_dispatch( connection) == DBUS_DISPATCH_DATA_REMAINS) ;
-#else
-#if  0
-	g_dbus_connection_flush_sync( ibus_bus_get_connection( ibus_bus) , NULL , NULL) ;
+	g_signal_connect( context , "update-preedit-text" ,
+			G_CALLBACK( update_preedit_text) , ibus) ;
+	g_signal_connect( context , "hide-preedit-text" ,
+			G_CALLBACK( hide_preedit_text) , ibus) ;
+	g_signal_connect( context , "commit-text" , G_CALLBACK( commit_text) , ibus) ;
+	g_signal_connect( context , "forward-key-event" ,
+			G_CALLBACK( forward_key_event) , ibus) ;
+#ifdef  USE_FRAMEBUFFER
+	g_signal_connect( context , "update-lookup-table" ,
+			G_CALLBACK( update_lookup_table) , ibus) ;
+	g_signal_connect( context , "show-lookup-table" ,
+			G_CALLBACK( show_lookup_table) , ibus) ;
+	g_signal_connect( context , "hide-lookup-table" ,
+			G_CALLBACK( hide_lookup_table) , ibus) ;
 #endif
-	g_main_context_iteration( g_main_context_default() , FALSE) ;
+
+	if( engine)
+	{
+		set_engine( context , engine) ;
+	}
+#if  defined(USE_FRAMEBUFFER) && IBUS_CHECK_VERSION(1,5,0)
+	else
+	{
+		next_engine( context) ;
+	}
 #endif
+
+	return  context ;
+}
+
+static void
+connected(
+	IBusBus *  bus ,
+	gpointer  data
+	)
+{
+	im_ibus_t *  ibus ;
+
+	if( bus != ibus_bus || ! ibus_bus_is_connected( ibus_bus) || ! add_event_source())
+	{
+		return ;
+	}
+
+	for( ibus = ibus_list ; ibus ; ibus = kik_slist_next( ibus_list))
+	{
+		ibus->context = context_new( ibus , NULL) ;
+	}
+}
+
+static void
+disconnected(
+	IBusBus *  bus ,
+	gpointer  data
+	)
+{
+	im_ibus_t *  ibus ;
+
+	if( bus != ibus_bus)
+	{
+		return ;
+	}
+
+	remove_event_source(0) ;
+
+	for( ibus = ibus_list ; ibus ; ibus = kik_slist_next( ibus_list))
+	{
+	#ifdef  DBUS_H
+		ibus_object_destroy( (IBusObject*)ibus->context) ;
+	#else
+		ibus_proxy_destroy( (IBusProxy*)ibus->context) ;
+	#endif
+		ibus->context = NULL ;
+		ibus->is_enabled = FALSE ;
+	}
 }
 
 
@@ -993,8 +1152,6 @@ im_ibus_new(
 
 	if( ! ibus_bus)
 	{
-		int  fd ;
-
 		syms = export_syms ;
 
 		/* g_getenv( "DISPLAY") will be called in ibus_get_socket_path(). */
@@ -1011,33 +1168,20 @@ im_ibus_new(
 			goto  error ;
 		}
 
-	#ifdef  DBUS_H
-		if( ! dbus_connection_get_unix_fd( ibus_connection_get_connection(
-				ibus_bus_get_connection( ibus_bus)) , &fd))
+		if( ! add_event_source())
 		{
 			goto  error ;
 		}
-	#else
-		/*
-		 * GIOStream returned by g_dbus_connection_get_stream() is forcibly
-		 * regarded as GSocketConnection.
-		 */
-		if( ( fd = g_socket_get_fd( g_socket_connection_get_socket(
-				g_dbus_connection_get_stream(
-					ibus_bus_get_connection( ibus_bus))))) == -1)
-		{
-			goto  error ;
-		}
-	#endif
-		(*syms->x_event_source_add_fd)( fd , connection_handler) ;
-		(*syms->x_event_source_add_fd)( IBUS_ID , connection_handler) ;
-
-		kik_list_new( im_ibus_t , ibus_list) ;
 
 		if( ! ( parser_utf8 = (*syms->ml_parser_new)( ML_UTF8)))
 		{
 			goto  error ;
 		}
+
+		g_signal_connect( ibus_bus , "connected" ,
+				G_CALLBACK( connected) , NULL) ;
+		g_signal_connect( ibus_bus , "disconnected" ,
+				G_CALLBACK( disconnected) , NULL) ;
 	}
 
 	if( ! ( ibus = calloc( 1 , sizeof( im_ibus_t))))
@@ -1049,34 +1193,6 @@ im_ibus_new(
 		goto  error ;
 	}
 
-	ibus->context = ibus_bus_create_input_context( ibus_bus , "mlterm") ;
-	ibus_input_context_set_capabilities( ibus->context ,
-	#ifdef  USE_FRAMEBUFFER
-		IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_LOOKUP_TABLE
-	#else
-		IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS | IBUS_CAP_SURROUNDING_TEXT
-	#endif
-		) ;
-
-	g_signal_connect( ibus->context , "update-preedit-text" ,
-			G_CALLBACK( update_preedit_text) , ibus) ;
-	g_signal_connect( ibus->context , "hide-preedit-text" ,
-			G_CALLBACK( hide_preedit_text) , ibus) ;
-	g_signal_connect( ibus->context , "commit-text" , G_CALLBACK( commit_text) , ibus) ;
-	g_signal_connect( ibus->context , "forward-key-event" ,
-			G_CALLBACK( forward_key_event) , ibus) ;
-#ifdef  USE_FRAMEBUFFER
-	g_signal_connect( ibus->context , "update-lookup-table" ,
-			G_CALLBACK( update_lookup_table) , ibus) ;
-	g_signal_connect( ibus->context , "show-lookup-table" ,
-			G_CALLBACK( show_lookup_table) , ibus) ;
-	g_signal_connect( ibus->context , "hide-lookup-table" ,
-			G_CALLBACK( hide_lookup_table) , ibus) ;
-#endif
-
-	ibus->term_encoding = term_encoding ;
-	ibus->is_enabled = FALSE ;
-
 	if( term_encoding != ML_UTF8)
 	{
 		if( ! ( ibus->conv = (*syms->ml_conv_new)( term_encoding)))
@@ -1092,6 +1208,15 @@ im_ibus_new(
 	}
 #endif
 
+	ibus->term_encoding = term_encoding ;
+
+	if( ! ( ibus->context = context_new( ibus , engine)))
+	{
+		goto  error ;
+	}
+
+	ibus->is_enabled = FALSE ;
+
 	/*
 	 * set methods of x_im_t
 	 */
@@ -1102,18 +1227,7 @@ im_ibus_new(
 	ibus->im.focused = focused ;
 	ibus->im.unfocused = unfocused ;
 
-	if( engine)
-	{
-		set_engine( ibus->context , engine) ;
-	}
-#if  defined(USE_FRAMEBUFFER) && IBUS_CHECK_VERSION(1,5,0)
-	else
-	{
-		next_engine( ibus) ;
-	}
-#endif
-
-	kik_list_insert_head( im_ibus_t , ibus_list , ibus) ;
+	kik_slist_insert_head( ibus_list , ibus) ;
 
 	ref_count ++;
 
@@ -1126,6 +1240,8 @@ im_ibus_new(
 error:
 	if( ref_count == 0)
 	{
+		remove_event_source( 1) ;
+
 		ibus_object_destroy( (IBusObject*)ibus_bus) ;
 		ibus_bus = NULL ;
 
