@@ -35,9 +35,6 @@
 #define CONFIG_PATH "/etc"
 #endif
 
-#define  MOUSE_POS_LIMIT  (0xff - 0x20)
-#define  EXT_MOUSE_POS_LIMIT  (0x7ff - 0x20)
-
 /* Same as those defined in SWT.java */
 #define  AltMask (1 << 16)
 #define  ShiftMask  (1 << 17)
@@ -68,7 +65,8 @@ typedef struct  native_obj
 	ml_screen_event_listener_t  screen_listener ;
 	ml_xterm_event_listener_t  xterm_listener ;
 
-	char  prev_mouse_report_seq[5] ;
+	u_int16_t  prev_mouse_report_col ;
+	u_int16_t  prev_mouse_report_row ;
 
 } native_obj_t ;
 
@@ -102,6 +100,15 @@ pty_closed(
 #endif
 }
 
+static void
+show_config(
+	void *  p ,
+	char *  msg
+	)
+{
+	ml_term_show_message( ((native_obj_t *)p)->term , msg) ;
+}
+
 
 #ifdef  USE_LIBSSH2
 static JNIEnv *  env_for_dialog ;
@@ -337,13 +344,13 @@ resize(
 
 static void
 set_mouse_report(
-	void *  p ,
-	ml_mouse_report_mode_t  mode
+	void *  p
 	)
 {
-	if( ! mode)
+	if( ! ml_term_get_mouse_report_mode( ((native_obj_t*)p)->term))
 	{
-		memset( ((native_obj_t*)p)->prev_mouse_report_seq , 0 , 5) ;
+		((native_obj_t*)p)->prev_mouse_report_col =
+		((native_obj_t*)p)->prev_mouse_report_row = 0 ;
 	}
 }
 
@@ -603,7 +610,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 	{
 		kik_conf_t *  conf ;
 
-		kik_init_prog( "mlterm" , "3.3.8") ;
+		kik_init_prog( "mlterm" , "3.5.0") ;
 		kik_set_sys_conf_dir( CONFIG_PATH) ;
 		kik_locale_init( "") ;
 		kik_sig_child_init() ;
@@ -895,13 +902,12 @@ Java_mlterm_MLTermPty_nativeOpen(
 		(*env)->ReleaseStringUTFChars( env , jstr_encoding , p) ;
 	}
 
-	if( ! ( nativeObj->term = ml_create_term( cols , rows , tab_size , 0 , encoding ,
+	if( ! ( nativeObj->term = ml_create_term( "xterm" , cols , rows , tab_size , 0 , encoding ,
 					is_auto_encoding , use_auto_detect , logging_vt_seq ,
 					unicode_policy , col_size_a ,
 					use_char_combining , use_multi_col_char ,
-					0 /* use_bidi */ , 0 /* bidi_mode */ ,
+					0 /* use_ctl */ , 0 /* bidi_mode */ ,
 					NULL /* bidi_separators */ ,
-					0 /* use_ind */ , 1 /* use_bce */ ,
 					0 /* use_dynamic_comb */ , BSM_STATIC ,
 					0 /* vertical_mode */ , use_local_echo , NULL , NULL , 0)))
 	{
@@ -910,6 +916,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 
 	nativeObj->pty_listener.self = nativeObj ;
 	nativeObj->pty_listener.closed = pty_closed ;
+	nativeObj->pty_listener.show_config = show_config ;
 
 	nativeObj->config_listener.self = nativeObj ;
 	nativeObj->config_listener.set = set_config ;
@@ -1013,7 +1020,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 #endif
 
 	ret = ml_term_open_pty( nativeObj->term , cmd_path , argv , envv ,
-			host , NULL , pass , public_key , private_key) ;
+			host , NULL , pass , public_key , private_key , 0 , 0) ;
 
 #ifdef  USE_LIBSSH2
 	env_for_dialog = NULL ;
@@ -1359,6 +1366,177 @@ Java_mlterm_MLTermPty_nativeWrite(
 	return  JNI_TRUE ;
 }
 
+JNIEXPORT jboolean JNICALL
+Java_mlterm_MLTermPty_nativeWriteModifiedKey(
+	JNIEnv *  env ,
+	jobject  obj ,
+	jlong  nativeObj ,
+	jint  key ,
+	jint  modcode
+	)
+{
+	if( ml_term_write_modified_key( ((native_obj_t*)nativeObj)->term , key , modcode))
+	{
+		return  JNI_TRUE ;
+	}
+	else
+	{
+		return  JNI_FALSE ;
+	}
+}
+
+JNIEXPORT jboolean JNICALL
+Java_mlterm_MLTermPty_nativeWriteSpecialKey(
+	JNIEnv *  env ,
+	jobject  obj ,
+	jlong  nativeObj ,
+	jint  key ,
+	jint  modcode
+	)
+{
+	ml_special_key_t  spkey ;
+	u_int8_t  keys[] =
+	{
+		SPKEY_UP ,
+		SPKEY_DOWN ,
+		SPKEY_LEFT ,
+		SPKEY_RIGHT ,
+		SPKEY_PRIOR ,
+		SPKEY_NEXT ,
+		SPKEY_HOME ,
+		SPKEY_END ,
+		SPKEY_INSERT ,
+		SPKEY_F1 ,
+		SPKEY_F2 ,
+		SPKEY_F3 ,
+		SPKEY_F4 ,
+		SPKEY_F5 ,
+		SPKEY_F6 ,
+		SPKEY_F7 ,
+		SPKEY_F8 ,
+		SPKEY_F9 ,
+		SPKEY_F10 ,
+		SPKEY_F11 ,
+		SPKEY_F12 ,
+		SPKEY_F13 ,
+		SPKEY_F14 ,
+		SPKEY_F15 ,
+		SPKEY_F16 ,
+		SPKEY_F17 ,
+		SPKEY_F18 ,
+		SPKEY_F19 ,
+		SPKEY_F20 ,
+	} ;
+
+	u_int8_t  keypad_keys[] =
+	{
+		SPKEY_KP_MULTIPLY ,
+		SPKEY_KP_ADD ,
+		SPKEY_KP_MULTIPLY ,
+		SPKEY_KP_ADD ,
+		SPKEY_KP_SUBTRACT ,
+		SPKEY_KP_DELETE ,
+		SPKEY_KP_DIVIDE ,
+		SPKEY_KP_INSERT ,	/* 0 */
+		SPKEY_KP_END ,		/* 1 */
+		SPKEY_KP_DOWN ,		/* 2 */
+		SPKEY_KP_NEXT ,		/* 3 */
+		SPKEY_KP_LEFT ,		/* 4 */
+		SPKEY_KP_BEGIN ,	/* 5 */
+		SPKEY_KP_RIGHT ,	/* 6 */
+		SPKEY_KP_HOME ,		/* 7 */
+		SPKEY_KP_UP ,		/* 8 */
+		SPKEY_KP_PRIOR ,	/* 9 */
+	} ;
+
+	/* Definitions in SWT.java */
+	enum
+	{
+		KEYCODE_BIT = (1 << 24) ,
+		ARROW_UP = KEYCODE_BIT + 1 ,
+		ARROW_DOWN = KEYCODE_BIT + 2 ,
+		ARROW_LEFT = KEYCODE_BIT + 3 ,
+		ARROW_RIGHT = KEYCODE_BIT + 4 ,
+		PAGE_UP = KEYCODE_BIT + 5 ,
+		PAGE_DOWN = KEYCODE_BIT + 6 ,
+		HOME = KEYCODE_BIT + 7 ,
+		END = KEYCODE_BIT + 8 ,
+		INSERT = KEYCODE_BIT + 9 ,
+		F1 = KEYCODE_BIT + 10 ,
+		F2 = KEYCODE_BIT + 11 ,
+		F3 = KEYCODE_BIT + 12 ,
+		F4 = KEYCODE_BIT + 13 ,
+		F5 = KEYCODE_BIT + 14 ,
+		F6 = KEYCODE_BIT + 15 ,
+		F7 = KEYCODE_BIT + 16 ,
+		F8 = KEYCODE_BIT + 17 ,
+		F9 = KEYCODE_BIT + 18 ,
+		F10 = KEYCODE_BIT + 19 ,
+		F11 = KEYCODE_BIT + 20 ,
+		F12 = KEYCODE_BIT + 21 ,
+		F13 = KEYCODE_BIT + 22 ,
+		F14 = KEYCODE_BIT + 23 ,
+		F15 = KEYCODE_BIT + 24 ,
+		F16 = KEYCODE_BIT + 25 ,
+		F17 = KEYCODE_BIT + 26 ,
+		F18 = KEYCODE_BIT + 27 ,
+		F19 = KEYCODE_BIT + 28 ,
+		F20 = KEYCODE_BIT + 29 ,
+
+		KEYPAD_MULTIPLY = KEYCODE_BIT + 42 ,
+		KEYPAD_ADD = KEYCODE_BIT + 43 ,
+		KEYPAD_SUBTRACT = KEYCODE_BIT + 45 ,
+		KEYPAD_DECIMAL = KEYCODE_BIT + 46 ,
+		KEYPAD_DIVIDE = KEYCODE_BIT + 47 ,
+		KEYPAD_0 = KEYCODE_BIT + 48 ,
+		KEYPAD_1 = KEYCODE_BIT + 49 ,
+		KEYPAD_2 = KEYCODE_BIT + 50 ,
+		KEYPAD_3 = KEYCODE_BIT + 51 ,
+		KEYPAD_4 = KEYCODE_BIT + 52 ,
+		KEYPAD_5 = KEYCODE_BIT + 53 ,
+		KEYPAD_6 = KEYCODE_BIT + 54 ,
+		KEYPAD_7 = KEYCODE_BIT + 55 ,
+		KEYPAD_8 = KEYCODE_BIT + 56 ,
+		KEYPAD_9 = KEYCODE_BIT + 57 ,
+		KEYPAD_EQUAL = KEYCODE_BIT + 61 ,
+		KEYPAD_CR = KEYCODE_BIT + 80 ,
+		HELP = KEYCODE_BIT + 81 ,
+		CAPS_LOCK = KEYCODE_BIT + 82 ,
+		NUM_LOCK = KEYCODE_BIT + 83 ,
+		SCROLL_LOCK = KEYCODE_BIT + 84 ,
+		PAUSE = KEYCODE_BIT + 85 ,
+		BREAK = KEYCODE_BIT + 86 ,
+		PRINT_SCREEN = KEYCODE_BIT + 87 ,
+	} ;
+
+	if( ARROW_UP <= key && key <= F20)
+	{
+		spkey = keys[key - ARROW_UP] ;
+	}
+	else if( KEYPAD_MULTIPLY <= key && key <= KEYPAD_9)
+	{
+		spkey = keypad_keys[key - KEYPAD_MULTIPLY] ;
+	}
+	else if( key == '\x1b')
+	{
+		spkey = SPKEY_ESCAPE ;
+	}
+	else
+	{
+		return  JNI_FALSE ;
+	}
+
+	if( ml_term_write_special_key( ((native_obj_t*)nativeObj)->term ,
+			spkey , modcode , 0))
+	{
+		return  JNI_TRUE ;
+	}
+	else
+	{
+		return  JNI_FALSE ;
+	}
+}
+
 JNIEXPORT jboolean JNICALL
 Java_mlterm_MLTermPty_nativeResize(
 	JNIEnv *  env ,
@@ -1369,7 +1547,7 @@ Java_mlterm_MLTermPty_nativeResize(
 	)
 {
 	if( nativeObj && ((native_obj_t*)nativeObj)->term &&
-	    ml_term_resize( ((native_obj_t*)nativeObj)->term , cols , rows))
+	    ml_term_resize( ((native_obj_t*)nativeObj)->term , cols , rows , 0 , 0))
 	{
 		return  JNI_TRUE ;
 	}
@@ -1724,23 +1902,6 @@ Java_mlterm_MLTermPty_nativeGetCaretCol(
 	}
 }
 
-JNIEXPORT jboolean JNICALL
-Java_mlterm_MLTermPty_nativeIsAppCursorKeys(
-	JNIEnv *  env ,
-	jobject  obj ,
-	jlong  nativeObj
-	)
-{
-	if( ml_term_is_app_cursor_keys( ((native_obj_t*)nativeObj)->term))
-	{
-		return  JNI_TRUE ;
-	}
-	else
-	{
-		return  JNI_FALSE ;
-	}
-}
-
 JNIEXPORT jboolean JNICALL
 Java_mlterm_MLTermPty_nativeIsTrackingMouse(
 	JNIEnv *  env ,
@@ -1786,12 +1947,6 @@ Java_mlterm_MLTermPty_nativeReportMouseTracking(
 	ml_line_t *  line ;
 	int  col ;
 	int  key_state ;
-	/*
-	 * Max length is SGR style => ESC [ < %d ; %d(col) ; %d(row) ; %c('M' or 'm') NULL
-	 *                            1   1 1 3  1 3       1  3      1 1              1
-	 */
-	u_char  seq[17] ;
-	size_t  seq_len ;
 
 	nativeObj = nobj ;
 
@@ -1803,6 +1958,13 @@ Java_mlterm_MLTermPty_nativeReportMouseTracking(
 	}
 #endif
 
+	if( ml_term_get_mouse_report_mode( nativeObj->term) >= LOCATOR_CHARCELL_REPORT)
+	{
+		/* Not supported */
+
+		return ;
+	}
+
 	/*
 	 * XXX
 	 * Not considering BiDi etc.
@@ -1843,13 +2005,10 @@ Java_mlterm_MLTermPty_nativeReportMouseTracking(
 	 * Following is the same as x_screen.c:report_mouse_tracking().
 	 */
 
-	if( ( isReleased && ml_term_get_extended_mouse_report_mode( nativeObj->term) !=
-				EXTENDED_MOUSE_REPORT_SGR) ||
-	    ( isMotion && button == 0) )
+	if( /* isMotion && */ button == 0)
 	{
-		/* ButtonRelease or PointerMotion */
+		/* PointerMotion */
 		key_state = 0 ;
-		button = 3 ;
 	}
 	else
 	{
@@ -1866,120 +2025,26 @@ Java_mlterm_MLTermPty_nativeReportMouseTracking(
 				((state & AltMask) ? 8 : 0) +
 				((state & ControlMask) ? 16 : 0) +
 				(isMotion ? 32 : 0) ;
-
-		if( isReleased)
-		{
-			/* is EXTENDED_MOUSE_REPORT_SGR */
-			key_state += 0x80 ;
-		}
-
-		/* if( button > 0) */
-		{
-			/* ButtonPress */
-			button -- ;	/* Button1 = 1 -> 0 */
-
-			while( button >= 3)
-			{
-				/* Wheel mouse */
-				key_state += 64 ;
-				button -= 3 ;
-			}
-		}
 	}
 
 	/* count starts from 1, not 0 */
 	col ++ ;
 	row ++ ;
 
-	/* clear all bytes of seq to compare with prev_mouse_report_seq. */
-	memcpy( seq , "\x1b[M\0\0\0\0\0" , 8) ;
-
-	seq[3] = 0x20 + button + key_state ;
-
-	if( ml_term_get_extended_mouse_report_mode( nativeObj->term))
+	if( isMotion &&
+	    button <= 3 &&	/* not wheel mouse */
+	    nativeObj->prev_mouse_report_col == col &&
+	    nativeObj->prev_mouse_report_row == row)
 	{
-		int  ch ;
-		u_char *  p ;
-
-		p = seq + 4 ;
-
-		if( col > EXT_MOUSE_POS_LIMIT)
-		{
-			col = EXT_MOUSE_POS_LIMIT ;
-		}
-
-		if( (ch = 0x20 + col) >= 0x80)
-		{
-			*(p ++) = ((ch >> 6) & 0x1f) | 0xc0 ;
-			*(p ++) = (ch & 0x3f) | 0x80 ;
-		}
-		else
-		{
-			*(p ++) = ch ;
-		}
-
-		if( row > EXT_MOUSE_POS_LIMIT)
-		{
-			row = EXT_MOUSE_POS_LIMIT ;
-		}
-
-		if( (ch = 0x20 + row) >= 0x80)
-		{
-			*(p ++) = ((ch >> 6) & 0x1f) | 0xc0 ;
-			*p = (ch & 0x3f) | 0x80 ;
-		}
-		else
-		{
-			*p = ch ;
-		}
-
-		seq_len = p - seq + 1 ;
-	}
-	else
-	{
-		seq[4] = 0x20 + (col < MOUSE_POS_LIMIT ? col : MOUSE_POS_LIMIT) ;
-		seq[5] = 0x20 + (row < MOUSE_POS_LIMIT ? row : MOUSE_POS_LIMIT) ;
-		seq_len = 6 ;
+		/* Pointer is not moved. */
+		return ;
 	}
 
-	if( key_state >= 64 ||						/* Wheeling mouse */
-	    memcmp( nativeObj->prev_mouse_report_seq , seq + 3 , 5) != 0) /* Position is changed */
-	{
-		memcpy( nativeObj->prev_mouse_report_seq , seq + 3 , 5) ;
+	ml_term_report_mouse_tracking( nativeObj->term , col , row , button ,
+		isReleased , key_state , 0) ;
 
-		if( ml_term_get_extended_mouse_report_mode( nativeObj->term) >
-			EXTENDED_MOUSE_REPORT_UTF8)
-		{
-			if( ml_term_get_extended_mouse_report_mode( nativeObj->term) ==
-				EXTENDED_MOUSE_REPORT_SGR)
-			{
-				sprintf( seq , "\x1b[<%d;%d;%d%c" ,
-					(button + key_state) & 0x7f , col , row ,
-					((button + key_state) & 0x80) ? 'm' : 'M') ;
-			}
-			else /* if( ml_term_get_extended_mouse_report_mode( nativeObj->term) ==
-					EXTENDED_MOUSE_REPORT_URXVT) */
-			{
-				sprintf( seq , "\x1b[%d;%d;%dM" ,
-					0x20 + button + key_state , col , row) ;
-			}
-
-			seq_len = strlen( seq) ;
-		}
-
-		ml_term_write( nativeObj->term , seq , seq_len) ;
-
-	#ifdef  __DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " [reported cursor pos] %d %d\n" , col , row) ;
-	#endif
-	}
-#ifdef  __DEBUG
-	else
-	{
-		kik_debug_printf( KIK_DEBUG_TAG
-			" cursor pos %d %d is not changed and not reported.\n") ;
-	}
-#endif
+	nativeObj->prev_mouse_report_col = col ;
+	nativeObj->prev_mouse_report_row = row ;
 }
 
 JNIEXPORT jlong JNICALL
