@@ -62,7 +62,8 @@ int  kik_conf_add_opt( kik_conf_t *  conf , char  short_opt , char *  long_opt ,
 
 int  kik_conf_set_end_opt( kik_conf_t *  conf , char  opt , char *  long_opt , char *  key , char *  help) ;
 
-int  kik_conf_parse_args( kik_conf_t *  conf , int *  argc , char ***  argv) ;
+int  kik_conf_parse_args( kik_conf_t *  conf , int *  argc , char ***  argv ,
+	int  ignore_unknown_opt) ;
 
 int  kik_conf_write( kik_conf_t *  conf , char *  filename) ;
 
