@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_dlfcn_none.c,v 1.2 2004/10/06 05:26:14 seiichi Exp $
+ *	$Id$
  */
 
 #include  "kik_dlfcn.h"
@@ -18,8 +18,8 @@
  
 kik_dl_handle_t
 kik_dl_open(
-	char *  dirpath ,
-	char *  name
+	const char *  dirpath ,
+	const char *  name
 	)
 {
 	return  NULL ;
@@ -36,7 +36,7 @@ kik_dl_close(
 void *
 kik_dl_func_symbol(
 	kik_dl_handle_t  handle ,
-	char *  symbol
+	const char *  symbol
 	)
 {
 	return  NULL ;
@@ -44,7 +44,7 @@ kik_dl_func_symbol(
 
 int
 kik_dl_is_module(
-	char * name
+	const char *  name
 	)
 {
 	return  0 ;
