@@ -6,14 +6,20 @@
 #define  __KIK_NET_H__
 
 
+#ifdef  USE_WIN32API
+
+#undef  _WIN32_WINNT
+#define  _WIN32_WINNT  0x0501	/* for getaddrinfo */
+#include  <windows.h>
+#include  <ws2tcpip.h>		/* addrinfo */
+
+#else	/* USE_WIN32API */
+
 #include  "kik_types.h"		/* socklen_t */
-
-#ifndef  USE_WIN32API
-
-#include  <sys/types.h>
-#include  <sys/socket.h>	
+#include  <sys/socket.h>
 #include  <sys/un.h>
-
+#include  <netdb.h>
+#include  <netinet/in.h>
 
 #ifndef  AF_LOCAL
 #define  AF_LOCAL  AF_UNIX
