@@ -17,8 +17,8 @@ kik_pty_fork(
 }
 
 int
-kik_pty_helper_close(
-	int  pty
+kik_pty_close(
+	int  master
 	)
 {
 	return  0 ;
