@@ -48,10 +48,10 @@ struct  kik_utmp
 
 kik_utmp_t
 kik_utmp_new(
-	     char * tty,
-	     char * host,
-	     int    pty_fd
-	    )
+	const char * tty,
+	const char * host,
+	int  pty_fd
+	)
 {
 #ifdef UTMPX
   struct utmpx	ut;
@@ -201,9 +201,9 @@ kik_utmp_delete(
   memset( &ut, 0, sizeof(ut) );
 
   memcpy( ut.ut_id, utmp->ut_pos,
-	  K_MIN(sizeof(ut.ut_id), strlen(utmp->ut_pos)) );
+	  K_MIN(sizeof(ut.ut_id), sizeof(utmp->ut_pos)) );
   memcpy( ut.ut_line, utmp->ut_line,
-	  K_MIN(sizeof(ut.ut_line), strlen(utmp->ut_line)) );
+	  K_MIN(sizeof(ut.ut_line), sizeof(utmp->ut_line)) );
   memset( ut.ut_user, 0, sizeof(ut.ut_user));
 
   ut.ut_pid	= getpid();
