@@ -7,7 +7,7 @@
 
 struct tabl *
 libind_get_table(
-	u_int *  table_size
+	unsigned int *  table_size
 	)
 {
 	*table_size = sizeof( iscii_hindi_table) / sizeof( struct tabl) ;
