@@ -7,7 +7,7 @@
 
 struct a2i_tabl *
 libind_get_table(
-	u_int *  table_size
+	unsigned int *  table_size
 	)
 {
 	*table_size = sizeof( isciikey_iitkeyb_table) / sizeof( struct a2i_tabl) ;
