@@ -107,7 +107,7 @@ convert_to_intern(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -125,7 +125,7 @@ convert_to_intern(
 					&is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
