@@ -83,7 +83,7 @@ convert_to_euccn_intern(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -96,7 +96,7 @@ convert_to_euccn_intern(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -110,7 +110,7 @@ convert_to_euccn_intern(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -126,7 +126,7 @@ convert_to_euccn_intern(
 			
 			if( filled_size + 3 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -151,7 +151,7 @@ convert_to_euccn_intern(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
