@@ -148,7 +148,7 @@ convert_to_eucjp(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -161,7 +161,7 @@ convert_to_eucjp(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 				
 				return  filled_size ;
 			}
@@ -175,7 +175,7 @@ convert_to_eucjp(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -189,7 +189,7 @@ convert_to_eucjp(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -208,7 +208,7 @@ convert_to_eucjp(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
