@@ -80,7 +80,7 @@ convert_to_euckr_intern(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -93,7 +93,7 @@ convert_to_euckr_intern(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -107,7 +107,7 @@ convert_to_euckr_intern(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -125,7 +125,7 @@ convert_to_euckr_intern(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
