@@ -53,7 +53,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -67,7 +67,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -80,7 +80,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -93,7 +93,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -106,7 +106,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -119,7 +119,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -132,7 +132,7 @@ convert_to_euctw(
 		{
 			if( filled_size + 2 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -145,7 +145,7 @@ convert_to_euctw(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 			
 				return  filled_size ;
 			}
@@ -162,7 +162,7 @@ convert_to_euctw(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
