@@ -62,7 +62,7 @@ convert_to_iso2022cn(
 		{
 			if( filled_size + ch.size > dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -83,7 +83,7 @@ convert_to_iso2022cn(
 			{
 				if( filled_size + ch.size + 6 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
@@ -103,7 +103,7 @@ convert_to_iso2022cn(
 			{
 				if( filled_size + ch.size + 2 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
@@ -120,7 +120,7 @@ convert_to_iso2022cn(
 			{
 				if( filled_size + ch.size + 5 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
@@ -139,7 +139,7 @@ convert_to_iso2022cn(
 			{
 				if( filled_size + ch.size + 1 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
@@ -155,7 +155,7 @@ convert_to_iso2022cn(
 		{
 			if( filled_size + ch.size + 1 > dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -182,7 +182,7 @@ convert_to_iso2022cn(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
