@@ -12,7 +12,6 @@
 #include  "mkf_iso2022_intern.h"
 #include  "mkf_viet_map.h"
 #include  "mkf_ru_map.h"
-#include  "mkf_ucs4_usascii.h"
 #include  "mkf_ucs4_iso8859.h"
 #include  "mkf_ucs4_map.h"
 
@@ -29,8 +28,7 @@ remap_unsupported_charset(
 
 	if( ch->cs == ISO10646_UCS4_1)
 	{
-		if( mkf_map_ucs4_to_us_ascii( &c , mkf_char_to_int( ch)) ||
-			mkf_map_ucs4_to_cs( &c , ch , gr_cs))
+		if( mkf_map_ucs4_to_cs( &c , ch , gr_cs))
 		{
 			*ch = c ;
 
@@ -64,7 +62,7 @@ convert_to_iso8859(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -76,7 +74,7 @@ convert_to_iso8859(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -92,7 +90,7 @@ convert_to_iso8859(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
