@@ -87,7 +87,7 @@ convert_to_johab(
 			
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -110,7 +110,7 @@ convert_to_johab(
 			
 			if( filled_size + 1 >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -203,7 +203,7 @@ convert_to_johab(
 		{
 			if( filled_size >= dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -220,7 +220,7 @@ convert_to_johab(
 			size = (*conv->illegal_char)( conv , dst , dst_size - filled_size , &is_full , &ch) ;
 			if( is_full)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
