@@ -80,6 +80,19 @@ __mkf_parser_reset(
 	parser->left = parser->marked_left ;
 }
 
+void
+__mkf_parser_full_reset(
+	mkf_parser_t *  parser
+	)
+{
+	if( parser->is_eos && parser->marked_left > parser->left)
+	{
+		parser->is_eos = 0 ;
+	}
+
+	__mkf_parser_reset( parser) ;
+}
+
 /*
  * short cut function. (ignoring error)
  */
