@@ -36,6 +36,8 @@ typedef struct  mkf_parser
 
 #define  mkf_parser_reset( parser)  __mkf_parser_reset( ( mkf_parser_t*) parser)
 
+#define  mkf_parser_full_reset( parser)  __mkf_parser_full_reset( ( mkf_parser_t*) parser)
+
 #define  mkf_parser_mark( parser)  __mkf_parser_mark( ( mkf_parser_t*) parser)
 
 
@@ -49,6 +51,8 @@ void  __mkf_parser_mark( mkf_parser_t *  parser) ;
 
 void  __mkf_parser_reset( mkf_parser_t *  parser) ;
 
+void  __mkf_parser_full_reset( mkf_parser_t *  parser) ;
+
 int  mkf_parser_next_char( mkf_parser_t *  parser , mkf_char_t *  ch) ;
 
 
