@@ -11,7 +11,7 @@ typedef enum mkf_property
 	MKF_COMBINING = 0x1u ,
 
 	/* only for UCS */
-	MKF_BIWIDTH = 0x2u ,
+	MKF_FULLWIDTH = 0x2u ,
 	MKF_AWIDTH = 0x4u
 
 } mkf_property_t ;
