@@ -15,8 +15,8 @@
 
 #else
 
-mkf_map_func( "mkf_zh", mkf_map_gbk_to_ucs4, 16)
+mkf_map_func( zh, mkf_map_gbk_to_ucs4, 16)
 
-mkf_map_func( "mkf_zh", mkf_map_ucs4_to_gbk, 32)
+mkf_map_func( zh, mkf_map_ucs4_to_gbk, 32)
 
 #endif
