@@ -62,12 +62,13 @@ convert_to_utf16(
 		{
 			return  filled_size ;
 		}
-		
+
+	#if  0
 		if( ch.cs == ISO10646_UCS2_1)
 		{
 			if( filled_size + 2 > dst_size)
 			{
-				mkf_parser_reset( parser) ;
+				mkf_parser_full_reset( parser) ;
 
 				return  filled_size ;
 			}
@@ -78,6 +79,22 @@ convert_to_utf16(
 			filled_size += 2 ;
 		}
 		else
+	#endif
+		if( ch.cs == US_ASCII)
+		{
+			if( filled_size + 2 > dst_size)
+			{
+				mkf_parser_full_reset( parser) ;
+
+				return  filled_size ;
+			}
+
+			(*dst++) = '\0' ;
+			(*dst++) = ch.ch[0] ;
+
+			filled_size += 2 ;
+		}
+		else
 		{
 			if( ch.cs != ISO10646_UCS4_1)
 			{
@@ -100,7 +117,7 @@ convert_to_utf16(
 							&is_full , &ch) ;
 					if( is_full)
 					{
-						mkf_parser_reset( parser) ;
+						mkf_parser_full_reset( parser) ;
 
 						return  filled_size ;
 					}
@@ -115,7 +132,7 @@ convert_to_utf16(
 				
 				if( filled_size + 2 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
@@ -134,7 +151,7 @@ convert_to_utf16(
 				
 				if( filled_size + 4 > dst_size)
 				{
-					mkf_parser_reset( parser) ;
+					mkf_parser_full_reset( parser) ;
 
 					return  filled_size ;
 				}
