@@ -102,7 +102,8 @@ xct_non_iso2022_is_started(
 	{
 		if( cs_len == 9 && strncmp( (char *)cs_str , "iscii-dev" , cs_len) == 0)
 		{
-			xct_parser->cs = ISCII ;
+			/* XXX */
+			xct_parser->cs = ISCII_HINDI ;
 		}
 		else if( cs_len == 6 && strncmp( (char *)cs_str , "koi8-r" , cs_len) == 0)
 		{
