@@ -5,7 +5,6 @@
 #include  "mkf_zh_hk_map.h"
 
 #include  "mkf_ucs4_map.h"
-#include  "mkf_ucs4_usascii.h"
 #include  "mkf_ucs4_big5.h"
 
 
@@ -13,7 +12,6 @@
 
 static mkf_map_ucs4_to_func_t  map_ucs4_to_funcs[] =
 {
-	mkf_map_ucs4_to_us_ascii ,
 	mkf_map_ucs4_to_hkscs ,
 	mkf_map_ucs4_to_big5 ,
 } ;
