@@ -11,6 +11,9 @@ void  ml_line_iscii_convert_logical_char_index_to_visual(void) ;
 void  ml_iscii_copy(void) ;
 void  ml_iscii_reset(void) ;
 void  ml_line_iscii_need_shape(void) ;
+void  ml_line_iscii_render(void) ;
+void  ml_line_iscii_visual(void) ;
+void  ml_line_iscii_logical(void) ;
 
 
 /* --- global variables --- */
@@ -23,10 +26,12 @@ void *  ml_ctl_iscii_func_table[MAX_CTL_ISCII_FUNCS] =
 	ml_convert_ascii_to_iscii ,
 	ml_line_set_use_iscii ,
 	ml_line_iscii_convert_logical_char_index_to_visual ,
-	ml_logvis_iscii_new ,
-	ml_iscii_shape_new ,
+	ml_shape_iscii ,
 	ml_iscii_copy ,
 	ml_iscii_reset ,
 	ml_line_iscii_need_shape ,
+	ml_line_iscii_render ,
+	ml_line_iscii_visual ,
+	ml_line_iscii_logical ,
 
 } ;
