@@ -35,6 +35,86 @@ struct  ml_isciikey_state
 } ;
 
 
+#ifdef  STATIC_LINK_INDIC_TABLES
+
+/* for Android */
+
+#include  <table/bengali.table>
+#include  <table/hindi.table>
+#if  0
+#include  <table/assamese.table>
+#include  <table/gujarati.table>
+#include  <table/kannada.table>
+#include  <table/malayalam.table>
+#include  <table/oriya.table>
+#include  <table/punjabi.table>
+#include  <table/roman.table>
+#include  <table/tamil.table>
+#include  <table/telugu.table>
+#endif
+
+
+/* --- static variables --- */
+
+static struct
+{
+	struct tabl *  tabl ;
+	size_t  size ;
+
+} tables[] =
+{
+#if  0
+	{ iscii_assamese_table , sizeof( iscii_assamese_table) / sizeof( struct tabl) , } ,
+	{ iscii_bengali_table , sizeof( iscii_bengali_table) / sizeof( struct tabl) , } ,
+	{ iscii_gujarati_table , sizeof( iscii_gujarati_table) / sizeof( struct tabl) , } ,
+	{ iscii_hindi_table , sizeof( iscii_hindi_table) / sizeof( struct tabl) , } ,
+	{ iscii_kannada_table , sizeof( iscii_kannada_table) / sizeof( struct tabl) , } ,
+	{ iscii_malayalam_table , sizeof( iscii_malayalam_table) / sizeof( struct tabl) , } ,
+	{ iscii_oriya_table , sizeof( iscii_oriya_table) / sizeof( struct tabl) , } ,
+	{ iscii_punjabi_table , sizeof( iscii_punjabi_table) / sizeof( struct tabl) , } ,
+	{ iscii_roman_table , sizeof( iscii_roman_table) / sizeof( struct tabl) , } ,
+	{ iscii_tamil_table , sizeof( iscii_tamil_table) / sizeof( struct tabl) , } ,
+	{ iscii_telugu_table , sizeof( iscii_telugu_table) / sizeof( struct tabl) , } ,
+#else
+	{ NULL , 0 , } ,
+	{ iscii_bengali_table , sizeof( iscii_bengali_table) / sizeof( struct tabl) , } ,
+	{ NULL , 0 , } ,
+	{ iscii_hindi_table , sizeof( iscii_hindi_table) / sizeof( struct tabl) , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+	{ NULL , 0 , } ,
+#endif
+} ;
+
+
+/* --- static functions --- */
+
+static struct tabl *
+get_iscii_table(
+	int  idx ,
+	size_t *  size
+	)
+{
+	*size = tables[idx].size ;
+
+	return  tables[idx].tabl ;
+}
+
+static struct a2i_tabl *
+get_isciikey_table(
+	int  is_inscript ,
+	size_t *  size
+	)
+{
+	return  NULL ;
+}
+
+#else	/* STATIC_LINK_INDIC_TABLES */
+
 /* --- static variables --- */
 
 static char *  iscii_table_files[] =
@@ -128,6 +208,8 @@ get_isciikey_table(
 	}
 }
 
+#endif	/* STATIC_LINK_INDIC_TABLES */
+
 
 /* --- global functions --- */
 
@@ -213,6 +295,7 @@ ml_iscii(
 	u_int  iscii_filled ;
 	mkf_charset_t  cs ;
 	mkf_charset_t  prev_cs ;
+	int  has_ucs ;
 
 	iscii_buf_len = src_len * 4 + 1 ;
 	if( ( iscii_buf = alloca( iscii_buf_len)) == NULL)
@@ -235,6 +318,7 @@ ml_iscii(
 	state->has_iscii = 0 ;
 	dst_pos = -1 ;
 	prev_cs = cs = UNKNOWN_CS ;
+	has_ucs = 0 ;
 	for( src_pos = 0 ; src_pos < src_len ; src_pos ++)
 	{
 		cs = ml_char_cs( src + src_pos) ;
@@ -296,13 +380,18 @@ ml_iscii(
 		}
 		else
 		{
+			if( cs == ISO10646_UCS4_1)
+			{
+				has_ucs = 1 ;
+			}
+
 			state->num_of_chars_array[++dst_pos] = 1 ;
 		}
 	}
 
 	state->size = dst_pos + 1 ;
 
-	return  1 ;
+	return  (! state->has_iscii && has_ucs) ? -1 : 1 ;
 }
 
 int
