@@ -24,6 +24,7 @@
 #include  <stdio.h>		/* sprintf */
 #ifdef  __CYGWIN__
 #include  <sys/wait.h>
+#include  <windows.h>	/* GetModuleHandle */
 #endif
 
 
@@ -62,7 +63,7 @@ typedef struct ssh_session
 
 	int  use_x11_forwarding ;
 
-	int  doing_scp ;
+	int  suspended ;
 
 	LIBSSH2_CHANNEL **  pty_channels ;
 	u_int  num_of_ptys ;
@@ -221,6 +222,111 @@ kbd_callback(
 	(void)abstract ;
 }
 
+#ifdef  OPEN_PTY_ASYNC
+
+#ifdef  USE_WIN32API
+
+static HANDLE *  openssl_locks ;
+
+static void
+openssl_lock_callback(
+	int  mode ,
+	int  type ,
+	const char *  file ,
+	int  line
+	)
+{
+	if( mode & 1 /* CRYPTO_LOCK */)
+	{
+		WaitForSingleObject( openssl_locks[type] , INFINITE) ;
+	}
+	else
+	{
+		ReleaseMutex( openssl_locks[type]) ;
+	}
+}
+
+#else
+
+static pthread_mutex_t *  openssl_locks ;
+
+static void
+openssl_lock_callback(
+	int  mode ,
+	int  type ,
+	const char *  file ,
+	int  line
+	)
+{
+	if( mode & 1 /* CRYPTO_LOCK */)
+	{
+		pthread_mutex_lock( &openssl_locks[type]) ;
+	}
+	else
+	{
+		pthread_mutex_unlock( &openssl_locks[type]) ;
+	}
+}
+
+#endif
+
+int  CRYPTO_num_locks(void) ;
+void  CRYPTO_set_locking_callback( void (*func)( int , int , const char * , int)) ;
+
+/* gcrypt is not supported. */
+static void
+set_use_multi_thread(
+	int  use
+	)
+{
+	static int  num_locks ;
+	int  count ;
+
+	if( use)
+	{
+		num_locks = CRYPTO_num_locks() ;
+
+		if( ( openssl_locks = malloc( num_locks * sizeof(*openssl_locks))))
+		{
+			for( count = 0 ; count < num_locks ; count++)
+			{
+			#ifdef  USE_WIN32API
+				openssl_locks[count] = CreateMutex( NULL , FALSE , NULL) ;
+			#else
+				openssl_locks[count] = PTHREAD_MUTEX_INITIALIZER ;
+			#endif
+			}
+
+			CRYPTO_set_locking_callback( openssl_lock_callback) ;
+		}
+		else
+		{
+			num_locks = 0 ;
+		}
+	}
+	else
+	{
+		if( openssl_locks)
+		{
+			CRYPTO_set_locking_callback( NULL) ;
+
+		#ifdef  USE_WIN32API
+			for( count = 0 ; count < num_locks ; count++)
+			{
+				CloseHandle( openssl_locks[count]) ;
+			}
+		#endif
+
+			free( openssl_locks) ;
+			openssl_locks = NULL ;
+		}
+	}
+}
+
+#else
+#define  set_use_multi_thread(use)  (0)
+#endif
+
 #ifdef  AI_PASSIVE
 #define  HAVE_GETADDRINFO
 #endif
@@ -270,6 +376,8 @@ ssh_connect(
 		return  NULL ;
 	}
 
+	set_use_multi_thread( 1) ;
+
 	if( num_of_sessions == 0 && libssh2_init( 0) != 0)
 	{
 	#ifdef  DEBUG
@@ -376,6 +484,7 @@ ssh_connect(
 #endif
 
 	libssh2_session_set_blocking( session->obj , 1) ;
+	libssh2_session_set_timeout( session->obj , 30000) ;	/* 30 sec */
 
 	if( cipher_list)
 	{
@@ -605,8 +714,7 @@ ssh_connect(
 
 	if( ! auth_success && strstr( userauthlist , "keyboard-interactive"))
 	{
-		free( pass_response) ;
-		pass_response = strdup( pass) ;
+		pass_response = pass ;
 
 		if( libssh2_userauth_keyboard_interactive(
 			session->obj , user , &kbd_callback) == 0)
@@ -659,6 +767,7 @@ ssh_connect(
 		sessions = p ;
 	}
 
+	libssh2_session_set_timeout( session->obj , 0) ;
 	libssh2_session_set_blocking( session->obj , 0) ;
 
 	session->host = strdup( host) ;
@@ -683,6 +792,7 @@ error2:
 	}
 
 error1:
+	set_use_multi_thread( 0) ;
 	free( session) ;
 
 	return  NULL ;
@@ -785,10 +895,18 @@ static int
 set_winsize(
 	ml_pty_t *  pty ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
-	libssh2_channel_request_pty_size( ((ml_pty_ssh_t*)pty)->channel , cols , rows) ;
+#ifdef  __DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " win size cols %d rows %d width %d height %d.\n" ,
+		cols , rows , width_pix , height_pix) ;
+#endif
+
+	libssh2_channel_request_pty_size_ex( ((ml_pty_ssh_t*)pty)->channel ,
+		cols , rows , width_pix , height_pix) ;
 
 	return  1 ;
 }
@@ -802,12 +920,10 @@ write_to_pty(
 {
 	ssize_t  ret ;
 
-#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
-	if( ((ml_pty_ssh_t*)pty)->session->doing_scp)
+	if( ((ml_pty_ssh_t*)pty)->session->suspended)
 	{
 		return  0 ;
 	}
-#endif
 
 	ret = libssh2_channel_write( ((ml_pty_ssh_t*)pty)->channel , buf , len) ;
 
@@ -851,12 +967,10 @@ read_pty(
 		return  len ;
 	}
 
-#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
-	if( ((ml_pty_ssh_t*)pty)->session->doing_scp)
+	if( ((ml_pty_ssh_t*)pty)->session->suspended)
 	{
 		return  0 ;
 	}
-#endif
 
 	ret = libssh2_channel_read( ((ml_pty_ssh_t*)pty)->channel , buf , len) ;
 
@@ -883,7 +997,7 @@ scp_stop(
 	ml_pty_ssh_t *  pty_ssh
 	)
 {
-	pty_ssh->session->doing_scp = -1 ;
+	pty_ssh->session->suspended = -1 ;
 
 	return  1 ;
 }
@@ -984,7 +1098,7 @@ error1:
 #endif	/* USE_WIN32API */
 
 #ifdef  __CYGWIN__
-static void
+static int
 check_sig_child(
 	pid_t  pid
 	)
@@ -995,6 +1109,12 @@ check_sig_child(
 	if( pid > 0 && waitpid( pid , &status , WNOHANG) == pid)
 	{
 		kik_trigger_sig_child( pid) ;
+
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
 	}
 }
 #endif
@@ -1007,7 +1127,14 @@ lo_read_pty(
 	)
 {
 #ifdef  __CYGWIN__
-	check_sig_child( pty->config_menu.pid) ;
+	if( check_sig_child( pty->config_menu.pid))
+	{
+		/*
+		 * ml_pty_set_use_loopback(0) is called from sig_child()
+		 * in ml_config_menu.c is called
+		 */
+		return  0 ;
+	}
 #endif
 
 	return  read( pty->master , buf , len) ;
@@ -1021,7 +1148,14 @@ lo_write_to_pty(
 	)
 {
 #ifdef  __CYGWIN__
-	check_sig_child( pty->config_menu.pid) ;
+	if( check_sig_child( pty->config_menu.pid))
+	{
+		/*
+		 * ml_pty_set_use_loopback(0) is called from sig_child()
+		 * in ml_config_menu.c is called
+		 */
+		return  0 ;
+	}
 #endif
 
 	if( len == 1 && buf[0] == '\x03')
@@ -1098,6 +1232,8 @@ use_loopback(
 
 	pty->stored->ref_count = 1 ;
 
+	((ml_pty_ssh_t*)pty)->session->suspended = 1 ;
+
 	return  1 ;
 }
 
@@ -1150,6 +1286,8 @@ unuse_loopback(
 	free( pty->stored) ;
 	pty->stored = NULL ;
 
+	((ml_pty_ssh_t*)pty)->session->suspended = 0 ;
+
 	return  1 ;
 }
 
@@ -1181,7 +1319,7 @@ scp_thread(
 
 	ml_write_to_pty( &scp->pty_ssh->pty , msg1 , sizeof(msg1) - 1) ;
 
-	while( rd_len < scp->src_size && scp->pty_ssh->session->doing_scp > 0)
+	while( rd_len < scp->src_size && scp->pty_ssh->session->suspended > 0)
 	{
 		int  new_progress ;
 		ssize_t  len ;
@@ -1244,7 +1382,7 @@ scp_thread(
 		}
 	}
 
-	if( scp->pty_ssh->session->doing_scp > 0)
+	if( scp->pty_ssh->session->suspended > 0)
 	{
 		ml_write_to_pty( &scp->pty_ssh->pty , msg2 , sizeof(msg2) - 1) ;
 	}
@@ -1264,7 +1402,7 @@ scp_thread(
 
 	unuse_loopback( &scp->pty_ssh->pty) ;
 
-	scp->pty_ssh->session->doing_scp = 0 ;
+	scp->pty_ssh->session->suspended = 0 ;
 
 	free( scp) ;
 
@@ -1628,111 +1766,6 @@ ssh_to_xserver(
 	}
 }
 
-#ifdef  OPEN_PTY_ASYNC
-
-#ifdef  USE_WIN32API
-
-static HANDLE *  openssl_locks ;
-
-static void
-openssl_lock_callback(
-	int  mode ,
-	int  type ,
-	const char *  file ,
-	int  line
-	)
-{
-	if( mode & 1 /* CRYPTO_LOCK */)
-	{
-		WaitForSingleObject( openssl_locks[type] , INFINITE) ;
-	}
-	else
-	{
-		ReleaseMutex( openssl_locks[type]) ;
-	}
-}
-
-#else
-
-static pthread_mutex_t *  openssl_locks ;
-
-static void
-openssl_lock_callback(
-	int  mode ,
-	int  type ,
-	const char *  file ,
-	int  line
-	)
-{
-	if( mode & 1 /* CRYPTO_LOCK */)
-	{
-		pthread_mutex_lock( &openssl_locks[type]) ;
-	}
-	else
-	{
-		pthread_mutex_unlock( &openssl_locks[type]) ;
-	}
-}
-
-#endif
-
-int  CRYPTO_num_locks(void) ;
-void  CRYPTO_set_locking_callback( void (*func)( int , int , const char * , int)) ;
-
-/* gcrypt is not supported. */
-static void
-set_use_multi_thread(
-	int  use
-	)
-{
-	static int  num_locks ;
-	int  count ;
-
-	if( use)
-	{
-		num_locks = CRYPTO_num_locks() ;
-
-		if( ( openssl_locks = malloc( num_locks * sizeof(*openssl_locks))))
-		{
-			for( count = 0 ; count < num_locks ; count++)
-			{
-			#ifdef  USE_WIN32API
-				openssl_locks[count] = CreateMutex( NULL , FALSE , NULL) ;
-			#else
-				openssl_locks[count] = PTHREAD_MUTEX_INITIALIZER ;
-			#endif
-			}
-
-			CRYPTO_set_locking_callback( openssl_lock_callback) ;
-		}
-		else
-		{
-			num_locks = 0 ;
-		}
-	}
-	else
-	{
-		if( openssl_locks)
-		{
-			CRYPTO_set_locking_callback( NULL) ;
-
-		#ifdef  USE_WIN32API
-			for( count = 0 ; count < num_locks ; count++)
-			{
-				CloseHandle( openssl_locks[count]) ;
-			}
-		#endif
-
-			free( openssl_locks) ;
-			openssl_locks = NULL ;
-		}
-	}
-}
-
-#else
-#define  set_use_multi_thread(use)  (0)
-#endif
-
 
 /* --- global functions --- */
 
@@ -1749,7 +1782,9 @@ ml_pty_ssh_new(
 	const char *  pubkey ,	/* can be NULL */
 	const char *  privkey ,	/* can be NULL */
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	ml_pty_ssh_t *  pty ;
@@ -1761,29 +1796,27 @@ ml_pty_ssh_new(
 	void *  p ;
 	int  ret ;
 
-	if( ( pty = calloc( 1 , sizeof( ml_pty_ssh_t))) == NULL)
-	{
-		return  NULL ;
-	}
-
 	if( ! kik_parse_uri( &proto , &user , &host , &port , NULL , NULL ,
 			kik_str_alloca_dup( uri)))
 	{
-		goto  error1 ;
+		return  NULL ;
 	}
 
 	/* USER: unix , USERNAME: win32 */
 	if( ! user && ! (user = getenv( "USER")) && ! (user = getenv( "USERNAME")))
 	{
-		goto  error1 ;
+		return  NULL ;
 	}
 
 	if( proto && strcmp( proto , "ssh") != 0)
 	{
-		goto  error1 ;
+		return  NULL ;
 	}
 
-	set_use_multi_thread( 1) ;
+	if( ( pty = calloc( 1 , sizeof( ml_pty_ssh_t))) == NULL)
+	{
+		return  NULL ;
+	}
 
 	if( ( pty->session = ssh_connect( host , port ? port : "22" , user , pass ,
 					pubkey , privkey)) == NULL)
@@ -1791,41 +1824,73 @@ ml_pty_ssh_new(
 		goto  error1 ;
 	}
 
+	if( pty->session->suspended)
+	{
+		goto  error2 ;
+	}
+
 	if( ! ( p = realloc( pty->session->pty_channels ,
 			(pty->session->num_of_ptys + 1) * sizeof(LIBSSH2_CHANNEL*))))
 	{
-		goto  error1 ;
+		goto  error2 ;
 	}
 
 	pty->session->pty_channels = p ;
 
+#if  0
 	while( ! ( pty->channel = libssh2_channel_open_session( pty->session->obj)))
+#else
+	while( ! ( pty->channel = libssh2_channel_open_ex( pty->session->obj ,
+					"session" , sizeof("session") - 1 ,
+					/* RLogin's window size */
+					64 * LIBSSH2_CHANNEL_PACKET_DEFAULT ,
+					LIBSSH2_CHANNEL_PACKET_DEFAULT , NULL , 0)))
+#endif
 	{
 		if( libssh2_session_last_errno( pty->session->obj) != LIBSSH2_ERROR_EAGAIN)
 		{
 		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " Unable to open a session\n") ;
+			kik_debug_printf( KIK_DEBUG_TAG " Unable to open a channel\n") ;
 		#endif
 
 			goto  error2 ;
 		}
 	}
 
-	pty->session->doing_scp = 0 ;
+	pty->session->suspended = 0 ;
 
-#ifdef  LIBSSH2_FORWARD_AGENT
 	if( auth_agent_is_available)
 	{
+	#if  defined(__CYGWIN__)
+		static int  (*func)( LIBSSH2_CHANNEL *) ;
+		static int  is_tried ;
+
+		if( ! is_tried)
+		{
+			func = GetProcAddress( GetModuleHandle("cygssh2-1") ,
+					"libssh2_channel_request_auth_agent") ;
+			is_tried = 1 ;
+		}
+
+		if( func)
+		{
+			while( ( ret = (*func)( pty->channel)) == LIBSSH2_ERROR_EAGAIN) ;
+			if( ret == 0)
+			{
+				kik_msg_printf( "Agent forwarding.\n") ;
+			}
+		}
+	#elif  defined(LIBSSH2_FORWARD_AGENT)
 		while( ( ret = libssh2_channel_request_auth_agent( pty->channel))
 				== LIBSSH2_ERROR_EAGAIN) ;
 		if( ret == 0)
 		{
 			kik_msg_printf( "Agent forwarding.\n") ;
 		}
+	#endif
 
 		auth_agent_is_available = 0 ;
 	}
-#endif
 
 	term = NULL ;
 	if( env)
@@ -1868,7 +1933,8 @@ ml_pty_ssh_new(
 		if( ret != LIBSSH2_ERROR_EAGAIN)
 		{
 		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " Failed to request pty\n") ;
+			kik_debug_printf( KIK_DEBUG_TAG " Failed to request pty. (Err %d)\n" ,
+				ret) ;
 		#endif
 
 			goto  error3 ;
@@ -1886,8 +1952,7 @@ ml_pty_ssh_new(
 	if( cmd_path)
 	{
 		int  count ;
-		char *   cmd_line ;
-		size_t   cmd_line_len ;
+		size_t  cmd_line_len ;
 
 		/* Because cmd_path == cmd_argv[0], cmd_argv[0] is ignored. */
 
@@ -1899,26 +1964,27 @@ ml_pty_ssh_new(
 			cmd_line_len += (strlen(cmd_argv[count]) + 3) ;
 		}
 
-		if( ( cmd_line = alloca( sizeof(char) * cmd_line_len)) == NULL)
+		if( ( pty->pty.cmd_line = malloc( sizeof(char) * cmd_line_len)) == NULL)
 		{
 			goto  error3 ;
 		}
 
-		strcpy( cmd_line, cmd_path) ;
+		strcpy( pty->pty.cmd_line , cmd_path) ;
 		for( count = 1 ; cmd_argv[count] != NULL ; count ++)
 		{
-			sprintf( cmd_line + strlen(cmd_line) ,
+			sprintf( pty->pty.cmd_line + strlen(pty->pty.cmd_line) ,
 				strchr( cmd_argv[count] , ' ') ? " \"%s\"" : " %s" ,
 				cmd_argv[count]) ;
 		}
 
-		while( ( ret = libssh2_channel_exec( pty->channel , cmd_line)) < 0)
+		while( ( ret = libssh2_channel_exec( pty->channel , pty->pty.cmd_line)) < 0)
 		{
 			if( ret != LIBSSH2_ERROR_EAGAIN)
 			{
 			#ifdef  DEBUG
 				kik_debug_printf( KIK_DEBUG_TAG
-					" Unable to exec %s on allocated pty\n" , cmd_line) ;
+					" Unable to exec %s on allocated pty. (Err %d)\n" ,
+					pty->pty.cmd_line , ret) ;
 			#endif
 
 				goto  error3 ;
@@ -1934,7 +2000,8 @@ ml_pty_ssh_new(
 			{
 			#ifdef  DEBUG
 				kik_debug_printf( KIK_DEBUG_TAG
-					" Unable to request shell on allocated pty\n") ;
+					" Unable to request shell on allocated pty. (Err %d)\n" ,
+					ret) ;
 			#endif
 
 				goto  error3 ;
@@ -1950,7 +2017,7 @@ ml_pty_ssh_new(
 	pty->pty.write = write_to_pty ;
 	pty->pty.read = read_pty ;
 
-	if( set_winsize( &pty->pty , cols , rows) == 0)
+	if( set_winsize( &pty->pty , cols , rows , width_pix , height_pix) == 0)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_set_pty_winsize() failed.\n") ;
@@ -1998,17 +2065,16 @@ ml_pty_ssh_new(
 	return  &pty->pty ;
 
 error3:
-	libssh2_session_set_blocking( pty->session->obj , 1) ;
+	libssh2_session_set_blocking( pty->session->obj , 1) ;	/* unblock in ssh_disconnect */
 	libssh2_channel_free( pty->channel) ;
 
 error2:
 	ssh_disconnect( pty->session) ;
+	set_use_multi_thread( 0) ;
 
 error1:
 	free( pty) ;
 
-	set_use_multi_thread( 0) ;
-
 	return  NULL ;
 }
 
@@ -2047,7 +2113,14 @@ ml_pty_set_use_loopback(
 {
 	if( use)
 	{
-		return  use_loopback( pty) ;
+		if( ((ml_pty_ssh_t*)pty)->session->suspended)
+		{
+			return  0 ;
+		}
+		else
+		{
+			return  use_loopback( pty) ;
+		}
 	}
 	else
 	{
@@ -2075,7 +2148,7 @@ ml_pty_ssh_scp_intern(
 		return  0 ;
 	}
 
-	if( ((ml_pty_ssh_t*)pty)->session->doing_scp)
+	if( ((ml_pty_ssh_t*)pty)->session->suspended)
 	{
 		kik_msg_printf( "SCP: Another scp process is working.\n") ;
 
@@ -2088,7 +2161,7 @@ ml_pty_ssh_scp_intern(
 	}
 	scp->pty_ssh = (ml_pty_ssh_t*)pty ;
 
-	scp->pty_ssh->session->doing_scp = 1 ;
+	scp->pty_ssh->session->suspended = 1 ;
 
 	if( src_is_remote)
 	{
@@ -2186,7 +2259,7 @@ ml_pty_ssh_scp_intern(
 	return  1 ;
 
 error:
-	scp->pty_ssh->session->doing_scp = 0 ;
+	scp->pty_ssh->session->suspended = 0 ;
 	free( scp) ;
 
 	return  0 ;
@@ -2265,12 +2338,10 @@ ml_pty_ssh_poll(
 	{
 		u_int  idx ;
 
-	#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
-		if( sessions[count]->doing_scp)
+		if( sessions[count]->suspended)
 		{
 			continue ;
 		}
-	#endif
 
 		for( idx = 0 ; idx < sessions[count]->num_of_ptys ; idx++)
 		{
@@ -2375,12 +2446,10 @@ ml_pty_ssh_send_recv_x11(
 
 	session = sessions[count] ;
 
-#if  defined(USE_WIN32API) || defined(HAVE_PTHREAD)
-	if( session->doing_scp)
+	if( session->suspended)
 	{
 		return  0 ;
 	}
-#endif
 
 	if( session->x11_fds[idx] == -1 ||	/* Failed to connect X server */
 	    ! ( ( ! bidirection ||
