@@ -63,27 +63,6 @@ read_challenge(void)
 	return  1 ;
 }
 
-static int
-challenge_it(
-	char *  chal
-	)
-{
-	if( challenge && strcmp( chal , challenge) == 0)
-	{
-		return  1 ;
-	}
-	
-	/*
-	 * Challenge could be re-generated.
-	 */
-	if( read_challenge() && challenge && strcmp( chal , challenge) == 0)
-	{
-		return  1 ;
-	}
-	
-	return  0 ;
-}
-
 
 /* --- global functions --- */
 
@@ -139,18 +118,53 @@ ml_gen_proto_challenge(void)
 }
 
 char *
+ml_get_proto_challenge(void)
+{
+	return  challenge ;
+}
+
+/*
+ * Returns 0 if illegal format.
+ * Returns -1 if do_challenge is 1 and challenge failed.
+ */
+int
 ml_parse_proto_prefix(
-	char **  str
+	char **  dev ,	/* can be NULL */
+	char **  str ,
+	int  do_challenge
 	)
 {
-	char *  beg ;
 	char *  p ;
 
-	beg = p = *str ;
+	p = *str ;
 
-	if( strncmp( p , "/dev" , 4) == 0)
+	while( do_challenge)
 	{
-		p += 3 ;
+		char *  chal ;
+
+		chal = p ;
+
+		if( ( p = strchr( p , ';')))
+		{
+			*(p ++) = '\0' ;
+
+			if( ( challenge && strcmp( chal , challenge) == 0) ||
+			    /* Challenge could have been re-generated. */
+			    ( read_challenge() && challenge && strcmp( chal , challenge) == 0))
+			{
+				/* challenge succeeded. */
+				break ;
+			}
+		}
+
+		return  -1 ;
+	}
+
+	*str = p ;	/* for no_dev */
+
+	if( strncmp( p , "/dev/" , 5) == 0)
+	{
+		p += 4 ;
 		while( *(++p) != ':')
 		{
 			/* Don't contain ';' in "/dev/...". */
@@ -161,7 +175,7 @@ ml_parse_proto_prefix(
 				kik_warn_printf( KIK_DEBUG_TAG " Illegal protocol format.\n") ;
 			#endif
 
-				return  NULL ;
+				goto  no_dev ;
 			}
 		}
 	}
@@ -187,19 +201,32 @@ ml_parse_proto_prefix(
 			}
 			else
 			{
-				return  NULL ;
+				goto  no_dev ;
 			}
 		}
 	}
 
+	if( dev)
+	{
+		*dev = *str ;
+	}
+
 	*(p ++) = '\0' ;
 	*str = p ;
 
-	return  beg ;
+	return  1 ;
+
+no_dev:
+	if( dev)
+	{
+		*dev = NULL ;
+	}
+
+	return  1 ;
 }
 
 /*
- * Returns 0 if error happens.
+ * Returns 0 if illegal format.
  * Returns -1 if do_challenge is 1 and challenge failed.
  * If finished parsing str, *str is set NULL(see *str = strchr( p , ';')).
  */
@@ -214,36 +241,12 @@ ml_parse_proto(
 	)
 {
 	char *  p ;
-	char *  _dev ;
 
 	p = *str ;
-	
-	if( do_challenge)
+
+	if( ml_parse_proto_prefix( dev , &p , do_challenge) < 0)
 	{
-		char *  chal ;
-
-		chal = p ;
-
-		if( ( p = strchr( p , ';')) == NULL)
-		{
-			/* Illegal format */
-
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " Illegal protocol format.\n") ;
-		#endif
-
-			return  0 ;
-		}
-
-		*(p ++) = '\0' ;
-
-		if( ! challenge_it( chal))
-		{
-			kik_msg_printf( "Protocol 5380 is not permitted "
-				"because client password is wrong.\n") ;
-
-			return  -1 ;
-		}
+		return  -1 ;
 	}
 
 	if( sep_by_semicolon)
@@ -259,12 +262,6 @@ ml_parse_proto(
 		*str = NULL ;
 	}
 
-	_dev = ml_parse_proto_prefix( &p) ;
-	if( dev)
-	{
-		*dev = _dev ;
-	}
-	
 	if( key)
 	{
 		*key = p ;
