@@ -35,14 +35,6 @@
 #endif
 
 
-/* --- static functions --- */
-
-#ifdef  USE_LIBSSH2
-
-
-#endif	/* USE_LIBSSH2 */
-
-
 /* --- global functions --- */
 
 ml_pty_t *
@@ -56,7 +48,9 @@ ml_pty_new(
 	const char *  pubkey ,		/* can be NULL */
 	const char *  privkey ,		/* can be NULL */
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	ml_pty_t *  pty ;
@@ -65,14 +59,14 @@ ml_pty_new(
 	if( ! pass)
 	{
 		pty = ml_pty_unix_new( cmd_path , cmd_argv , env , host ,
-				work_dir , cols , rows) ;
+				work_dir , cols , rows , width_pix , height_pix) ;
 	}
 	else
 #endif
 	{
 	#if  defined(USE_LIBSSH2)
 		pty = ml_pty_ssh_new( cmd_path , cmd_argv , env , host , pass ,
-				pubkey , privkey , cols , rows) ;
+				pubkey , privkey , cols , rows , width_pix , height_pix) ;
 	#elif  defined(USE_WIN32API)
 		pty = ml_pty_pipe_new( cmd_path , cmd_argv , env , host , pass , cols , rows) ;
 	#else
@@ -94,7 +88,9 @@ ml_pty_new_with(
 	int  slave ,
 	pid_t  child_pid ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	ml_pty_t *  pty ;
@@ -103,7 +99,7 @@ ml_pty_new_with(
 	if( ptsname( master))
 	{
 		pty = ml_pty_unix_new_with( master , slave , child_pid ,
-				":0.0" , cols , rows) ;
+				":0.0" , cols , rows , width_pix , height_pix) ;
 	}
 	else
 #endif
@@ -138,6 +134,7 @@ ml_pty_delete(
 #endif
 
 	free( pty->buf) ;
+	free( pty->cmd_line) ;
 	ml_config_menu_final( &pty->config_menu) ;
 
 	(*pty->final)( pty) ;
@@ -151,10 +148,12 @@ int
 ml_set_pty_winsize(
 	ml_pty_t *  pty ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
-	return  (*pty->set_winsize)( pty , cols , rows) ;
+	return  (*pty->set_winsize)( pty , cols , rows , width_pix , height_pix) ;
 }
 
 int
@@ -382,7 +381,16 @@ ml_response_config(
 	kik_debug_printf( KIK_DEBUG_TAG " %s\n" , res) ;
 #endif
 
-	if( to_menu)
+	if( to_menu < 0)
+	{
+		if( pty->pty_listener && pty->pty_listener->show_config)
+		{
+			/* '\n' -> '\0' */
+			res[strlen(res) - 1] = '\0' ;
+			(*pty->pty_listener->show_config)( pty->pty_listener->self , res + 1) ;
+		}
+	}
+	else if( to_menu > 0)
 	{
 		ml_config_menu_write( &pty->config_menu , res , res_len) ;
 	}
@@ -459,3 +467,11 @@ ml_start_config_menu(
 	return  ml_config_menu_start( &pty->config_menu , cmd_path ,
 				x , y , display , pty) ;
 }
+
+char *
+ml_pty_get_cmd_line(
+	ml_pty_t *  pty
+	)
+{
+	return  pty->cmd_line ;
+}
