@@ -33,6 +33,8 @@ typedef struct  ml_pty_event_listener
 	/* Called when ml_pty_delete. */
 	void  (*closed)( void *) ;
 
+	void  (*show_config)( void * , char *) ;
+
 } ml_pty_event_listener_t ;
 
 typedef struct  ml_pty *  ml_pty_ptr_t ;
@@ -40,16 +42,18 @@ typedef struct  ml_pty *  ml_pty_ptr_t ;
 
 ml_pty_ptr_t  ml_pty_new( const char *  cmd_path , char **  cmd_argv , char **  env ,
 	const char *  host , const char *  work_dir , const char *  pass ,
-	const char *  pubkey , const char *  privkey , u_int  cols , u_int  rows) ;
+	const char *  pubkey , const char *  privkey , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 ml_pty_ptr_t  ml_pty_new_with( int  master , int  slave , pid_t  child_pid ,
-	u_int  cols , u_int  rows) ;
+	u_int  cols , u_int  rows , u_int  width_pix , u_int  height_pix) ;
 
 int  ml_pty_delete( ml_pty_ptr_t  pty) ;
 
 int  ml_pty_set_listener( ml_pty_ptr_t  pty,  ml_pty_event_listener_t *  pty_listener) ;
 
-int  ml_set_pty_winsize( ml_pty_ptr_t  pty , u_int  cols , u_int  rows) ;
+int  ml_set_pty_winsize( ml_pty_ptr_t  pty , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 size_t  ml_write_to_pty( ml_pty_ptr_t  pty , u_char *  buf , size_t  len) ;
 
@@ -68,6 +72,8 @@ char *  ml_pty_get_slave_name( ml_pty_ptr_t  pty) ;
 int  ml_start_config_menu( ml_pty_ptr_t  pty , char *  cmd_path ,
 	int  x , int  y , char *  display) ;
 
+char *  ml_pty_get_cmd_line( ml_pty_ptr_t  pty) ;
+
 #ifdef  USE_LIBSSH2
 void *  ml_search_ssh_session( const char *  host , const char *  port , const char *  user) ;
 
@@ -90,7 +96,6 @@ int  ml_pty_ssh_poll( void *  fds) ;
 u_int  ml_pty_ssh_get_x11_fds( int **  fds) ;
 
 int  ml_pty_ssh_send_recv_x11( int  idx , int  bidirection) ;
-
 #endif
 
 
