@@ -21,7 +21,7 @@ typedef struct  ml_pty
 	size_t  size ;
 
 	int (*final)( ml_pty_ptr_t) ;
-	int (*set_winsize)( ml_pty_ptr_t , u_int , u_int) ;
+	int (*set_winsize)( ml_pty_ptr_t , u_int , u_int , u_int , u_int) ;
 	ssize_t (*write)( ml_pty_ptr_t , u_char * , size_t) ;
 	ssize_t (*read)( ml_pty_ptr_t , u_char * , size_t) ;
 
@@ -40,18 +40,23 @@ typedef struct  ml_pty
 
 	} *  stored ;
 
+	char *  cmd_line ;
+
 } ml_pty_t ;
 
 
 ml_pty_t *  ml_pty_unix_new( const char *  cmd_path , char **  cmd_argv , char **  env ,
-	const char *  host , const char *  work_dir , u_int  cols , u_int  rows) ;
+	const char *  host , const char *  work_dir , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 ml_pty_t *  ml_pty_unix_new_with( int  master , int  slave , pid_t  child_pid ,
-	const char *  host , u_int  cols , u_int  rows) ;
+	const char *  host , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 ml_pty_t *  ml_pty_ssh_new( const char *  cmd_path , char **  cmd_argv , char **  env ,
 	const char *  host , const char *  pass ,
-	const char *  pubkey , const char *  privkey , u_int  cols , u_int  rows) ;
+	const char *  pubkey , const char *  privkey , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 ml_pty_t *  ml_pty_pipe_new( const char *  cmd_path , char **  cmd_argv , char **  env ,
 	const char *  host , const char *  pass , u_int  cols , u_int  rows) ;
