@@ -16,6 +16,7 @@
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_pty.h>
 #include  <kiklib/kik_path.h>	/* kik_basename */
+#include  <kiklib/kik_unistd.h>	/* kik_killpg */
 #ifdef  USE_UTMP
 #include  <kiklib/kik_utmp.h>
 #endif
@@ -64,19 +65,22 @@ static int
 set_winsize(
 	ml_pty_t *  pty ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	struct winsize  ws ;
 
 #ifdef  __DEBUG
-	kik_debug_printf( KIK_DEBUG_TAG " win size cols %d rows %d.\n" , cols , rows) ;
+	kik_debug_printf( KIK_DEBUG_TAG " win size cols %d rows %d width %d height %d.\n" ,
+		cols , rows , width_pix , height_pix) ;
 #endif
 
 	ws.ws_col = cols ;
 	ws.ws_row = rows ;
-	ws.ws_xpixel = 0 ;
-	ws.ws_ypixel = 0 ;
+	ws.ws_xpixel = width_pix ;
+	ws.ws_ypixel = height_pix ;
 	
 	if( ioctl( pty->master , TIOCSWINSZ , &ws) < 0)
 	{
@@ -87,7 +91,24 @@ set_winsize(
 		return  0 ;
 	}
 
-	kill( pty->child_pid , SIGWINCH) ;
+	if( pty->child_pid > 1)
+	{
+		int  pgrp ;
+
+	#ifdef  TIOCGPGRP
+		if( ioctl( pty->master , TIOCGPGRP , &pgrp) != -1)
+		{
+			if( pgrp > 0)
+			{
+				kik_killpg( pgrp , SIGWINCH) ;
+			}
+		}
+		else
+	#endif
+		{
+			kik_killpg( pty->child_pid , SIGWINCH) ;
+		}
+	}
 
 	return  1 ;
 }
@@ -123,7 +144,9 @@ ml_pty_unix_new(
 	const char *  host ,
 	const char *  work_dir ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	ml_pty_t *  pty ;
@@ -153,11 +176,6 @@ ml_pty_unix_new(
 		signal(SIGCHLD, SIG_DFL) ;
 		signal(SIGPIPE, SIG_DFL) ;
 
-		if( ! cmd_path)
-		{
-			goto  return_pty ;
-		}
-
 		/*
 		 * setting environmental variables.
 		 */
@@ -187,6 +205,11 @@ ml_pty_unix_new(
 		signal(SIGTTOU , SIG_IGN) ;
 	#endif
 
+		if( ! cmd_path)
+		{
+			goto  return_pty ;
+		}
+
 		if( strchr( cmd_path , '/') == NULL)
 		{
 			execvp( cmd_path , cmd_argv) ;
@@ -206,7 +229,8 @@ ml_pty_unix_new(
 	/* parent process */
 
 return_pty:
-	if( ! ( pty = ml_pty_unix_new_with( master , slave , pid , host , cols , rows)))
+	if( ! ( pty = ml_pty_unix_new_with( master , slave , pid , host , cols , rows ,
+			width_pix , height_pix)))
 	{
 		close( master) ;
 		close( slave) ;
@@ -222,7 +246,9 @@ ml_pty_unix_new_with(
 	pid_t  child_pid ,
 	const char *  host ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	ml_pty_t *  pty ;
@@ -254,7 +280,7 @@ ml_pty_unix_new_with(
 		}
 	#endif
 
-		if( set_winsize( pty , cols , rows) == 0)
+		if( set_winsize( pty , cols , rows , width_pix , height_pix) == 0)
 		{
 		#ifdef  DEBUG
 			kik_warn_printf( KIK_DEBUG_TAG " ml_set_pty_winsize() failed.\n") ;
