@@ -5,6 +5,7 @@
 #include  "ml_screen.h"
 
 #include  <stdlib.h>		/* abs */
+#include  <sys/time.h>
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>	/* malloc/free */
 #include  <kiklib/kik_str.h>	/* strdup */
@@ -113,7 +114,9 @@ receive_scrolled_out_line(
 	}
 	else
 	{
-		line->num_of_filled_chars = ml_line_get_num_of_filled_chars_except_spaces( line) ;
+		line->num_of_filled_chars =
+			ml_line_get_num_of_filled_chars_except_spaces_with_func(
+				line , ml_char_equal) ;
 	}
 	
 	ml_log_add( &screen->logs , line) ;
@@ -779,6 +782,21 @@ check_or_copy_region(
 	return  region_size ;
 }
 
+static u_int32_t
+get_msec_time(void)
+{
+#ifdef  HAVE_GETTIMEOFDAY
+	struct timeval  tv ;
+
+	gettimeofday( &tv , NULL) ;
+
+	/* XXX '/ 1000' => '>> 10' and '* 1000' => '<< 10' */
+	return  (tv.tv_sec << 10) + (tv.tv_usec >> 10) ;
+#else
+	return  time(NULL) << 10 ;
+#endif
+}
+
 
 /* --- global functions --- */
 
@@ -820,6 +838,12 @@ ml_set_word_separators(
 	return  1 ;
 }
 
+char *
+ml_get_word_separators(void)
+{
+	return  word_separators ;
+}
+
 
 ml_screen_t *
 ml_screen_new(
@@ -832,7 +856,7 @@ ml_screen_new(
 	)
 {
 	ml_screen_t *  screen ;
-	
+
 	if( ( screen = calloc( 1 , sizeof( ml_screen_t))) == NULL)
 	{
 	#ifdef  DEBUG
@@ -943,6 +967,11 @@ ml_screen_resize(
 	ml_edit_resize( &screen->normal_edit , cols , rows) ;
 	ml_edit_resize( &screen->alt_edit , cols , rows) ;
 
+	if( screen->stored_edit)
+	{
+		ml_edit_resize( &screen->stored_edit->edit , cols , rows) ;
+	}
+
 	return  1 ;
 }
 
@@ -1082,14 +1111,6 @@ ml_screen_get_line_in_screen(
 	}
 }
 
-ml_line_t *
-ml_screen_get_cursor_line(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_get_line( screen->edit , ml_cursor_row( screen->edit)) ;
-}
-
 int
 ml_screen_set_modified_all(
 	ml_screen_t *  screen
@@ -2058,7 +2079,6 @@ ml_screen_get_n_prev_char(
 {
 	int  char_index ;
 	int  row ;
-	ml_char_t *  ch ;
 	ml_line_t *  line ;
 
 	if( ! get_n_prev_char_pos( screen , &char_index , &row , 1))
@@ -2071,12 +2091,7 @@ ml_screen_get_n_prev_char(
 		return  NULL ;
 	}
 
-	if( ( ch = ml_char_at( line , char_index)) == NULL)
-	{
-		return  NULL ;
-	}
-
-	return  ch ;
+	return  ml_char_at( line , char_index) ;
 }
 
 int
@@ -2199,7 +2214,8 @@ ml_screen_delete_lines(
 int
 ml_screen_go_forward(
 	ml_screen_t *  screen ,
-	u_int  size
+	u_int  size ,
+	int  scroll
 	)
 {
 	u_int  count ;
@@ -2208,8 +2224,27 @@ ml_screen_go_forward(
 	{
 		if( ! ml_edit_go_forward( screen->edit , 0))
 		{
+			if( scroll)
+			{
+				if( size > ml_edit_get_cols( screen->edit))
+				{
+					size = ml_edit_get_cols( screen->edit) ;
+					if( size <= count)
+					{
+						break ;
+					}
+				}
+
+				ml_edit_scroll_leftward( screen->edit , size - count) ;
+
+				break ;
+			}
 		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go forward any more.\n") ;
+			else
+			{
+				kik_warn_printf( KIK_DEBUG_TAG
+					" cursor cannot go forward any more.\n") ;
+			}
 		#endif
 		
 			return  0 ;
@@ -2222,7 +2257,8 @@ ml_screen_go_forward(
 int
 ml_screen_go_back(
 	ml_screen_t *  screen ,
-	u_int  size
+	u_int  size ,
+	int  scroll
 	)
 {
 	u_int  count ;
@@ -2231,8 +2267,27 @@ ml_screen_go_back(
 	{
 		if( ! ml_edit_go_back( screen->edit , 0))
 		{
+			if( scroll)
+			{
+				if( size > ml_edit_get_cols( screen->edit))
+				{
+					size = ml_edit_get_cols( screen->edit) ;
+					if( size <= count)
+					{
+						break ;
+					}
+				}
+
+				ml_edit_scroll_rightward( screen->edit , size - count) ;
+
+				break ;
+			}
 		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go back any more.\n") ;
+			else
+			{
+				kik_warn_printf( KIK_DEBUG_TAG
+					" cursor cannot go back any more.\n") ;
+			}
 		#endif
 					
 			return  0 ;
@@ -2288,24 +2343,6 @@ ml_screen_go_downward(
 	return  1 ;
 }
 
-int
-ml_screen_go_horizontally(
-	ml_screen_t *  screen ,
-	int  col
-	)
-{
-	return  ml_screen_goto( screen , col , ml_cursor_row( screen->edit)) ;
-}
-
-int
-ml_screen_go_vertically(
-	ml_screen_t *  screen ,
-	int  row
-	)
-{
-	return  ml_screen_goto( screen , ml_cursor_col( screen->edit) , row) ;
-}
-
 int
 ml_screen_cursor_visible(
 	ml_screen_t *  screen
@@ -2362,6 +2399,8 @@ ml_screen_use_normal_edit(
 {
 	if( screen->edit != &screen->normal_edit)
 	{
+		ml_screen_disable_local_echo( screen) ;
+
 		screen->normal_edit.bce_ch = screen->alt_edit.bce_ch ;
 		screen->edit = &screen->normal_edit ;
 
@@ -2384,6 +2423,8 @@ ml_screen_use_alternative_edit(
 {
 	if( screen->edit != &screen->alt_edit)
 	{
+		ml_screen_disable_local_echo( screen) ;
+
 		screen->alt_edit.bce_ch = screen->normal_edit.bce_ch ;
 		screen->edit = &screen->alt_edit ;
 
@@ -2402,63 +2443,58 @@ ml_screen_use_alternative_edit(
 	return  1 ;
 }
 
+/*
+ * This function must be called in logical context in order to swap
+ * stored_edit->edit and screen->edit in the same conditions.
+ */
 int
 ml_screen_enable_local_echo(
 	ml_screen_t *  screen
 	)
 {
-	if( screen->stored_edits)
+	if( ! screen->stored_edit)
 	{
-		screen->stored_edits->time = clock() / (CLOCKS_PER_SEC/10) ;
+		if( ! ( screen->stored_edit = malloc( sizeof( ml_stored_edit_t))))
+		{
+			return  0 ;
+		}
 
-		return  1 ;
+		screen->stored_edit->edit = *screen->edit ;
+
+		/*
+		 * New data is allocated in screen->edit, not in stored_edit->edit
+		 * because screen->edit allocated here will be deleted and
+		 * stored_edit->edit will be revived in ml_screen_disable_local_echo().
+		 */
+		if( ! ml_edit_clone( screen->edit , &screen->stored_edit->edit))
+		{
+			free( screen->stored_edit) ;
+			screen->stored_edit = NULL ;
+
+			return  0 ;
+		}
+
+		screen->edit->is_logging = 0 ;
 	}
 
-	if( ! ( screen->stored_edits = malloc( sizeof( ml_stored_edits_t))))
-	{
-		return  0 ;
-	}
-
-	screen->stored_edits->normal_edit = screen->normal_edit ;
-	screen->stored_edits->alt_edit = screen->alt_edit ;
-
-	if( ! ml_edit_clone( &screen->normal_edit , &screen->stored_edits->normal_edit))
-	{
-		goto  error1 ;
-	}
-
-	if( ! ml_edit_clone( &screen->alt_edit , &screen->stored_edits->alt_edit))
-	{
-		goto  error2 ;
-	}
-
-	screen->stored_edits->time = clock() / (CLOCKS_PER_SEC/10) ;
+	screen->stored_edit->time = get_msec_time() ;
 
 	return  1 ;
-
-error2:
-	ml_edit_final( &screen->stored_edits->normal_edit) ;
-
-error1:
-	free( screen->stored_edits) ;
-	screen->stored_edits = NULL ;
-
-	return  0 ;
 }
 
 int
 ml_screen_local_echo_wait(
 	ml_screen_t *  screen ,
-	int  msec		/* If 0 is specified, time is reset. */
+	int  msec	/* 0: stored_edit->time = 0 (>= get_msec_time() is always false.) */
 	)
 {
-	if( screen->stored_edits)
+	if( screen->stored_edit)
 	{
 		if( msec == 0)
 		{
-			screen->stored_edits->time = 0 ;
+			screen->stored_edit->time = 0 ;
 		}
-		else if( screen->stored_edits->time + msec / 100 >= clock() / (CLOCKS_PER_SEC/10))
+		else if( screen->stored_edit->time + msec >= get_msec_time())
 		{
 			return  1 ;
 		}
@@ -2467,6 +2503,10 @@ ml_screen_local_echo_wait(
 	return  0 ;
 }
 
+/*
+ * This function must be called in logical context in order to swap
+ * stored_edit->edit and screen->edit in the same conditions.
+ */
 int
 ml_screen_disable_local_echo(
 	ml_screen_t *  screen
@@ -2477,9 +2517,9 @@ ml_screen_disable_local_echo(
 	ml_line_t *  old_line ;
 	ml_line_t *  new_line ;
 
-	if( ! screen->stored_edits)
+	if( ! screen->stored_edit)
 	{
-		return  1 ;
+		return  0 ;
 	}
 
 	num_of_rows = ml_edit_get_rows( screen->edit) ;
@@ -2490,15 +2530,12 @@ ml_screen_disable_local_echo(
 	 */
 	for( row = 0 ; row < num_of_rows ; row++)
 	{
-		if( ( new_line = ml_edit_get_line(
-					( screen->edit == &screen->normal_edit ?
-					  &screen->stored_edits->normal_edit :
-					  &screen->stored_edits->alt_edit) , row)) &&
+		if( ( new_line = ml_edit_get_line( &screen->stored_edit->edit , row)) &&
 		    ( old_line = ml_edit_get_line( screen->edit , row)) &&
 		    ( ml_line_is_modified( old_line) ||
 		      old_line->num_of_filled_chars != new_line->num_of_filled_chars
-		#if  0
-		      || ml_str_bytes_equal( old_line->chars , new_line->chars ,
+		#if  1
+		      || ! ml_str_bytes_equal( old_line->chars , new_line->chars ,
 				new_line->num_of_filled_chars)
 		#endif
 		      ))
@@ -2507,14 +2544,11 @@ ml_screen_disable_local_echo(
 		}
 	}
 
-	ml_edit_final( &screen->normal_edit) ;
-	ml_edit_final( &screen->alt_edit) ;
+	ml_edit_final( screen->edit) ;
+	*screen->edit = screen->stored_edit->edit ;
 
-	screen->normal_edit = screen->stored_edits->normal_edit ;
-	screen->alt_edit = screen->stored_edits->alt_edit ;
-
-	free( screen->stored_edits) ;
-	screen->stored_edits = NULL ;
+	free( screen->stored_edit) ;
+	screen->stored_edit = NULL ;
 
 	return  1 ;
 }
