@@ -35,13 +35,12 @@ typedef enum  ml_bs_mode
 	
 } ml_bs_mode_t ;
 
-typedef struct  ml_stored_edits
+typedef struct  ml_stored_edit
 {
-	ml_edit_t  normal_edit ;
-	ml_edit_t  alt_edit ;
-	clock_t  time ;
+	ml_edit_t  edit ;
+	u_int32_t  time ;
 
-} ml_stored_edits_t ;
+} ml_stored_edit_t ;
 
 typedef struct  ml_screen
 {
@@ -54,7 +53,7 @@ typedef struct  ml_screen
 
 	ml_edit_t  normal_edit ;
 	ml_edit_t  alt_edit ;
-	ml_stored_edits_t *  stored_edits ;	/* Store logical edits. */
+	ml_stored_edit_t *  stored_edit ;	/* Store logical edits. */
 
 	ml_edit_scroll_event_listener_t  edit_scroll_listener ;
 	
@@ -88,6 +87,8 @@ typedef struct  ml_screen
 
 int  ml_set_word_separators( char *  seps) ;
 
+char *  ml_get_word_separators(void) ;
+
 #define  ml_free_word_separators()  ml_set_word_separators(NULL)
 
 
@@ -100,6 +101,10 @@ int  ml_screen_set_listener( ml_screen_t *  screen , ml_screen_event_listener_t
 
 int  ml_screen_resize( ml_screen_t *  screen , u_int  cols , u_int  rows) ;
 
+#define  ml_screen_set_use_bce( screen , use) \
+		ml_edit_set_use_bce( &(screen)->normal_edit , \
+			ml_edit_set_use_bce( &(screen)->normal_edit , use))
+
 #define  ml_screen_set_bce_fg_color( screen , fg_color) \
 		ml_edit_set_bce_fg_color( (screen)->edit , fg_color)
 
@@ -144,7 +149,8 @@ ml_line_t *  ml_screen_get_line( ml_screen_t *  screen , int  row) ;
 
 ml_line_t *  ml_screen_get_line_in_screen( ml_screen_t *  screen , int  row) ;
 
-ml_line_t *  ml_screen_get_cursor_line( ml_screen_t *  screen) ;
+#define  ml_screen_get_cursor_line( screen) \
+		ml_edit_get_line( (screen)->edit , ml_cursor_row( (screen)->edit))
 
 int  ml_screen_set_modified_all( ml_screen_t *  screen) ;
 
@@ -165,6 +171,8 @@ ml_bs_mode_t  ml_screen_is_backscrolling( ml_screen_t *  screen) ;
 
 int  ml_set_backscroll_mode( ml_screen_t *  screen , ml_bs_mode_t  mode) ;
 
+#define  ml_get_backscroll_mode( screen)  ((screen)->backscroll_mode)
+
 int  ml_enter_backscroll_mode( ml_screen_t *  screen) ;
 
 int  ml_exit_backscroll_mode( ml_screen_t *  screen) ;
@@ -228,11 +236,11 @@ int  ml_screen_insert_chars( ml_screen_t *  screen , ml_char_t *  chars , u_int
 #define  ml_screen_insert_blank_chars( screen , len) \
 		ml_edit_insert_blank_chars( (screen)->edit , len)
 
-#define  ml_screen_vertical_forward_tabs( screen , num) \
-		ml_edit_vertical_forward_tabs( (screen)->edit , num) \
+#define  ml_screen_forward_tabs( screen , num) \
+		ml_edit_forward_tabs( (screen)->edit , num) \
 
-#define  ml_screen_vertical_backward_tabs( screen , num) \
-		ml_edit_vertical_backward_tabs( (screen)->edit , num) \
+#define  ml_screen_backward_tabs( screen , num) \
+		ml_edit_backward_tabs( (screen)->edit , num) \
 
 #define  ml_screen_set_tab_stop( screen)  ml_edit_set_tab_stop( (screen)->edit)
 
@@ -260,14 +268,14 @@ int  ml_screen_delete_lines( ml_screen_t *  screen , u_int  size) ;
 
 #define  ml_screen_clear_above( screen)  ml_edit_clear_above( (screen)->edit)
 
-#define  ml_screen_set_scroll_region( screen , beg , end) \
-		ml_edit_set_scroll_region( (screen)->edit , beg , end)
+#define  ml_screen_set_vmargin( screen , beg , end) \
+		ml_edit_set_vmargin( (screen)->edit , beg , end)
 
-#define  ml_screen_set_use_margin( screen , use) \
-		ml_edit_set_use_margin( (screen)->edit , use)
+#define  ml_screen_set_use_hmargin( screen , use) \
+		ml_edit_set_use_hmargin( (screen)->edit , use)
 
-#define  ml_screen_set_margin( screen , beg , end) \
-		ml_edit_set_margin( (screen)->edit , beg , end)
+#define  ml_screen_set_hmargin( screen , beg , end) \
+		ml_edit_set_hmargin( (screen)->edit , beg , end)
 
 #define  ml_screen_index( screen)  ml_edit_go_downward( (screen)->edit , SCROLL)
 
@@ -281,9 +289,15 @@ int  ml_screen_delete_lines( ml_screen_t *  screen , u_int  size) ;
 
 #define  ml_screen_scroll_rightward( screen , size)  ml_edit_scroll_rightward( (screen)->edit , size)
 
-int  ml_screen_go_forward( ml_screen_t *  screen , u_int  size) ;
+#define  ml_screen_scroll_leftward_from_cursor( screen , size) \
+	ml_edit_scroll_leftward_from_cursor( (screen)->edit , size)
 
-int  ml_screen_go_back( ml_screen_t *  screen , u_int  size) ;
+#define  ml_screen_scroll_rightward_from_cursor( screen , size) \
+	ml_edit_scroll_rightward_from_cursor( (screen)->edit , size)
+
+int  ml_screen_go_forward( ml_screen_t *  screen , u_int  size , int  scroll) ;
+
+int  ml_screen_go_back( ml_screen_t *  screen , u_int  size , int  scroll) ;
 
 int  ml_screen_go_upward( ml_screen_t *  screen , u_int  size) ;
 
@@ -291,9 +305,11 @@ int  ml_screen_go_downward( ml_screen_t *  screen , u_int  size) ;
 
 #define  ml_screen_goto_beg_of_line( screen)  ml_edit_goto_beg_of_line( (screen)->edit)
 
-int  ml_screen_go_horizontally( ml_screen_t *  screen , int  col) ;
+#define  ml_screen_go_horizontally( screen , col) \
+		ml_screen_goto( screen , col , ml_cursor_row( (screen)->edit))
 
-int  ml_screen_go_vertically( ml_screen_t *  screen , int  row) ;
+#define  ml_screen_go_vertically( screen , row) \
+		ml_screen_goto( screen , ml_cursor_col( (screen)->edit) , row)
 
 #define  ml_screen_goto_home( screen)  ml_edit_goto_home( (screen)->edit)
 
@@ -305,6 +321,8 @@ int  ml_screen_go_vertically( ml_screen_t *  screen , int  row) ;
 
 #define  ml_screen_set_auto_wrap( screen , flag)  ml_edit_set_auto_wrap( (screen)->edit , flag)
 
+#define  ml_screen_is_auto_wrap( screen)  ml_edit_is_auto_wrap( (screen)->edit)
+
 #define  ml_screen_save_cursor( screen)  ml_edit_save_cursor( (screen)->edit)
 
 #define  ml_screen_restore_cursor( screen)  ml_edit_restore_cursor( (screen)->edit)
@@ -321,6 +339,8 @@ int  ml_screen_use_alternative_edit( ml_screen_t *  screen) ;
 
 int  ml_screen_is_alternative_edit( ml_screen_t *  screen) ;
 
+#define  ml_screen_is_local_echo_mode( screen)  ((screen)->stored_edit)
+
 int  ml_screen_enable_local_echo( ml_screen_t *  screen) ;
 
 int  ml_screen_local_echo_wait( ml_screen_t *  screen , int  msec) ;
@@ -338,6 +358,17 @@ int  ml_screen_fill_area( ml_screen_t *  screen , int  code ,
 #define  ml_screen_erase_area( screen , col , row , num_of_cols , num_of_rows) \
 		ml_edit_erase_area( (screen)->edit , col , row , num_of_cols , num_of_rows)
 
+#define  ml_screen_change_attr_area( screen , col , row , num_of_cols , num_of_rows , attr) \
+	ml_edit_change_attr_area( (screen)->edit , col , row , num_of_cols , num_of_rows , \
+		ml_char_change_attr , attr)
+
+#define  ml_screen_reverse_attr_area( screen , col , row , num_of_cols , num_of_rows , attr) \
+	ml_edit_change_attr_area( (screen)->edit , col , row , num_of_cols , num_of_rows , \
+		ml_char_reverse_attr , attr)
+
+#define  ml_screen_set_use_rect_attr_select( screen , use) \
+	ml_edit_set_use_rect_attr_select( (screen)->edit , use)
+
 void  ml_screen_enable_blinking( ml_screen_t *  screen) ;
 
 
