@@ -8,6 +8,7 @@
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_sig_child.h>
+#include  <kiklib/kik_path.h>	/* kik_parse_uri */
 
 #include  "ml_pty.h"
 #include  "ml_vt100_parser.h"
@@ -34,6 +35,8 @@ typedef struct
 	char *  pass ;
 	char *  pubkey ;
 	char *  privkey ;
+	u_int  width_pix ;
+	u_int  height_pix ;
 
 } pty_args_t ;
 
@@ -97,7 +100,9 @@ pty_args_new(
 	const char *  work_dir ,
 	const char *  pass ,
 	const char *  pubkey ,
-	const char *  privkey
+	const char *  privkey ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	pty_args_t *  args ;
@@ -141,6 +146,9 @@ pty_args_new(
 		args->privkey = strdup( privkey) ;
 	}
 
+	args->width_pix = width_pix ;
+	args->height_pix = height_pix ;
+
 	if( argv)
 	{
 		for( num = 0 ; argv[num] ; num++) ;
@@ -212,7 +220,8 @@ open_pty(
 	pty = ml_pty_new( args->cmd_path , args->argv , args->env , args->host ,
 			args->work_dir , args->pass , args->pubkey , args->privkey ,
 			ml_screen_get_logical_cols( args->term->screen) ,
-			ml_screen_get_logical_rows( args->term->screen)) ;
+			ml_screen_get_logical_rows( args->term->screen) ,
+			args->width_pix , args->height_pix) ;
 
 	if( pty)
 	{
@@ -261,8 +270,16 @@ set_use_local_echo(
 
 /* --- global functions --- */
 
+void
+ml_term_final(void)
+{
+	ml_vt100_parser_final() ;
+	ml_termcap_final() ;
+}
+
 ml_term_t *
 ml_term_new(
+	const char *  term_type ,
 	u_int  cols ,
 	u_int  rows ,
 	u_int  tab_size ,
@@ -275,11 +292,9 @@ ml_term_new(
 	u_int  col_size_a ,
 	int  use_char_combining ,
 	int  use_multi_col_char ,
-	int  use_bidi ,
+	int  use_ctl ,
 	ml_bidi_mode_t  bidi_mode ,
 	const char *  bidi_separators ,
-	int  use_ind ,
-	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode ,
@@ -289,8 +304,14 @@ ml_term_new(
 	ml_alt_color_mode_t  alt_color_mode
 	)
 {
+	ml_termcap_entry_t *  termcap ;
 	ml_term_t *  term ;
 
+	if( ! ( termcap = ml_termcap_get( term_type)))
+	{
+		return  NULL ;
+	}
+
 	if( ( term = calloc( 1 , sizeof( ml_term_t))) == NULL)
 	{
 	#ifdef  DEBUG
@@ -300,8 +321,9 @@ ml_term_new(
 		return  NULL ;
 	}
 
-	if( ( term->screen = ml_screen_new( cols , rows ,
-				tab_size , log_size , use_bce , bs_mode)) == NULL)
+	if( ! ( term->screen = ml_screen_new( cols , rows , tab_size ,
+				log_size , ml_termcap_get_bool_field( termcap , TC_BCE) ,
+				bs_mode)))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_screen_new failed.\n") ;
@@ -310,8 +332,8 @@ ml_term_new(
 		goto  error ;
 	}
 
-	if( ! ( term->parser = ml_vt100_parser_new( term->screen , encoding , is_auto_encoding ,
-				use_auto_detect , logging_vt_seq ,
+	if( ! ( term->parser = ml_vt100_parser_new( term->screen , termcap , encoding ,
+				is_auto_encoding , use_auto_detect , logging_vt_seq ,
 				policy , col_size_a , use_char_combining , use_multi_col_char ,
 				win_name , icon_name , alt_color_mode)))
 	{
@@ -322,7 +344,7 @@ ml_term_new(
 		goto  error ;
 	}
 
-	ml_vt100_parser_set_use_bidi( term->parser , use_bidi) ;
+	ml_vt100_parser_set_use_ctl( term->parser , use_ctl) ;
 
 	if( bidi_separators)
 	{
@@ -331,8 +353,7 @@ ml_term_new(
 
 	term->vertical_mode = vertical_mode ;
 	term->bidi_mode = bidi_mode ;
-	term->use_bidi = use_bidi ;
-	term->use_ind = use_ind ;
+	term->use_ctl = use_ctl ;
 	term->use_dynamic_comb = use_dynamic_comb ;
 	term->use_local_echo = use_local_echo ;
 
@@ -376,11 +397,6 @@ ml_term_delete(
 	{
 		(*term->pty_listener->closed)( term->pty_listener->self) ;
 	}
-	
-	if( term->shape)
-	{
-		(*term->shape->delete)( term->shape) ;
-	}
 
 	free( term->uri) ;
 	free( term->icon_path) ;
@@ -430,70 +446,95 @@ ml_term_open_pty(
 	const char *  work_dir ,
 	const char *  pass ,
 	const char *  pubkey ,
-	const char *  privkey
+	const char *  privkey ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
+	if( term->vertical_mode)
+	{
+		u_int  tmp ;
+
+		tmp = width_pix ;
+		width_pix = height_pix ;
+		height_pix = tmp ;
+	}
+
 	if( ! term->pty)
 	{
 	#ifdef  OPEN_PTY_ASYNC
-		pty_args_t *  args ;
+		char *  user ;
+		char *  server ;
+		char *  port ;
 
-		if( ! ( args = pty_args_new( term , cmd_path , argv , env , host ,
-					work_dir , pass , pubkey , privkey)))
+		if( pass &&
+		    kik_parse_uri( NULL , &user , &server , &port , NULL , NULL ,
+			kik_str_alloca_dup( host)) &&
+		    ! ml_search_ssh_session( server , port , user))
 		{
-			return  0 ;
-		}
+			pty_args_t *  args ;
 
-	#ifdef  USE_WIN32API
-		{
-			HANDLE  thrd ;
-			u_int  tid ;
-
-			if( ( thrd = _beginthreadex( NULL , 0 , open_pty , args , 0 , &tid)))
-			{
-				CloseHandle( thrd) ;
-
-				return  1 ;
-			}
-
-			return  0 ;
-		}
-	#else
-		{
-			pthread_t  thrd ;
-
-			if( pthread_create( &thrd , NULL , open_pty , args) == 0)
-			{
-				return  1 ;
-			}
-			else
+			if( ! ( args = pty_args_new( term , cmd_path , argv , env , host ,
+						work_dir , pass , pubkey , privkey ,
+						width_pix , height_pix)))
 			{
 				return  0 ;
 			}
-		}
-	#endif
-	#else	/* OPEN_PTY_ASYNC */
-		ml_pty_ptr_t  pty ;
 
-		if( ! ( pty = ml_pty_new( cmd_path , argv , env , host , work_dir ,
-				pass , pubkey , privkey ,
-				ml_screen_get_logical_cols( term->screen) ,
-				ml_screen_get_logical_rows( term->screen))))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " ml_pty_new failed.\n") ;
+		#ifdef  USE_WIN32API
+			{
+				HANDLE  thrd ;
+				u_int  tid ;
+
+				if( ( thrd = _beginthreadex( NULL , 0 , open_pty , args , 0 , &tid)))
+				{
+					CloseHandle( thrd) ;
+
+					return  1 ;
+				}
+
+				return  0 ;
+			}
+		#else
+			{
+				pthread_t  thrd ;
+
+				if( pthread_create( &thrd , NULL , open_pty , args) == 0)
+				{
+					return  1 ;
+				}
+				else
+				{
+					return  0 ;
+				}
+			}
 		#endif
-
-			return  0 ;
 		}
-
-		if( pass)
+		else
+	#endif /* OPEN_PTY_ASYNC */
 		{
-			term->uri = strdup( host) ;
-		}
+			ml_pty_ptr_t  pty ;
 
-		ml_term_plug_pty( term , pty) ;
-	#endif	/* OPEN_PTY_ASYNC */
+			if( ! ( pty = ml_pty_new( cmd_path , argv , env , host , work_dir ,
+					pass , pubkey , privkey ,
+					ml_screen_get_logical_cols( term->screen) ,
+					ml_screen_get_logical_rows( term->screen) ,
+					width_pix , height_pix)))
+			{
+			#ifdef  DEBUG
+				kik_warn_printf( KIK_DEBUG_TAG " ml_pty_new failed.\n") ;
+			#endif
+
+				return  0 ;
+			}
+
+			if( pass)
+			{
+				term->uri = strdup( host) ;
+			}
+
+			ml_term_plug_pty( term , pty) ;
+		}
 	}
 
 	return  1 ;
@@ -584,32 +625,14 @@ ml_term_detach(
 }
 
 int
-ml_term_set_use_bidi(
+ml_term_set_use_ctl(
 	ml_term_t *  term ,
 	int  flag
 	)
 {
-	if( ( term->use_bidi = flag))
+	if( ( term->use_ctl = flag))
 	{
-		/* disable iscii */
-		term->use_ind = 0 ;
-
-		ml_vt100_parser_set_use_bidi( term->parser , flag) ;
-	}
-
-	return  1 ;
-}
-
-int
-ml_term_set_use_ind(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	if( ( term->use_ind = flag))
-	{
-		/* disable bidi */
-		ml_term_set_use_bidi( term , 0) ;
+		ml_vt100_parser_set_use_ctl( term->parser , flag) ;
 	}
 
 	return  1 ;
@@ -699,12 +722,14 @@ int
 ml_term_resize(
 	ml_term_t *  term ,
 	u_int  cols ,
-	u_int  rows
+	u_int  rows ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	if( term->pty)
 	{
-		ml_set_pty_winsize( term->pty , cols , rows) ;
+		ml_set_pty_winsize( term->pty , cols , rows , width_pix , height_pix) ;
 	}
 
 	ml_screen_logical( term->screen) ;
@@ -992,12 +1017,6 @@ ml_term_update_special_visual(
 	int  has_logvis = 0 ;
 	int  need_comb = 0 ;
 
-	if( term->shape)
-	{
-		(*term->shape->delete)( term->shape) ;
-		term->shape = NULL ;
-	}
-
 	term->screen->use_dynamic_comb = 0 ;
 	had_logvis = ml_screen_delete_logical_visual( term->screen) ;
 
@@ -1057,10 +1076,11 @@ ml_term_update_special_visual(
 		}
 	#endif
 	}
-	else if( term->use_bidi && ml_term_get_encoding( term) == ML_UTF8)
+	else if( term->use_ctl &&
+	         ( ml_term_get_encoding( term) == ML_UTF8 ||
+		   IS_ISCII_ENCODING( ml_term_get_encoding( term))))
 	{
-		if( ( term->shape = ml_arabic_shape_new()) &&
-		    ( logvis = ml_logvis_bidi_new( term->bidi_mode , term->bidi_separators)))
+		if( ( logvis = ml_logvis_ctl_new( term->bidi_mode , term->bidi_separators)))
 		{
 			if( ml_screen_add_logical_visual( term->screen , logvis))
 			{
@@ -1079,37 +1099,7 @@ ml_term_update_special_visual(
 	#ifdef  DEBUG
 		else
 		{
-			kik_warn_printf( KIK_DEBUG_TAG
-				" x_arabic_shape_new()/ml_logvis_bidi_new() failed.\n") ;
-		}
-	#endif
-	}
-	else if( IS_ISCII_ENCODING( ml_term_get_encoding( term))
-	         || (/* ! term->use_bidi && */ term->use_ind &&
-	             ml_term_get_encoding( term) == ML_UTF8) )
-	{
-		if( ( term->shape = ml_iscii_shape_new()) &&
-		    ( logvis = ml_logvis_iscii_new()))
-		{
-			if( ml_screen_add_logical_visual( term->screen , logvis))
-			{
-				has_logvis = 1 ;
-			}
-			else
-			{
-			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG
-					" ml_screen_add_logical_visual failed.\n") ;
-			#endif
-
-				(*logvis->delete)( logvis) ;
-			}
-		}
-	#ifdef  DEBUG
-		else
-		{
-			kik_warn_printf( KIK_DEBUG_TAG
-				" ml_iscii_shape_new()/ml_logvis_iscii_new() failed.\n") ;
+			kik_warn_printf( KIK_DEBUG_TAG " ml_logvis_ctl_new() failed.\n") ;
 		}
 	#endif
 	}
@@ -1122,12 +1112,6 @@ ml_term_update_special_visual(
 
 	if( ! has_logvis)
 	{
-		if( term->shape)
-		{
-			(*term->shape->delete)( term->shape) ;
-			term->shape = NULL ;
-		}
-
 		term->screen->use_dynamic_comb = 0 ;
 		ml_screen_delete_logical_visual( term->screen) ;
 	}
@@ -1202,7 +1186,7 @@ ml_term_set_bidi_separators(
 int
 ml_term_get_config(
 	ml_term_t *  term ,
-	ml_term_t *  output ,
+	ml_term_t *  output ,	/* if term == output, NULL is set */
 	char *  key ,
 	int  to_menu ,
 	int *  flag
@@ -1210,7 +1194,8 @@ ml_term_get_config(
 {
 	char *  value ;
 
-	if( ml_vt100_parser_get_config( term->parser , output->pty , key , to_menu , flag))
+	if( ml_vt100_parser_get_config( term->parser , output ? output->pty : NULL ,
+		key , to_menu , flag))
 	{
 		return  1 ;
 	}
@@ -1230,20 +1215,9 @@ ml_term_get_config(
 			value = "false" ;
 		}
 	}
-	else if( strcmp( key , "use_bidi") == 0)
+	else if( strcmp( key , "use_ctl") == 0)
 	{
-		if( term->use_bidi)
-		{
-			value = "true" ;
-		}
-		else
-		{
-			value = "false" ;
-		}
-	}
-	else if( strcmp( key , "use_ind") == 0)
-	{
-		if( term->use_ind)
+		if( term->use_ctl)
 		{
 			value = "true" ;
 		}
@@ -1265,7 +1239,7 @@ ml_term_get_config(
 	}
 	else if( strcmp( key , "pty_name") == 0)
 	{
-		if( term != output)
+		if( output)
 		{
 			if( ( value = ml_get_window_name( term->parser)) == NULL)
 			{
@@ -1295,17 +1269,26 @@ ml_term_get_config(
 			value = "false" ;
 		}
 	}
+	else if( strcmp( key , "word_separators") == 0)
+	{
+		value = ml_get_word_separators() ;
+	}
 	else
 	{
 		/* Continue to process it in x_screen.c */
 		return  0 ;
 	}
 
+	if( ! output)
+	{
+		output = term ;
+	}
+
 	/* value is never set NULL above. */
 #if  0
 	if( ! value)
 	{
-		ml_response_config( term->pty , "error" , NULL , to_menu) ;
+		ml_response_config( output->pty , "error" , NULL , to_menu) ;
 	}
 #endif
 
@@ -1315,7 +1298,7 @@ ml_term_get_config(
 	}
 	else
 	{
-		ml_response_config( term->pty , key , value , to_menu) ;
+		ml_response_config( output->pty , key , value , to_menu) ;
 	}
 
 	return  1 ;
