@@ -30,11 +30,6 @@ typedef struct ml_term
 	ml_vt100_parser_t *  parser ;
 	ml_screen_t *  screen ;
 
-	/*
-	 * public(read/write)
-	 */
-	ml_shape_t *  shape ;
-
 	/*
 	 * private
 	 */
@@ -44,8 +39,7 @@ typedef struct ml_term
 	char *  icon_path ;
 	char *  uri ;
 
-	int8_t  use_bidi ;
-	int8_t  use_ind ;
+	int8_t  use_ctl ;
 	int8_t  use_dynamic_comb ;
 	int8_t  use_local_echo ;
 	int8_t  is_attached ;
@@ -61,12 +55,17 @@ typedef struct ml_term
 /* XXX */
 extern void (*ml_term_pty_closed_event)( ml_term_t *) ;
 
-ml_term_t *  ml_term_new( u_int  cols , u_int  rows , u_int  tab_size , u_int  log_size ,
-	ml_char_encoding_t  encoding , int  is_auto_encoding , int  use_auto_detect ,
+#define  ml_term_init()  ml_vt100_parser_init()
+
+void  ml_term_final(void) ;
+
+ml_term_t *  ml_term_new( const char *  term_type , u_int  cols , u_int  rows ,
+	u_int  tab_size , u_int  log_size , ml_char_encoding_t  encoding ,
+	int  is_auto_encoding , int  use_auto_detect ,
 	int  logging_vt_seq , ml_unicode_policy_t  policy ,
-	u_int  col_size_a , int  use_char_combining , int  use_multi_col_char , int  use_bidi ,
-	ml_bidi_mode_t  bidi_mode , const char *  bidi_separators , int  use_ind ,
-	int  use_bce , int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
+	u_int  col_size_a , int  use_char_combining , int  use_multi_col_char , int  use_ctl ,
+	ml_bidi_mode_t  bidi_mode , const char *  bidi_separators ,
+	int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode , int  use_local_echo ,
 	const char *  win_name , const char *  icon_name , ml_alt_color_mode_t  alt_color_mode) ;
 
@@ -76,10 +75,12 @@ int  ml_term_zombie( ml_term_t *  term) ;
 
 int  ml_term_open_pty( ml_term_t *  term , const char *  cmd_path , char ** argv , char **  env ,
 	const char *  host , const char *  work_dir , const char *  pass ,
-	const char *  pubkey , const char *  privkey) ;
+	const char *  pubkey , const char *  privkey , u_int  width_pix , u_int  height_pix) ;
 
 int  ml_term_plug_pty( ml_term_t *  term , ml_pty_ptr_t  pty) ;
 
+#define  ml_term_pty_is_opened( term)  ((term)->pty != NULL)
+
 int  ml_term_attach( ml_term_t *  term , ml_xterm_event_listener_t *  xterm_listener ,
 	ml_config_event_listener_t *  config_listener ,
 	ml_screen_event_listener_t *  screen_listener ,
@@ -110,18 +111,14 @@ int  ml_term_detach( ml_term_t *  term) ;
 
 #define  ml_term_get_encoding( term)  ml_vt100_parser_get_encoding( (term)->parser)
 
-int  ml_term_set_use_bidi( ml_term_t *  term , int  flag) ;
+int  ml_term_set_use_ctl( ml_term_t *  term , int  flag) ;
 
-#define  ml_term_is_using_bidi( term)  ((term)->use_bidi)
+#define  ml_term_is_using_ctl( term)  ((term)->use_ctl)
 
 #define  ml_term_set_bidi_mode( term , mode)  ((term)->bidi_mode = (mode))
 
 #define  ml_term_get_bidi_mode( term)  ((term)->bidi_mode)
 
-int  ml_term_set_use_ind( ml_term_t *  term , int  flag) ;
-
-#define  ml_term_is_using_ind( term)  ((term)->use_ind)
-
 #define  ml_term_set_vertical_mode( term , mode)  ((term)->vertical_mode = (mode))
 
 #define  ml_term_get_vertical_mode( term)  ((term)->vertical_mode)
@@ -147,11 +144,27 @@ pid_t  ml_term_get_child_pid( ml_term_t *  term) ;
 
 size_t  ml_term_write( ml_term_t *  term , u_char *  buf , size_t  len) ;
 
+#define  ml_term_write_modified_key( term , key , modcode) \
+	(term)->pty ? ml_vt100_parser_write_modified_key( (term)->parser , key , modcode) : 0
+
+#define  ml_term_write_special_key( term , key , modcode , is_numlock) \
+	(term)->pty ? \
+		ml_vt100_parser_write_special_key( (term)->parser , key , modcode , is_numlock) : 0
+
 /* Must be called in visual context. */
 #define  ml_term_write_loopback( term , buf , len) \
-		ml_vt100_parser_write_loopback( (term)->parser , buf , len)
+	ml_vt100_parser_write_loopback( (term)->parser , buf , len)
 
-int  ml_term_resize( ml_term_t *  term , u_int  cols , u_int  rows) ;
+/* Must be called in visual context. */
+#define  ml_term_show_message( term , msg)  ml_vt100_parser_show_message( (term)->parser , msg)
+
+#ifdef  __ANDROID__
+/* Must be called in visual context. */
+#define  ml_term_preedit( term , buf , len)  ml_vt100_parser_preedit( (term)->parser , buf , len)
+#endif
+
+int  ml_term_resize( ml_term_t *  term , u_int  cols , u_int  rows ,
+	u_int  width_pix , u_int  height_pix) ;
 
 #define  ml_term_cursor_col( term)  ml_screen_cursor_col( (term)->screen)
 
@@ -267,21 +280,8 @@ int  ml_term_enter_backscroll_mode( ml_term_t *  term) ;
 #define  ml_term_get_mouse_report_mode( term) \
 		ml_vt100_parser_get_mouse_report_mode((term)->parser)
 
-#define  ml_term_get_extended_mouse_report_mode( term) \
-		ml_vt100_parser_get_extended_mouse_report_mode((term)->parser)
-
-#define  ml_term_is_app_keypad( term) \
-		ml_vt100_parser_is_app_keypad((term)->parser)
-
-#define  ml_term_is_app_cursor_keys( term) \
-		ml_vt100_parser_is_app_cursor_keys((term)->parser)
-
-#define  ml_term_is_app_escape( term)  ml_vt100_parser_is_app_escape((term)->parser)
-
 #define  ml_term_want_focus_event( term)  ml_vt100_parser_want_focus_event((term)->parser)
 
-#define  ml_term_modify_other_keys( term)  ml_vt100_parser_modify_other_keys((term)->parser)
-
 #define  ml_term_set_alt_color_mode( term , mode) \
 		ml_vt100_parser_set_alt_color_mode((term)->parser , mode)
 
@@ -308,7 +308,10 @@ int  ml_term_set_icon_path( ml_term_t *  term , const char *  path) ;
 
 void  ml_term_set_bidi_separators( ml_term_t *  term , const char *  bidi_separators) ;
 
+#define  ml_term_get_cmd_line( term)  ml_pty_get_cmd_line( (term)->pty)
+
 #define  ml_term_start_config_menu( term , cmd_path , x , y , display) \
+		ml_reset_pending_vt100_sequence( (term)->parser) ; \
 		ml_start_config_menu( (term)->pty , cmd_path , x , y , display)
 
 int  ml_term_get_config( ml_term_t *  term , ml_term_t *  output , char *  key ,
@@ -316,7 +319,12 @@ int  ml_term_get_config( ml_term_t *  term , ml_term_t *  output , char *  key ,
 
 int  ml_term_set_config( ml_term_t *  term , char *  key , char *  value) ;
 
-#define  ml_term_exec_cmd( term , cmd)  ml_vt100_parser_exec_cmd( term->parser , cmd)
+#define  ml_term_exec_cmd( term , cmd)  ml_vt100_parser_exec_cmd( (term)->parser , cmd)
+
+#define  ml_term_report_mouse_tracking( term , col , row , button , is_released , \
+		key_state , button_state) \
+		ml_vt100_parser_report_mouse_tracking( (term)->parser , col , row , \
+			button , is_released , key_state , button_state)
 
 #define  ml_term_search_init( term , match)  ml_screen_search_init( (term)->screen , match)
 
