@@ -15,8 +15,6 @@
 #include  <kiklib/kik_file.h>	/* kik_file_unset_cloexec */
 #include  <kiklib/kik_unistd.h>	/* kik_setenv/kik_unsetenv */
 
-#include  "ml_config_proto.h"
-
 
 #define  MAX_TERMS  (MTU * max_terms_multiple)	/* Default MAX_TERMS is 32. */
 #define  MTU        (8 * sizeof(*dead_mask))	/* MAX_TERMS unit */
@@ -29,6 +27,10 @@
 #define  __DEBUG
 #endif
 
+#if  0
+#define  INFINIT_RESTART
+#endif
+
 
 /* --- static variables --- */
 
@@ -186,7 +188,7 @@ ml_term_manager_init(
 	}
 
 	kik_add_sig_child_listener( NULL , sig_child) ;
-	ml_config_proto_init() ;
+	ml_term_init() ;
 
 	return  1 ;
 }
@@ -197,7 +199,7 @@ ml_term_manager_final(void)
 	int  count ;
 
 	kik_remove_sig_child_listener( NULL , sig_child) ;
-	ml_config_proto_final() ;
+	ml_term_final() ;
 
 	for( count = num_of_terms - 1 ; count >= 0 ; count --)
 	{
@@ -229,7 +231,13 @@ ml_set_auto_restart_cmd(
 	)
 {
 #if  ! defined(USE_WIN32API) && ! defined(DEBUG)
-	if( cmd && *cmd)
+	char *  env ;
+
+	if(
+	#ifndef  INFINIT_RESTART
+	    ( ! ( env = getenv( "INHERIT_PTY_LIST")) || *env == '\0') &&
+	#endif
+	    cmd && *cmd)
 	{
 		if( ! auto_restart_cmd)
 		{
@@ -272,6 +280,7 @@ ml_set_auto_restart_cmd(
 
 ml_term_t *
 ml_create_term(
+	const char *  term_type ,
 	u_int  cols ,
 	u_int  rows ,
 	u_int  tab_size ,
@@ -284,11 +293,9 @@ ml_create_term(
 	int  col_size_a ,
 	int  use_char_combining ,
 	int  use_multi_col_char ,
-	int  use_bidi ,
+	int  use_ctl ,
 	ml_bidi_mode_t  bidi_mode ,
 	const char *  bidi_separators ,
-	int  use_ind ,
-	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode ,
@@ -325,20 +332,20 @@ ml_create_term(
 				 * cols + 1 is for redrawing screen by ml_set_pty_winsize() below.
 				 */
 				if( ( pty = ml_pty_new_with( master , slave , child_pid ,
-							cols + 1 , rows)))
+							cols + 1 , rows , 0 , 0)))
 				{
-					if( ( terms[num_of_terms] = ml_term_new( cols , rows ,
-						tab_size , log_size , encoding , is_auto_encoding ,
-						use_auto_detect , logging_vt_seq ,
-						policy , col_size_a , use_char_combining ,
-						use_multi_col_char , use_bidi , bidi_mode ,
-						bidi_separators , use_ind , use_bce ,
+					if( ( terms[num_of_terms] = ml_term_new( term_type ,
+						cols , rows , tab_size , log_size , encoding ,
+						is_auto_encoding , use_auto_detect ,
+						logging_vt_seq , policy , col_size_a ,
+						use_char_combining , use_multi_col_char ,
+						use_ctl , bidi_mode , bidi_separators ,
 						use_dynamic_comb , bs_mode , vertical_mode ,
 						use_local_echo , win_name , icon_name ,
 						alt_color_mode)))
 					{
 						ml_term_plug_pty( terms[num_of_terms++] , pty) ;
-						ml_set_pty_winsize( pty , cols , rows) ;
+						ml_set_pty_winsize( pty , cols , rows , 0 , 0) ;
 
 						continue ;
 					}
@@ -353,7 +360,9 @@ ml_create_term(
 			}
 		}
 
+	#ifdef  INFINIT_RESTART
 		kik_unsetenv( "INHERIT_PTY_LIST") ;
+	#endif
 
 		if( num_of_terms > 0)
 		{
@@ -372,11 +381,11 @@ ml_create_term(
 	 * If sig_child here...
 	 */
 
-	if( ! ( terms[num_of_terms] = ml_term_new( cols , rows , tab_size , log_size , encoding ,
-				is_auto_encoding , use_auto_detect , logging_vt_seq , policy ,
-				col_size_a , use_char_combining ,
-				use_multi_col_char , use_bidi , bidi_mode , bidi_separators ,
-				use_ind , use_bce , use_dynamic_comb , bs_mode , vertical_mode ,
+	if( ! ( terms[num_of_terms] = ml_term_new( term_type , cols , rows , tab_size ,
+				log_size , encoding , is_auto_encoding , use_auto_detect ,
+				logging_vt_seq , policy , col_size_a , use_char_combining ,
+				use_multi_col_char , use_ctl , bidi_mode , bidi_separators ,
+				use_dynamic_comb , bs_mode , vertical_mode ,
 				use_local_echo , win_name , icon_name , alt_color_mode)))
 	{
 		return  NULL ;
