@@ -14,6 +14,7 @@
 #include  "ml_screen.h"
 #include  "ml_char_encoding.h"
 #include  "ml_drcs.h"
+#include  "ml_termcap.h"
 
 
 #define  PTY_WR_BUFFER_SIZE  100
@@ -46,6 +47,8 @@ typedef enum  ml_mouse_report_mode
 	MOUSE_REPORT = 0x1 ,
 	BUTTON_EVENT_MOUSE_REPORT = 0x2 ,
 	ANY_EVENT_MOUSE_REPORT = 0x3 ,
+	LOCATOR_CHARCELL_REPORT = 0x4 ,
+	LOCATOR_PIXEL_REPORT = 0x5 ,
 
 } ml_mouse_report_mode_t ;
 
@@ -59,6 +62,16 @@ typedef enum  ml_extended_mouse_report_mode
 
 } ml_extended_mouse_report_mode_t ;
 
+typedef enum  ml_locator_report_mode
+{
+	LOCATOR_BUTTON_DOWN = 0x1 ,
+	LOCATOR_BUTTON_UP = 0x2 ,
+	LOCATOR_ONESHOT = 0x4 ,
+	LOCATOR_REQUEST = 0x8 ,
+	LOCATOR_FILTER_RECT = 0x10 ,
+
+} ml_locator_report_mode_t ;
+
 typedef enum  ml_alt_color_mode
 {
 	ALT_COLOR_BOLD = 0x1 ,
@@ -101,7 +114,8 @@ typedef struct  ml_xterm_event_listener
 
 	void (*resize)( void * , u_int , u_int) ;	/* called in visual context. */
 	void (*reverse_video)( void * , int) ;		/* called in visual context. */
-	void (*set_mouse_report)( void * , ml_mouse_report_mode_t) ;/* called in visual context. */
+	void (*set_mouse_report)( void *) ;	/* called in visual context. */
+	void (*request_locator)( void *) ;	/* called in visual context. */
 	void (*set_window_name)( void * , u_char *) ;	/* called in logical context. */
 	void (*set_icon_name)( void * , u_char *) ;	/* called in logical context. */
 	void (*bel)( void *) ;				/* called in visual context. */
@@ -111,8 +125,12 @@ typedef struct  ml_xterm_event_listener
 	int (*get_window_size)( void * , u_int * , u_int *) ;	/* called in logical context. */
 	int (*get_rgb)( void * , u_int8_t * , u_int8_t * ,
 			u_int8_t * , ml_color_t) ;		/* called in logical context. */
-	ml_char_t *  (*get_picture_data)( void * , char * , int * , int *) ; /* called in logical context. */
+	ml_char_t *  (*get_picture_data)( void * , char * ,
+			int * , int * , u_int32_t **) ; /* called in logical context. */
+	int (*get_emoji_data)( void * , ml_char_t * , ml_char_t *) ;	/* called in logical context. */
+	void (*show_sixel)( void * , char *) ;		/* called in logical context. */
 	void (*add_frame_to_animation)( void * , char * , int * , int *) ; /* called in logical context. */
+	void (*hide_cursor)( void * , int) ;		/* called in logical context. */
 
 } ml_xterm_event_listener_t ;
 
@@ -135,6 +153,7 @@ typedef struct  ml_config_event_listener
 	void (*set_font)( void * , char * , char * , char * , int) ;
 	void (*get_font)( void * , char * , char * , char * , int) ;
 	void (*set_color)( void * , char * , char * , char * , int) ;
+	void (*get_color)( void * , char * , int) ;
 
 } ml_config_event_listener_t ;
 
@@ -177,8 +196,9 @@ typedef struct  ml_vt100_parser
 	ml_write_buffer_t  w_buf ;
 	
 	ml_pty_ptr_t  pty ;
-	
+
 	ml_screen_t *  screen ;
+	ml_termcap_entry_t *  termcap ;
 
 	mkf_parser_t *  cc_parser ;	/* char code parser */
 	mkf_conv_t *  cc_conv ;		/* char code converter */
@@ -197,10 +217,20 @@ typedef struct  ml_vt100_parser
 	char *  win_name ;
 	char *  icon_name ;
 
+	struct
+	{
+		u_int16_t  top ;
+		u_int16_t  left ;
+		u_int16_t  bottom ;
+		u_int16_t  right ;
+
+	} loc_filter ;
+
 	/* ml_unicode_policy_t */ int8_t  unicode_policy ;
 
 	/* ml_mouse_report_mode_t */ int8_t  mouse_mode ;
 	/* ml_extended_mouse_report_mode_t */ int8_t  ext_mouse_mode ;
+	/* ml_locator_report_mode_t */ int8_t  locator_mode ;
 
 	/* Used for non iso2022 encoding */
 	mkf_charset_t  gl ;
@@ -227,6 +257,7 @@ typedef struct  ml_vt100_parser
 	int8_t  is_app_cursor_keys ;
 	int8_t  is_app_escape ;
 	int8_t  is_bracketed_paste_mode ;
+	int8_t  allow_deccolm ;
 
 	int8_t  want_focus_event ;
 
@@ -238,13 +269,14 @@ typedef struct  ml_vt100_parser
 #endif
 	int8_t  modify_other_keys ;
 
-	int8_t  use_bidi ;
+	int8_t  use_ctl ;
 
 #ifdef  USE_VT52
 	int8_t  is_vt52_mode ;
 #endif
 
 	int8_t  sixel_scrolling ;
+	int8_t  cursor_to_right_of_sixel ;
 	int8_t  yield ;
 
 	int8_t  is_auto_encoding ;
@@ -262,6 +294,8 @@ typedef struct  ml_vt100_parser
 	ml_macro_t *  macros ;
 	u_int  num_of_macros ;
 
+	u_int32_t *  sixel_palette ;
+
 } ml_vt100_parser_t ;
 
 
@@ -281,7 +315,12 @@ void  ml_set_use_scp_full( int  use) ;
 #define  ml_set_use_scp_full(use)  (0)
 #endif
 
-ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen , ml_char_encoding_t  encoding ,
+void  ml_vt100_parser_init(void) ;
+
+void  ml_vt100_parser_final(void) ;
+
+ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen ,
+	ml_termcap_entry_t *  termcap , ml_char_encoding_t  encoding ,
 	int  is_auto_encoding , int  use_auto_detect , int  logging_vt_seq ,
 	ml_unicode_policy_t  policy , u_int  col_size_a ,
 	int  use_char_combining , int  use_multi_col_char ,
@@ -301,11 +340,22 @@ int  ml_parse_vt100_sequence( ml_vt100_parser_t *  vt100_parser) ;
 
 void  ml_reset_pending_vt100_sequence( ml_vt100_parser_t *  vt100_parser) ;
 
+
+int  ml_vt100_parser_write_modified_key( ml_vt100_parser_t *  vt100_parser ,
+	int  key , int  modcode) ;
+
+int  ml_vt100_parser_write_special_key( ml_vt100_parser_t *  vt100_parser ,
+	ml_special_key_t  key , int  modcode , int  is_numlock) ;
+
 /* Must be called in visual context. */
 int  ml_vt100_parser_write_loopback( ml_vt100_parser_t *  vt100_parser ,
 	const u_char *  buf , size_t  len) ;
 
+/* Must be called in visual context. */
+int  ml_vt100_parser_show_message( ml_vt100_parser_t *  vt100_parser , char *  msg) ;
+
 #ifdef  __ANDROID__
+/* Must be called in visual context. */
 int  ml_vt100_parser_preedit( ml_vt100_parser_t *  vt100_parser ,
 	const u_char *  buf , size_t  len) ;
 #endif
@@ -344,28 +394,17 @@ int  ml_init_encoding_conv( ml_vt100_parser_t *  vt100_parser) ;
 
 #define  ml_vt100_parser_get_mouse_report_mode( vt100_parser)  ((vt100_parser)->mouse_mode)
 
-#define  ml_vt100_parser_get_extended_mouse_report_mode( vt100_parser) \
-		((vt100_parser)->ext_mouse_mode)
-
-#define  ml_vt100_parser_is_app_keypad( vt100_parser)  ((vt100_parser)->is_app_keypad)
-
-#define  ml_vt100_parser_is_app_cursor_keys( vt100_parser)  ((vt100_parser)->is_app_cursor_keys)
-
-#define  ml_vt100_parser_is_app_escape( vt100_parser)  ((vt100_parser)->is_app_escape)
-
 #define  ml_vt100_parser_is_bracketed_paste_mode( vt100_parser) \
 		((vt100_parser)->is_bracketed_paste_mode)
 
 #define  ml_vt100_parser_want_focus_event( vt100_parser)  ((vt100_parser)->want_focus_event)
 
-#define  ml_vt100_parser_modify_other_keys( vt100_parser)  ((vt100_parser)->modify_other_keys)
-
 #define  ml_vt100_parser_set_unicode_policy( vt100_parser , policy) \
 		((vt100_parser)->unicode_policy = (policy))
 
 #define  ml_vt100_parser_get_unicode_policy( vt100_parser)  ((vt100_parser)->unicode_policy)
 
-#define  ml_vt100_parser_set_use_bidi( vt100_parser , use)  ((vt100_parser)->use_bidi = (use))
+#define  ml_vt100_parser_set_use_ctl( vt100_parser , use)  ((vt100_parser)->use_ctl = (use))
 
 int  ml_set_auto_detect_encodings( char *  encodings) ;
 
@@ -388,5 +427,9 @@ int  ml_vt100_parser_set_config( ml_vt100_parser_t *  vt100_parser , char *  key
 
 int  ml_vt100_parser_exec_cmd( ml_vt100_parser_t *  vt100_parser , char *  cmd) ;
 
+void  ml_vt100_parser_report_mouse_tracking( ml_vt100_parser_t *  vt100_parser ,
+	int  col , int  row , int  button , int  is_released ,
+	int  key_state , int  button_state) ;
+
 
 #endif
