@@ -15,6 +15,9 @@ typedef struct  sample_sb_view
 
 	GC  gc ;
 
+	unsigned long  black_pixel ;
+	unsigned long  white_pixel ;
+
 	Pixmap  arrow_up ;
 	Pixmap  arrow_up_dent ;
 	Pixmap  arrow_down ;
