@@ -16,12 +16,11 @@
 
 GtkWidget * mc_combo_new( const char *  label_name , char **  item_names ,
 	u_int  item_num , char *  selected_item_name , int  is_readonly ,
-	gint (*callback)(GtkWidget * , gpointer) , gpointer  data) ;
+	GtkWidget **  entry) ;
 
 GtkWidget * mc_combo_new_with_width( const char * label_name, char ** item_names,
 	u_int item_num, char * selected_item_name, int is_readonly,
-	gint (*callback)(GtkWidget *, gpointer), gpointer data,
-	int entry_width);
+	int entry_width, GtkWidget **  entry) ;
 
 
 #endif
