@@ -5,6 +5,7 @@
 #include  "mc_font.h"
 
 #include  <stdio.h>
+#include  <ctype.h>
 #include  <kiklib/kik_str.h>
 #include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
@@ -17,7 +18,10 @@
 
 #include  "mc_combo.h"
 #include  "mc_flags.h"
-#include  "mc_vertical.h"
+#include  "mc_radio.h"
+#include  "mc_char_encoding.h"
+#include  "mc_color.h"
+#include  "mc_unicode_areas.h"
 
 
 #if  0
@@ -59,6 +63,7 @@ static cs_info_t  cs_info_table[] =
 	{ "UNICODE (FULLWIDTH)" , { "iso10646-1" , NULL , NULL , } , } ,
 	{ "UNICODE (ARABIC)" , { "iso10646-1" , NULL , NULL , } , } ,
 	{ "UNICODE (EMOJI)" , { "iso10646-1" , NULL , NULL , } , } ,
+	{ "UNICODE (HANKAKU KANA)" , { "iso10646-1" , NULL , NULL , } , } ,
 
 	{ "DEC_SPECIAL" , { "iso8859-1" , NULL , NULL , } , } ,
 	{ "ISO8859_1" , { "iso8859-1" , NULL , NULL , } , } ,
@@ -162,12 +167,16 @@ static int is_fontsize_changed ;
 static char *  new_fontname_list[sizeof(cs_info_table)/sizeof(cs_info_table[0])] ;
 static int  dont_change_new_fontname_list = 0 ;
 static int  selected_cs = 0 ;	/* 0 = DEFAULT */
+static GtkWidget *  fontcs_entry ;
 static GtkWidget *  fontname_entry ;
 static GtkWidget *  select_font_button ;
 static GtkWidget *  xft_flag ;
 static GtkWidget *  cairo_flag ;
 static GtkWidget *  aa_flag ;
 static GtkWidget *  vcol_flag ;
+static int  dont_change_type_engine ;
+static GtkWidget *  noconv_areas_button ;
+static char *  noconv_areas ;
 
 
 /* --- static functions  --- */
@@ -213,6 +222,10 @@ get_correct_cs(
 	{
 		return  "U+1F000-1F77F" ;	/* Emoji */
 	}
+	else if( idx == 5)
+	{
+		return  "U+FF61-FF9F" ;	/* Hankaku Kana */
+	}
 	else if( idx < sizeof(cs_info_table) / sizeof(cs_info_table[0]))
 	{
 		return  cs_info_table[idx].cs ;
@@ -233,7 +246,7 @@ get_font_file(void)
 		{
 			return "vaafont" ;
 		}
-		else if( mc_is_vertical())
+		else if( mc_radio_get_value( MC_RADIO_VERTICAL_MODE))
 		{
 			return "taafont" ;
 		}
@@ -248,7 +261,7 @@ get_font_file(void)
 		{
 			return "vfont" ;
 		}
-		else if( mc_is_vertical())
+		else if( mc_radio_get_value( MC_RADIO_VERTICAL_MODE))
 		{
 			return "tfont" ;
 		}
@@ -290,13 +303,16 @@ xft_flag_checked(
 	gpointer  data
 	)
 {
-	if( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)))
+	if( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)) &&
+	    ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(cairo_flag)))
 	{
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(aa_flag) , 0) ;
 	}
-	else
+	else if( ! dont_change_type_engine)
 	{
+		dont_change_type_engine = 1 ;
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cairo_flag) , 0) ;
+		dont_change_type_engine = 0 ;
 	}
 
 	reset_fontname_list() ;
@@ -315,13 +331,16 @@ cairo_flag_checked(
 	gpointer  data
 	)
 {
-	if( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)))
+	if( ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)) &&
+	    ! gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(xft_flag)))
 	{
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(aa_flag) , 0) ;
 	}
-	else
+	else if( ! dont_change_type_engine)
 	{
+		dont_change_type_engine = 1 ;
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(xft_flag) , 0) ;
+		dont_change_type_engine = 0 ;
 	}
 
 	reset_fontname_list() ;
@@ -350,6 +369,27 @@ vcol_flag_checked(
 	return  1 ;
 }
 
+static void
+vertical_mode_changed(void)
+{
+	reset_fontname_list() ;
+	gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
+		g_locale_to_utf8(
+			mc_get_font_name( get_font_file() , new_fontsize ,
+						get_correct_cs( selected_cs)) ,
+			-1 , NULL , NULL , NULL) ) ;
+
+	if( mc_radio_get_value( MC_RADIO_VERTICAL_MODE))
+	{
+		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(vcol_flag) , 0) ;
+		gtk_widget_set_sensitive( vcol_flag , 0) ;
+	}
+	else
+	{
+		gtk_widget_set_sensitive( vcol_flag , 1) ;
+	}
+}
+
 static gint
 fontsize_selected(
 	GtkWidget *  widget ,
@@ -373,13 +413,137 @@ fontsize_selected(
 	return  1 ;
 }
 
+static void
+fontcs_changed(void)
+{
+	dont_change_new_fontname_list = 1 ;
+
+	if( new_fontname_list[selected_cs])
+	{
+		gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
+			new_fontname_list[selected_cs]) ;
+	}
+	else
+	{
+		gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
+			g_locale_to_utf8(
+				mc_get_font_name( get_font_file() ,
+					new_fontsize ,
+					get_correct_cs(selected_cs)) ,
+				-1 , NULL , NULL , NULL) ) ;
+	}
+
+	dont_change_new_fontname_list = 0 ;
+}
+
+/* compare two encoding names, returns non-zero if equal
+ */
+static int
+compare(
+	const char *  e1 ,
+	const char *  e2
+	)
+{
+	while(1)
+	{
+		/* ')' is for "auto (currently EUC-JP)" */
+		if( *e1 == '-' || *e1 == '_' || *e1 == ')')
+		{
+			e1++ ;
+			continue ;
+		}
+		if( *e2 == '-' || *e2 == '_' || *e2 == ')')
+		{
+			e2++ ;
+			continue ;
+		}
+		if( (*e1 == 0 || *e1 == ' ') && (*e2 == 0 || *e2 == ' '))
+		{
+			return  1 ;
+		}
+		if( toupper(*e1) != toupper(*e2))
+		{
+			return  0 ;
+		}
+		e1++ ;
+		e2++ ;
+	}
+}
+
+static void
+fontcs_map(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	char *  encoding ;
+	int  count ;
+
+	encoding = mc_get_char_encoding() ;
+
+	if( ( strstr( encoding , "UTF") && mc_radio_get_value(MC_RADIO_FONT_POLICY) != 2) ||
+	    mc_radio_get_value(MC_RADIO_FONT_POLICY) == 1)
+	{
+		if( selected_cs == 1)
+		{
+			return ;
+		}
+
+		selected_cs = 1 ;	/* UNICODE */
+	}
+	else
+	{
+		for( count = 0 ; count < sizeof(cs_info_table)/sizeof(cs_info_table[0]) ; count++)
+		{
+			if( strcmp( cs_info_table[count].cs , "JISX0201_KATA") == 0)
+			{
+				break ;
+			}
+			else if( compare( encoding , cs_info_table[count].cs))
+			{
+				if( selected_cs == count)
+				{
+					return ;
+				}
+
+				selected_cs = count ;
+
+				goto  update_cs ;
+			}
+		}
+
+		if( selected_cs == 0)
+		{
+			return ;
+		}
+
+		selected_cs = 0 ;	/* DEFAULT */
+	}
+
+update_cs:
+	gtk_entry_set_text( GTK_ENTRY(widget) , cs_info_table[selected_cs].cs) ;
+	fontcs_changed() ;
+}
+
+static void
+font_policy_changed(void)
+{
+	fontcs_map( fontcs_entry , NULL) ;
+
+	if( noconv_areas_button)
+	{
+		gtk_widget_set_sensitive( noconv_areas_button ,
+			( mc_radio_get_value( MC_RADIO_FONT_POLICY) == 2)) ;
+	}
+}
+
 static gint
 fontcs_selected(
 	GtkWidget *  widget ,
 	gpointer  data
 	)
 {
-	char *  cs ;
+	const char *  cs ;
 	int  count ;
 
 	cs = gtk_entry_get_text(GTK_ENTRY(widget)) ;
@@ -398,24 +562,7 @@ fontcs_selected(
 
 				selected_cs = count ;
 
-				dont_change_new_fontname_list = 1 ;
-
-				if( new_fontname_list[selected_cs])
-				{
-					gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
-						new_fontname_list[selected_cs]) ;
-				}
-				else
-				{
-					gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
-						g_locale_to_utf8(
-							mc_get_font_name( get_font_file() ,
-								new_fontsize ,
-								get_correct_cs(selected_cs)) ,
-							-1 , NULL , NULL , NULL) ) ;
-				}
-
-				dont_change_new_fontname_list = 0 ;
+				fontcs_changed() ;
 
 			#if  0
 				kik_debug_printf( "After: cs %s fontname %s\n" ,
@@ -430,7 +577,6 @@ fontcs_selected(
 	return  0 ;
 }
 
-
 #if  ! defined(USE_WIN32GUI) && ! defined(G_PLATFORM_WIN32) && ! GTK_CHECK_VERSION(2,90,0)
 
 static gchar *
@@ -504,7 +650,7 @@ set_current_font_name(
 	GtkWidget *  dialog
 	)
 {
-	gchar *  font_name ;
+	const gchar *  font_name ;
 
 	font_name = gtk_entry_get_text(GTK_ENTRY(fontname_entry)) ;
 	if( font_name && *font_name)
@@ -622,7 +768,7 @@ my_gtk_font_selection_dialog_get_font_name(
 
 static char *
 get_gtk_font_name(
-	char *  font_name
+	const char *  font_name
 	)
 {
 	int  count ;
@@ -659,7 +805,7 @@ fontname_entry_edit(
 
 	if( ! dont_change_new_fontname_list)
 	{
-		char *  name ;
+		const char *  name ;
 
 		if( ! ( name = gtk_entry_get_text(GTK_ENTRY(fontname_entry))))
 		{
@@ -743,6 +889,34 @@ select_font(
 	}
 }
 
+static void
+edit_noconv_areas(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	char *  cur_areas ;
+	char *  new_areas ;
+
+	if( noconv_areas)
+	{
+		cur_areas = strdup( noconv_areas) ;
+	}
+	else
+	{
+		cur_areas = mc_get_str_value( "unicode_noconv_areas") ;
+	}
+
+	if( ( new_areas = mc_get_unicode_areas( cur_areas)) &&
+	    kik_compare_str( noconv_areas , new_areas) != 0)
+	{
+		free( noconv_areas) ;
+		noconv_areas = new_areas ;
+	}
+
+	free( cur_areas) ;
+}
+
 
 /* --- global functions --- */
 
@@ -751,7 +925,11 @@ mc_font_config_widget_new(void)
 {
 	GtkWidget *  vbox ;
 	GtkWidget *  hbox ;
+	GtkWidget *  fgcolor ;
 	GtkWidget *  combo ;
+	GtkWidget *  entry ;
+	GtkWidget *  radio ;
+	GtkWidget *  label ;
 	char *  fontlist[] =
 	{
 		"6" , "7" , "8" , "9" , "10" ,
@@ -763,67 +941,80 @@ mc_font_config_widget_new(void)
 	new_fontsize = strdup( old_fontsize = mc_get_str_value( "fontsize")) ;
 	is_fontsize_changed = 0;
 
-	vbox = gtk_vbox_new(FALSE , 0) ;
+	vbox = gtk_vbox_new( FALSE , 0) ;
 
-	combo = mc_combo_new_with_width(_("Font size (pixels)"), fontlist ,
-		sizeof(fontlist) / sizeof(fontlist[0]), new_fontsize , 1 ,
-		fontsize_selected, NULL, 80) ;
-	gtk_box_pack_start(GTK_BOX(vbox) , combo , TRUE , TRUE , 0) ;
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , FALSE , FALSE , 0) ;
+
+	combo = mc_combo_new_with_width(_("Font size (pixels)") , fontlist ,
+		sizeof(fontlist) / sizeof(fontlist[0]) , new_fontsize , 1 ,
+		50 , &entry) ;
+	g_signal_connect( entry , "changed" , G_CALLBACK(fontsize_selected) , NULL) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , combo , FALSE , FALSE , 0) ;
 #if  GTK_CHECK_VERSION(2,12,0)
-	gtk_widget_set_tooltip_text(combo ,
+	gtk_widget_set_tooltip_text( combo ,
 		"If you change fonts from \"Select\" button, "
 		"it is not recommended to change font size here.") ;
 #endif
 
+	fgcolor = mc_color_config_widget_new( MC_COLOR_FG) ;
+	gtk_widget_show( fgcolor) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , fgcolor , FALSE , FALSE , 5) ;
 
-	hbox = gtk_hbox_new(FALSE , 0) ;
-	gtk_widget_show(hbox) ;
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
 
 	xft_flag = mc_flag_config_widget_new( MC_FLAG_XFT) ;
 	gtk_widget_show( xft_flag) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , xft_flag , TRUE , TRUE , 0) ;
-	g_signal_connect(xft_flag , "toggled" ,
+	gtk_box_pack_start( GTK_BOX(hbox) , xft_flag , FALSE , FALSE , 0) ;
+	g_signal_connect( xft_flag , "toggled" ,
 		G_CALLBACK(xft_flag_checked) , NULL) ;
 	if( mc_gui_is_win32())
 	{
-		gtk_widget_set_sensitive(xft_flag, 0);
+		gtk_widget_set_sensitive( xft_flag , 0) ;
 	}
 	
 	cairo_flag = mc_flag_config_widget_new( MC_FLAG_CAIRO) ;
 	gtk_widget_show( cairo_flag) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , cairo_flag , TRUE , TRUE , 0) ;
-	g_signal_connect(cairo_flag , "toggled" ,
+	gtk_box_pack_start( GTK_BOX(hbox) , cairo_flag , FALSE , FALSE , 0) ;
+	g_signal_connect( cairo_flag , "toggled" ,
 		G_CALLBACK(cairo_flag_checked) , NULL) ;
 	if( mc_gui_is_win32())
 	{
-		gtk_widget_set_sensitive(cairo_flag, 0);
+		gtk_widget_set_sensitive( cairo_flag , 0) ;
 	}
 
 	aa_flag = mc_flag_config_widget_new( MC_FLAG_AA) ;
-#if  ! defined(USE_WIN32GUI)
-	gtk_widget_set_sensitive(aa_flag, 0) ;
-#endif
 	gtk_widget_show( aa_flag) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , aa_flag , TRUE , TRUE , 0) ;
-	g_signal_connect(aa_flag , "toggled" ,
+	gtk_box_pack_start( GTK_BOX(hbox) , aa_flag , FALSE , FALSE , 0) ;
+	g_signal_connect( aa_flag , "toggled" ,
 		G_CALLBACK(aa_flag_checked) , NULL) ;
+	if( mc_gui_is_win32())
+	{
+		gtk_widget_set_sensitive( aa_flag , 0) ;
+	}
 	
 	vcol_flag = mc_flag_config_widget_new( MC_FLAG_VCOL) ;
 	gtk_widget_show( vcol_flag) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , vcol_flag , TRUE , TRUE , 0) ;
-	g_signal_connect(vcol_flag , "toggled" ,
+	gtk_box_pack_start( GTK_BOX(hbox) , vcol_flag , FALSE , FALSE , 0) ;
+	g_signal_connect( vcol_flag , "toggled" ,
 		G_CALLBACK(vcol_flag_checked) , NULL) ;
-
-	gtk_box_pack_start(GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , FALSE , FALSE , 0) ;
 
 
-	hbox = mc_vertical_config_widget_new() ;
-	gtk_widget_show(hbox) ;
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+	radio = mc_radio_config_widget_new( MC_RADIO_VERTICAL_MODE) ;
+	gtk_widget_show( radio) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , radio , FALSE , FALSE , 0) ;
+	mc_radio_set_callback( MC_RADIO_VERTICAL_MODE , vertical_mode_changed) ;
+	if( mc_radio_get_value( MC_RADIO_VERTICAL_MODE))
+	{
+		gtk_widget_set_sensitive( vcol_flag , 0) ;
+	}
 
 
-	hbox = gtk_hbox_new(FALSE , 0) ;
-	gtk_widget_show(hbox) ;
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
 
 	if( ( cslist = alloca( sizeof(char*) * sizeof(cs_info_table) / sizeof(cs_info_table[0]))))
 	{
@@ -834,32 +1025,59 @@ mc_font_config_widget_new(void)
 		{
 			cslist[count] = cs_info_table[count].cs ;
 		}
-		
-		combo = mc_combo_new_with_width(_("Font name") , cslist , count ,
-				cslist[selected_cs] , 1 ,
-				fontcs_selected, NULL, 110) ;
-		gtk_box_pack_start(GTK_BOX(hbox) , combo , TRUE , TRUE , 0) ;
+
+		combo = mc_combo_new_with_width( _("Font name") , cslist , count ,
+				cslist[selected_cs] , 1 , 90 , &fontcs_entry) ;
+		g_signal_connect( fontcs_entry , "changed" , G_CALLBACK(fontcs_selected) , NULL) ;
+		g_signal_connect( fontcs_entry , "map" , G_CALLBACK(fontcs_map) , NULL) ;
+		gtk_box_pack_start( GTK_BOX(hbox) , combo , FALSE , FALSE , 0) ;
 	}
 
 	fontname_entry = gtk_entry_new() ;
-	gtk_entry_set_text(GTK_ENTRY(fontname_entry),
+	gtk_entry_set_text( GTK_ENTRY(fontname_entry) ,
 		g_locale_to_utf8(
 			mc_get_font_name( get_font_file() , new_fontsize ,
-					get_correct_cs(selected_cs)) ,
+					get_correct_cs( selected_cs)) ,
 			-1 , NULL , NULL , NULL) ) ;
-	gtk_widget_show(fontname_entry) ;
-	gtk_widget_set_size_request(fontname_entry , 100 , -1) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , fontname_entry , TRUE , TRUE , 1) ;
-	g_signal_connect(fontname_entry , "changed" ,
+	gtk_widget_show( fontname_entry) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , fontname_entry , TRUE , TRUE , 1) ;
+	g_signal_connect( fontname_entry , "changed" ,
 		G_CALLBACK(fontname_entry_edit) , NULL) ;
 
-	select_font_button = gtk_button_new_with_label( _(" Select ")) ;
+	select_font_button = gtk_button_new_with_label( _("Select")) ;
 	gtk_widget_show( select_font_button) ;
-	gtk_box_pack_start(GTK_BOX(hbox) , select_font_button , TRUE , TRUE , 1) ;
-	g_signal_connect(select_font_button , "clicked" ,
+	gtk_box_pack_start( GTK_BOX(hbox) , select_font_button , FALSE , FALSE , 1) ;
+	g_signal_connect( select_font_button , "clicked" ,
 		G_CALLBACK(select_font) , NULL) ;
 
-	gtk_box_pack_start(GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
+
+	radio = mc_radio_config_widget_new( MC_RADIO_FONT_POLICY) ;
+	gtk_widget_show( radio) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , radio , FALSE , FALSE , 0) ;
+	mc_radio_set_callback( MC_RADIO_FONT_POLICY , font_policy_changed) ;
+
+	hbox = gtk_hbox_new( FALSE , 0) ;
+	gtk_widget_show( hbox) ;
+
+	label = gtk_label_new( _("Unicode areas you won't convert to other charsets")) ;
+	gtk_widget_show( label) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , label , FALSE , FALSE , 5) ;
+	noconv_areas_button = gtk_button_new_with_label( _(" Edit ")) ;
+	gtk_widget_show( noconv_areas_button) ;
+	gtk_box_pack_start( GTK_BOX(hbox) , noconv_areas_button , FALSE , FALSE , 0) ;
+	g_signal_connect( noconv_areas_button , "clicked" , G_CALLBACK(edit_noconv_areas) , NULL) ;
+	if( mc_radio_get_value( MC_RADIO_FONT_POLICY) != 2)
+	{
+		gtk_widget_set_sensitive( noconv_areas_button , 0) ;
+	}
+
+	gtk_box_pack_start( GTK_BOX(vbox) , hbox , FALSE , FALSE , 0) ;
+
+	radio = mc_radio_config_widget_new( MC_RADIO_BOX_DRAWING) ;
+	gtk_widget_show( radio) ;
+	gtk_box_pack_start( GTK_BOX(vbox) , radio , FALSE , FALSE , 0) ;
+
 
 	return  vbox ;
 }
@@ -881,10 +1099,18 @@ mc_update_font_misc(void)
 	 * invalid in some environments.
 	 */
 	mc_update_flag_mode( MC_FLAG_AA) ;
+	mc_update_radio( MC_RADIO_VERTICAL_MODE) ;
+	/* vcol is forcibly disabled in vertical mode, so update after vertical mode. */
 	mc_update_flag_mode( MC_FLAG_VCOL) ;
 	mc_update_flag_mode( MC_FLAG_XFT) ;
 	mc_update_flag_mode( MC_FLAG_CAIRO) ;
-	mc_update_vertical_mode() ;
+	mc_update_radio( MC_RADIO_BOX_DRAWING) ;
+	mc_update_radio( MC_RADIO_FONT_POLICY) ;
+
+	if( noconv_areas)
+	{
+		mc_set_str_value( "unicode_noconv_areas" , noconv_areas) ;
+	}
 }
 
 void
