@@ -93,6 +93,66 @@ is_im_plugin(char *file_name)
 	return 0;
 }
 
+#ifdef  USE_WIN32API
+im_info_t *
+get_kbd_info(char *locale, char *encoding)
+{
+	im_info_t *result;
+
+	if (!(result = malloc(sizeof(im_info_t)))) {
+		return NULL ;
+	}
+
+	result->num_of_args = 14;
+
+	if (!(result->args = malloc(sizeof(char*) * result->num_of_args))) {
+		free(result);
+		return NULL;
+	}
+
+	if (!(result->readable_args = malloc(sizeof(char*) * result->num_of_args))) {
+		free(result->args);
+		free(result);
+		return NULL;
+	}
+
+	result->readable_args[0] = strdup("Unknown");
+	result->readable_args[1] = strdup("Arabic");
+	result->readable_args[2] = strdup("Hebrew");
+	result->readable_args[3] = strdup("Indic (ASSAMESE)");
+	result->readable_args[4] = strdup("Indic (BENGALI)");
+	result->readable_args[5] = strdup("Indic (GUJARATI)");
+	result->readable_args[6] = strdup("Indic (HINDI)");
+	result->readable_args[7] = strdup("Indic (KANNADA)");
+	result->readable_args[8] = strdup("Indic (MALAYALAM)");
+	result->readable_args[9] = strdup("Indic (ORIYA)");
+	result->readable_args[10] = strdup("Indic (PUNJABI)");
+	result->readable_args[11] = strdup("Indic (ROMAN)");
+	result->readable_args[12] = strdup("Indic (TAMIL)");
+	result->readable_args[13] = strdup("Indic (TELUGU)");
+
+	result->args[0] = strdup("");
+	result->args[1] = strdup("arabic");
+	result->args[2] = strdup("hebrew");
+	result->args[3] = strdup("isciiassamese");
+	result->args[4] = strdup("isciibengali");
+	result->args[5] = strdup("isciigujarati");
+	result->args[6] = strdup("isciihindi");
+	result->args[7] = strdup("isciikannada");
+	result->args[8] = strdup("isciimalayalam");
+	result->args[9] = strdup("isciioriya");
+	result->args[10] = strdup("isciipunjabi");
+	result->args[11] = strdup("isciiroman");
+	result->args[12] = strdup("isciitamil");
+	result->args[13] = strdup("isciitelugu");
+
+	result->id = strdup("kbd");
+	result->name = strdup("keyboard");
+
+	return  result;
+}
+#endif
+
 static int
 get_im_info(char *locale, char *encoding)
 {
@@ -102,9 +162,17 @@ get_im_info(char *locale, char *encoding)
 
 	if ((dir = opendir(IM_DIR))) {
 		im_dir_path = IM_DIR;
+#if 0
 	} else if ((dir = opendir("."))) {
 		im_dir_path = "";
+#endif
 	} else {
+	#ifdef  USE_WIN32API
+		if ((im_info_table[num_of_info] = get_kbd_info(locale, encoding))) {
+			num_of_info++;
+			return 1;
+		}
+	#endif
 		return 0;
 	}
 
@@ -239,6 +307,7 @@ xim_widget_new(const char *xim_name, const char *xim_locale, const char *cur_loc
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *combo;
+	GtkWidget *combo_entry;
 	int count;
 
 	default_xim_name = mc_get_str_value("default_xim_name");
@@ -304,7 +373,8 @@ xim_widget_new(const char *xim_name, const char *xim_locale, const char *cur_loc
 	snprintf(selected_xim_name, STR_LEN,
 		 "%s", xim_name ? xim_name : xim_auto_str);
 	combo = mc_combo_new(_("XIM Server"), xims, num_of_xims,
-			     selected_xim_name, 0, xim_selected, entry);
+			     selected_xim_name, 0, &combo_entry);
+	g_signal_connect(combo_entry, "changed", G_CALLBACK(xim_selected), entry);
 
 	label = gtk_label_new(_("XIM locale"));
 
@@ -352,6 +422,7 @@ static GtkWidget *
 im_widget_new(int nth_im, const char *value, char *locale)
 {
 	GtkWidget *combo;
+	GtkWidget *entry;
 	im_info_t *info;
 	int i;
 	int selected = 0;
@@ -392,7 +463,8 @@ im_widget_new(int nth_im, const char *value, char *locale)
 
 	combo = mc_combo_new(_("Option"), info->readable_args,
 			     info->num_of_args, info->readable_args[selected],
-			     1, im_selected, NULL);
+			     1, &entry);
+	g_signal_connect(entry, "changed", G_CALLBACK(im_selected), NULL);
 
 	return combo;
 }
@@ -404,10 +476,10 @@ static gint
 button_xim_checked(GtkWidget *widget, gpointer data)
 {
 	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget))) {
-		gtk_widget_show(GTK_WIDGET(data));
+		if (data) gtk_widget_show(GTK_WIDGET(data));
 		im_type = IM_XIM;
 	} else {
-		gtk_widget_hide(GTK_WIDGET(data));
+		if (data) gtk_widget_hide(GTK_WIDGET(data));
 	}
 
 	is_changed = 1;
@@ -494,7 +566,12 @@ mc_im_config_widget_new(void)
 	}
 	cur_im_type = im_type ;
 
-	xim = xim_widget_new(xim_name, xim_locale, cur_locale);
+	if (mc_gui_is_win32()) {
+		xim = NULL;
+		if (im_type == IM_NONE) im_type = IM_XIM;
+	} else {
+		xim = xim_widget_new(xim_name, xim_locale, cur_locale);
+	}
 	for (i = 0; i < num_of_info; i++) {
 		if (index == i)
 			im_opt_widget[i] = im_widget_new(i, value, cur_locale);
@@ -513,7 +590,7 @@ mc_im_config_widget_new(void)
 	gtk_widget_show(vbox);
 
 	hbox = gtk_hbox_new(FALSE, 5);
-	radio = gtk_radio_button_new_with_label(NULL, "XIM");
+	radio = gtk_radio_button_new_with_label(NULL, xim ? "XIM" : "MS IME");
 	g_signal_connect(radio, "toggled",
 			   G_CALLBACK(button_xim_checked), xim);
 	gtk_widget_show(GTK_WIDGET(radio));
@@ -535,33 +612,35 @@ mc_im_config_widget_new(void)
 						    TRUE);
 	}
 
-	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio));
-	radio = gtk_radio_button_new_with_label(group, _("None"));
-	g_signal_connect(radio, "toggled",
+	if (!mc_gui_is_win32()) {
+		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio));
+		radio = gtk_radio_button_new_with_label(group, _("None"));
+		g_signal_connect(radio, "toggled",
 			   G_CALLBACK(button_im_checked), NULL);
-	gtk_widget_show(GTK_WIDGET(radio));
-	gtk_box_pack_start(GTK_BOX(hbox), radio, FALSE, FALSE, 0);
-	if (im_type == IM_NONE)
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio) , TRUE);
+		gtk_widget_show(GTK_WIDGET(radio));
+		gtk_box_pack_start(GTK_BOX(hbox), radio, FALSE, FALSE, 0);
+		if (im_type == IM_NONE)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio) , TRUE);
+	}
 
 	gtk_widget_show(GTK_WIDGET(hbox));
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	switch (im_type) {
 	case IM_XIM:
-		gtk_widget_show(xim);
+		if (xim) gtk_widget_show(xim);
 		for (i = 0; i < num_of_info; i++)
 			if (im_opt_widget[i])
 				gtk_widget_hide(im_opt_widget[i]);
 		break;
 	case IM_NONE:
-		gtk_widget_hide(xim);
+		if (xim) gtk_widget_hide(xim);
 		for (i = 0; i < num_of_info; i++)
 			if (im_opt_widget[i])
 				gtk_widget_hide(im_opt_widget[i]);
 		break;
 	default: /* OTHER */
-		gtk_widget_hide(xim);
+		if (xim) gtk_widget_hide(xim);
 		for (i = 0; i < num_of_info; i++) {
 			if (!im_opt_widget[i])
 				continue;
@@ -574,7 +653,7 @@ mc_im_config_widget_new(void)
 		break;
 	}
 
-	gtk_box_pack_start(GTK_BOX(vbox), xim, TRUE, TRUE, 0);
+	if (xim) gtk_box_pack_start(GTK_BOX(vbox), xim, TRUE, TRUE, 0);
 	for (i = 0; i < num_of_info; i++)
 		if (im_opt_widget[i])
 			gtk_box_pack_start(GTK_BOX(vbox), im_opt_widget[i],
