@@ -198,24 +198,35 @@ typedef struct
 {
 	char *  file ;
 
-	int32_t  format ;
+	int32_t  format ;	/* XXX (fontsize|FONT_BOLD|FONT_ITALIC) on freetype. */
 
 	int32_t  num_of_glyphs ;
 	unsigned char *  glyphs ;
-	int32_t *  glyph_offsets ;
-	int32_t  glyph_width_bytes ;
 
-	int16_t  min_char_or_byte2 ;
-	int16_t  max_char_or_byte2 ;
-	int16_t  min_byte1 ;
-	int16_t  max_byte1 ;
-	int16_t *  glyph_indeces ;
+	int32_t  glyph_width_bytes ;
 
 	unsigned char  width ;
 	unsigned char  width_full ;
 	unsigned char  height ;
 	unsigned char  ascent ;
 
+	int16_t *  glyph_indeces ;
+
+	/* for pcf */
+	int16_t  min_char_or_byte2 ;
+	int16_t  max_char_or_byte2 ;
+	int16_t  min_byte1 ;
+	int16_t  max_byte1 ;
+	int32_t *  glyph_offsets ;
+
+#ifdef  USE_FREETYPE
+	/* for freetype */
+	void *  face ;
+	u_int32_t  num_of_indeces ;
+	u_int32_t  glyph_size ;
+	int  is_aa ;
+#endif
+
 	unsigned int  ref_count ;
 
 } XFontStruct ;
@@ -577,7 +588,7 @@ typedef int XFontSet ;	/* dummy */
 
 #endif	/* FreeBSD/Linux/NetBSD */
 
-#define XK_ISO_Level3_Lock	0xffa3	/* dummy */
+#define XK_ISO_Left_Tab	0xffa3	/* dummy */
 
 
 /* Same as definition in X11/X.h */
@@ -601,6 +612,7 @@ typedef struct
 
 /* Same as definition in X11/cursorfont.h */
 #define XC_xterm 152
+#define XC_left_ptr 68
 
 /* Same as definition in X11/Xutil.h */
 #define NoValue         0x0000
@@ -613,10 +625,6 @@ typedef struct
 #define YNegative       0x0020
 
 
-#define  XC_sb_v_double_arrow  0
-#define  XC_left_ptr  0
-
-
 int  XParseGeometry( char *  str , int *  x , int *  y ,
 	unsigned int  *  width , unsigned int  *  height) ;
 
