@@ -187,6 +187,33 @@ static struct cursor_shape
 
 /* --- static functions --- */
 
+static inline x_window_t *
+get_window_intern(
+	x_window_t *  win ,
+	int  x ,
+	int  y
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		x_window_t *  child ;
+
+		if( ( child = win->children[count])->is_mapped)
+		{
+			if( child->x <= x && x < child->x + ACTUAL_WIDTH(child) &&
+			    child->y <= y && y < child->y + ACTUAL_HEIGHT(child))
+			{
+				return  get_window_intern( child ,
+						x - child->x , y - child->y) ;
+			}
+		}
+	}
+
+	return  win ;
+}
+
 /*
  * _disp.roots[1] is ignored.
  * x and y are rotated values.
@@ -197,22 +224,7 @@ get_window(
 	int  y		/* Y in display */
 	)
 {
-	x_window_t *  win ;
-	u_int  count ;
-
-	for( count = 0 ; count < _disp.roots[0]->num_of_children ; count++)
-	{
-		if( ( win = _disp.roots[0]->children[count])->is_mapped)
-		{
-			if( win->x <= x && x < win->x + ACTUAL_WIDTH(win) &&
-			    win->y <= y && y < win->y + ACTUAL_HEIGHT(win))
-			{
-				return  win ;
-			}
-		}
-	}
-
-	return  _disp.roots[0] ;
+	return  get_window_intern( _disp.roots[0] , x , y) ;
 }
 
 static inline u_char *
@@ -841,6 +853,12 @@ expose_window(
 	u_int  height
 	)
 {
+	if( x + width <= win->x || win->x + ACTUAL_WIDTH(win) < x ||
+	    y + height <= win->y || win->y + ACTUAL_HEIGHT(win) < y)
+	{
+		return ;
+	}
+
 	if( x < win->x + win->hmargin || y < win->y + win->vmargin ||
 	    x - win->x + width > win->hmargin + win->width ||
 	    y - win->y + height > win->vmargin + win->height)
@@ -850,8 +868,27 @@ expose_window(
 
 	if( win->window_exposed)
 	{
-		(*win->window_exposed)( win , x - win->x - win->hmargin ,
-			y - win->y - win->vmargin , width , height) ;
+		if( x < win->x + win->hmargin)
+		{
+			width -= (win->x + win->hmargin - x) ;
+			x = 0 ;
+		}
+		else
+		{
+			x -= (win->x + win->hmargin) ;
+		}
+
+		if( y < win->y + win->vmargin)
+		{
+			height -= (win->y + win->vmargin - y) ;
+			y = 0 ;
+		}
+		else
+		{
+			y -= (win->y + win->vmargin) ;
+		}
+
+		(*win->window_exposed)( win , x , y , width , height) ;
 	}
 }
 
@@ -863,11 +900,8 @@ expose_display(
 	u_int  height
 	)
 {
-	x_window_t *  win1 ;
-	x_window_t *  win2 ;
-
-	win1 = get_window( x , y) ;
-	expose_window( win1 , x , y , width , height) ;
+	u_int  count ;
+	x_window_t *  kbd ;	/* maybe software keyboard */
 
 	/*
 	 * XXX
@@ -886,9 +920,16 @@ expose_display(
 		height = _disp.height - y ;
 	}
 
-	if( ( win2 = get_window( x + width - 1 , y + height - 1)) != win1)
+	expose_window( _disp.roots[0] , x , y , width , height) ;
+
+	for( count = 0 ; count < _disp.roots[0]->num_of_children ; count++)
 	{
-		expose_window( win2 , x , y , width , height) ;
+		expose_window( _disp.roots[0]->children[count] , x , y , width , height) ;
+	}
+
+	if( ( kbd = x_is_virtual_kbd_area( y + height - 1)))
+	{
+		expose_window( kbd , x , y , width , height) ;
 	}
 }
 
@@ -994,7 +1035,7 @@ check_virtual_kbd(
 	XKeyEvent  kev ;
 	int  ret ;
 
-	if( rotate_display || ! ( ret = x_is_virtual_kbd_event( &_disp , bev)))
+	if( ! ( ret = x_is_virtual_kbd_event( &_disp , bev)))
 	{
 		return  0 ;
 	}
@@ -1444,7 +1485,7 @@ x_display_open(
 		{
 			u_int  tmp ;
 
-			if( _display.bytes_per_pixel < 2)
+			if( _display.pixels_per_byte > 1)
 			{
 				rotate_display = 0 ;
 				rotate_mouse_cursor_shape() ;
@@ -1764,6 +1805,8 @@ x_display_set_use_ansi_colors(
 	use_ansi_colors = use ;
 }
 
+#if  defined(__NetBSD__) || defined(__OpenBSD__)
+
 void
 x_display_enable_to_change_cmap(
 	int  flag
@@ -1853,30 +1896,23 @@ x_display_set_cmap(
 	}
 }
 
+#endif
+
 void
 x_display_rotate(
 	int  rotate	/* 1: clockwise, -1: counterclockwise */
 	)
 {
-	if( rotate == rotate_display)
+	if( rotate == rotate_display ||
+	    /* rotate is available for 8 or more bpp */
+	    _display.pixels_per_byte > 1)
 	{
 		return ;
 	}
 
-	if( rotate)
+	if( DISP_IS_INITED)
 	{
-		if( DISP_IS_INITED)
-		{
-			if( _display.bytes_per_pixel < 2)
-			{
-				/* rotate is available for 16 or more bpp */
-
-				return ;
-			}
-
-			/* virual kbd is unavailable on the rotated display. */
-			x_virtual_kbd_hide() ;
-		}
+		x_virtual_kbd_hide() ;
 	}
 
 	if( rotate_display + rotate != 0)
@@ -2005,7 +2041,15 @@ x_display_put_image(
 
 		fb = get_fb( x , y) ;
 
-		if( _display.bytes_per_pixel == 2)
+		if( _display.bytes_per_pixel == 1)
+		{
+			for( count = 0 ; count < size ; count++)
+			{
+				*fb = image[count] ;
+				fb += line_length ;
+			}
+		}
+		else if( _display.bytes_per_pixel == 2)
 		{
 			size /= 2 ;
 			for( count = 0 ; count < size ; count++)
@@ -2253,6 +2297,24 @@ x_display_fill_with(
 	}
 	else
 	{
+		if( rotate_display)
+		{
+			u_int  tmp ;
+
+			if( rotate_display > 0)
+			{
+				fb = get_fb( _disp.height - y - height , x) ;
+			}
+			else /* if( rotate_display < 0) */
+			{
+				fb = get_fb( y , _disp.width - x - width) ;
+			}
+
+			tmp = width ;
+			width = height ;
+			height = tmp ;
+		}
+
 		if( ! ( buf = alloca( width)))
 		{
 			return ;
