@@ -21,5 +21,11 @@ u_char *  x_get_bitmap( XFontStruct *  xfont , u_char *  ch , size_t  len) ;
 #define  x_get_bitmap_cell( bitmap_line , x) \
 	( (bitmap_line)[(x) / 8] & (1 << (8 - ((x) & 7) - 1)))
 
+/*
+ * !!! Available only if xfont->is_aa is true !!!
+ * (xfont)->width_full == (xfont)->glyph_width_bytes / 3
+ */
+#define  x_get_bitmap_width( xfont)  (xfont)->width_full
+
 
 #endif
