@@ -29,7 +29,7 @@
 #endif
 
 #if  1
-#define  EMBEDDING_SIXEL
+#define  BUILTIN_SIXEL
 #endif
 
 
@@ -203,7 +203,7 @@ modify_pixmap(
 	}
 }
 
-#ifdef  EMBEDDING_SIXEL
+#ifdef  BUILTIN_SIXEL
 
 #include  <string.h>
 #include  <kiklib/kik_util.h>
@@ -346,7 +346,7 @@ error:
 #define  CARD_HEAD_SIZE  0
 #include  "../../common/c_sixel.c"
 
-#endif	/* EMBEDDING_SIXEL */
+#endif	/* BUILTIN_SIXEL */
 
 /* For old machines (not to use mlimgloader) */
 #if  (defined(__NetBSD__) || defined(__OpenBSD__)) && ! defined(USE_GRF)
@@ -367,14 +367,17 @@ load_sixel_with_mask_from_file_1bpp(
 	int  x ;
 	int  y ;
 	u_char *  src ;
+#if  0
 	u_char *  dst ;
+#endif
 
-	if( ! strstr( path , ".six") || ! ( *pixmap = calloc( 1 , sizeof(**pixmap))))
+	if( strcasecmp( path + strlen(path) - 4 , ".six") != 0 ||
+	    ! ( *pixmap = calloc( 1 , sizeof(**pixmap))))
 	{
 		return  0 ;
 	}
 
-	if( ! ( src = (*pixmap)->image = load_sixel_from_file_1bpp( path ,
+	if( ! ( (*pixmap)->image = load_sixel_from_file_1bpp( path ,
 						&(*pixmap)->width , &(*pixmap)->height)) ||
 	    /* resize_sixel() frees pixmap->image in failure. */
 	    ! resize_sixel( *pixmap , width , height , 1))
@@ -384,6 +387,9 @@ load_sixel_with_mask_from_file_1bpp(
 		return  0 ;
 	}
 
+	src = (*pixmap)->image ;
+
+#if  0
 	if( mask && ( dst = *mask = calloc( 1 , (*pixmap)->width * (*pixmap)->height)))
 	{
 		int  has_tp ;
@@ -427,6 +433,52 @@ load_sixel_with_mask_from_file_1bpp(
 			}
 		}
 	}
+#else
+	{
+		u_char  bg_color ;
+		u_char *  p ;
+
+		if( mask)
+		{
+			*mask = None ;
+		}
+
+		bg_color = 0 ;
+		p = src ;
+
+		/* Guess the current screen background color. */
+		for( y = 0 ; y < (*pixmap)->height ; y++)
+		{
+			for( x = 0 ; x < (*pixmap)->width ; x++)
+			{
+				if( *p >= 0x80)
+				{
+					bg_color = (((*p) & 0x7f) == 1) ? 0 : 1 ;
+					break ;
+				}
+
+				p ++ ;
+			}
+		}
+
+		for( y = 0 ; y < (*pixmap)->height ; y++)
+		{
+			for( x = 0 ; x < (*pixmap)->width ; x++)
+			{
+				if( *src >= 0x80)
+				{
+					/* clear opaque mark */
+					*(src ++) &= 0x7f ;
+				}
+				else
+				{
+					/* replace transparent pixel by the background color */
+					*(src ++) = bg_color ;
+				}
+			}
+		}
+	}
+#endif
 
 	return  1 ;
 }
@@ -469,22 +521,36 @@ load_file(
 	}
 	else
 #endif
-#ifdef  EMBEDDING_SIXEL
-	if( strstr( path , ".six") &&
+#ifdef  BUILTIN_SIXEL
+	if( strcasecmp( path + strlen(path) - 4 , ".six") == 0 &&
 	    /* For old machines and Android (not to use mlimgloader) */
 	#if  ! defined(__NetBSD__) && ! defined(__OpenBSD__) && ! defined(__ANDROID__)
 	    width == 0 && height == 0 &&
 	#endif
 	    ( *pixmap = calloc( 1 , sizeof(**pixmap))))
 	{
+	#if  defined(__NetBSD__) || defined(__OpenBSD__)
+		u_int32_t *  sixel_cmap ;
+
+		if( ! (sixel_cmap = custom_palette) &&
+		  ( sixel_cmap = alloca( sizeof(*sixel_cmap) * 257)))
+		{
+			sixel_cmap[256] = 0 ;	/* No active palette */
+			custom_palette = sixel_cmap ;
+		}
+	#endif
+
 		if( ( (*pixmap)->image = load_sixel_from_file( path ,
 						&(*pixmap)->width , &(*pixmap)->height)) &&
 		    /* resize_sixel() frees pixmap->image in failure. */
 		    resize_sixel( *pixmap , width , height , 4))
 		{
 		#if  defined(__NetBSD__) || defined(__OpenBSD__)
-			/* see set_wall_picture() in x_screen.c */
-			x_display_set_cmap( sixel_cmap , sixel_cmap_size) ;
+			if( sixel_cmap)
+			{
+				/* see set_wall_picture() in x_screen.c */
+				x_display_set_cmap( sixel_cmap , sixel_cmap[256]) ;
+			}
 		#endif
 
 			goto  loaded ;
