@@ -7,6 +7,7 @@
 #include  <string.h>
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>
+#include  <kiklib/kik_unistd.h>	/* kik_usleep */
 
 #include  "x_display.h"
 #include  "x_font.h"
@@ -24,6 +25,9 @@
 static x_color_t  black = { TP_COLOR , 0 , 0 , 0 , 0 } ;
 #endif
 
+#define  ParentRelative  (1L)
+#define  DummyPixmap  (2L)
+
 
 /* --- static variables --- */
 
@@ -58,27 +62,24 @@ scroll_region(
 	return  1 ;
 }
 
-static inline void
-copy_pixel(
-	u_char *  dst ,		/* should be aligned */
-	u_long  pixel ,
-	u_int  bpp
-	)
-{
-	switch( bpp)
-	{
-	case  1:
-		*dst = pixel ;
-		return ;
-	case  2:
-		*((u_int16_t*)dst) = pixel ;
-		return ;
-
-	/* case  4: */
-	default:
-		*((u_int32_t*)dst) = pixel ;
-	}
-}
+/*
+ * copy_pixel() is called more than twice, so if it is implemented as a function
+ * it may be uninlined in compiling.
+ * dst should be aligned.
+ */
+#define  copy_pixel( dst , pixel , bpp) \
+	switch( bpp) \
+	{ \
+	case  1: \
+		*(dst) = pixel ; \
+		break ; \
+	case  2: \
+		*((u_int16_t*)(dst)) = (pixel) ; \
+		break ; \
+	/* case  4: */ \
+	default: \
+		*((u_int32_t*)(dst)) = (pixel) ; \
+	} \
 
 static inline u_int16_t *
 memset16(
@@ -114,6 +115,53 @@ memset32(
 	return  dst ;
 }
 
+#ifdef  USE_FREETYPE
+#define  BLEND(fg,bg,alpha)  ((bg) + ((fg) - (bg)) * (alpha) / 255)
+static int
+copy_blended_pixel(
+	Display *  display ,
+	u_char *  dst ,
+	u_char **  bitmap ,
+	u_long  fg ,
+	u_long  bg ,
+	u_int  bpp
+	)
+{
+	int  a1 = *((*bitmap)++) ;
+	int  a2 = *((*bitmap)++) ;
+	int  a3 = *((*bitmap)++) ;
+
+	/* "> 20" is to avoid garbages in screen. */
+	if( a1 > 20 || a2 > 20 || a3 > 20)
+	{
+		int  r1 ;
+		int  g1 ;
+		int  b1 ;
+		int  r2 ;
+		int  g2 ;
+		int  b2 ;
+
+		r1 = PIXEL_RED(fg,display->rgbinfo) & 0xff ;
+		g1 = PIXEL_GREEN(fg,display->rgbinfo) & 0xff ;
+		b1 = PIXEL_BLUE(fg,display->rgbinfo) & 0xff ;
+		r2 = PIXEL_RED(bg,display->rgbinfo) & 0xff ;
+		g2 = PIXEL_GREEN(bg,display->rgbinfo) & 0xff ;
+		b2 = PIXEL_BLUE(bg,display->rgbinfo) & 0xff ;
+
+		copy_pixel( dst ,
+			RGB_TO_PIXEL( BLEND(r1,r2,a1) , BLEND(g1,g2,a2) ,
+				BLEND(b1,b2,a3) , display->rgbinfo) ,
+			bpp) ;
+
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+#endif
+
 static int
 draw_string(
 	x_window_t *  win ,
@@ -151,15 +199,33 @@ draw_string(
 
 	bpp = win->disp->display->bytes_per_pixel ;
 	font_width = font->width ;
+	xfont = font->xfont ;
 
-	if( ! ( src = alloca( ( size = len * font_width * bpp))) ||
+#if  defined(USE_FREETYPE)
+	if( xfont->is_aa && font->is_proportional)
+	{
+		u_int  width ;
+
+		/* width_full == glyph_width_bytes / 3 */
+		if( x + ( width = len * xfont->width_full) > win->width)
+		{
+			width = win->width - x ;
+		}
+
+		size = width * bpp ;
+	}
+	else
+#endif
+	{
+		size = len * font_width * bpp ;
+	}
+
+	if( ! ( src = alloca( size)) ||
 	    ! ( bitmaps = alloca( ( len * sizeof(*bitmaps)))))
 	{
 		return  0 ;
 	}
 
-	xfont = font->xfont ;
-
 	if( ch_len == 1)
 	{
 		for( count = 0 ; count < len ; count++)
@@ -257,12 +323,29 @@ draw_string(
 	if( wall_picture_bg)
 	{
 		/* bg_color is always NULL */
+		Pixmap  pic ;
+		int  pic_x ;
+		int  pic_y ;
 
-		picture_line_len = win->wall_picture->width * bpp ;
-		picture = win->wall_picture->image +
-		          (y + y_off - win->y) * picture_line_len +
+		if( win->wall_picture == ParentRelative)
+		{
+			pic = win->parent->wall_picture ;
+			pic_x = x ;
+			pic_y = y ;
+		}
+		else
+		{
+			pic = win->wall_picture ;
+			pic_x = x - win->x ;
+			pic_y = y - win->y ;
+		}
+
+		picture_line_len = pic->width * bpp ;
+		picture = pic->image +
+			  (pic_y + y_off) * picture_line_len +
 			  /* - picture_line_len is for picture += picture_line_len below. */
-			  (x - win->x) * bpp - picture_line_len ;
+			  pic_x * bpp - picture_line_len ;
+
 		src_bg_is_set = 1 ;
 	}
 	else
@@ -281,7 +364,11 @@ draw_string(
 
 #if  1
 	/* Optimization for most cases */
-	if( src_bg_is_set && ! font->double_draw_gap)
+	if( src_bg_is_set && ! font->double_draw_gap
+	#ifdef  USE_FREETYPE
+		&& ! xfont->face
+	#endif
+		)
 	{
 		u_char *  bitmap_line ;
 		int  x_off ;
@@ -409,6 +496,10 @@ draw_string(
 
 	for( ; y_off < font_height ; y_off++)
 	{
+	#if  defined(USE_FREETYPE)
+		int  prev_crowded_out = 0 ;
+	#endif
+
 		if( src_bg_is_set)
 		{
 			if( picture)
@@ -459,6 +550,150 @@ draw_string(
 					}
 				}
 			}
+		#if  defined(USE_FREETYPE)
+			else if( xfont->is_aa)
+			{
+				if( font->is_proportional)
+				{
+					/*
+					 * src_bg_is_set is always false
+					 * (see #ifdef USE_FRAMEBUFFER #ifdef USE_FREETYPE
+					 * #endif #endif in xcore_draw_str() in x_draw_str.c)
+					 */
+					int  retreat ;
+					int  advance ;
+					int  width ;
+
+					if( ( retreat = bitmaps[count][xfont->glyph_size - 2]) > 0)
+					{
+						u_int  filled ;
+
+						if( ( filled = (p - src) / bpp) < retreat)
+						{
+							bitmap_line += (retreat - filled) ;
+							x_off = -filled ;
+							p = src ;
+						}
+						else
+						{
+							x_off = -retreat ;
+							p -= (retreat * bpp) ;
+						}
+					}
+					else
+					{
+						x_off = 0 ;
+					}
+
+					width = bitmaps[count][xfont->glyph_size - 1] ;
+
+					/* width - retreat */
+					for( ; x_off < width - retreat ; x_off++ , p += bpp)
+					{
+						u_long  bg ;
+
+						bg = x_display_get_pixel( x + x_off , y + y_off) ;
+
+						if( copy_blended_pixel( win->disp->display ,
+							p , &bitmap_line , fg_color->pixel ,
+							bg , bpp))
+						{
+							continue ;
+						}
+						else if( count == 0 ||
+						         x_off >= prev_crowded_out)
+						{
+							copy_pixel( p , bg , bpp) ;
+						}
+					}
+
+					advance = bitmaps[count][xfont->glyph_size - 3] ;
+
+					for( ; x_off < advance ; x_off++ , p += bpp)
+					{
+						if( count == 0 ||
+						    x_off >= prev_crowded_out)
+						{
+							copy_pixel( p ,
+								x_display_get_pixel(
+									x + x_off , y + y_off) ,
+								bpp) ;
+						}
+					}
+
+					if( prev_crowded_out > advance)
+					{
+						prev_crowded_out -= advance ;
+					}
+					else
+					{
+						prev_crowded_out = 0 ;
+					}
+
+					if( advance > 0 && x_off != advance)
+					{
+						p += ((advance - x_off) * bpp) ;
+
+						if( x_off > advance &&
+						    prev_crowded_out < x_off - advance)
+						{
+							prev_crowded_out = x_off - advance ;
+						}
+					}
+
+					x = x + advance - font_width ;
+				}
+				else
+				{
+					for( x_off = 0 ; x_off < font_width ; x_off++ , p += bpp)
+					{
+						u_long  pixel ;
+
+						if( font->x_off <= x_off &&
+						    x_off < font->x_off +
+						            x_get_bitmap_width( font->xfont))
+						{
+							u_long  bg ;
+
+							if( src_bg_is_set)
+							{
+								if( picture)
+								{
+									bg = (bpp == 2) ?
+									     *((u_int16_t*)p) :
+									     *((u_int32_t*)p) ;
+								}
+								else
+								{
+									bg = bg_color->pixel ;
+								}
+							}
+							else
+							{
+								bg = x_display_get_pixel(
+									x + x_off , y + y_off) ;
+							}
+
+							if( copy_blended_pixel(
+								win->disp->display ,
+								p , &bitmap_line ,
+								fg_color->pixel ,
+								bg , bpp))
+							{
+								continue ;
+							}
+						}
+
+						if( ! src_bg_is_set)
+						{
+							pixel = x_display_get_pixel(
+								x + x_off , y + y_off) ;
+							copy_pixel( p , pixel , bpp) ;
+						}
+					}
+				}
+			}
+		#endif
 			else
 			{
 				int  force_fg ;
@@ -701,6 +936,63 @@ fix_rl_boundary(
 	return  1 ;
 }
 
+static void
+reset_input_focus(
+	x_window_t *  win
+	)
+{
+	u_int  count ;
+
+	if( win->inputtable)
+	{
+		win->inputtable = -1 ;
+	}
+	else
+	{
+		win->inputtable = 0 ;
+	}
+
+	if( win->is_focused)
+	{
+		win->is_focused = 0 ;
+
+		if( win->window_unfocused)
+		{
+			(*win->window_unfocused)( win) ;
+		}
+	}
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		reset_input_focus( win->children[count]) ;
+	}
+}
+
+static int
+check_child_window_area(
+	x_window_t *  win
+	)
+{
+	if( win->num_of_children > 0)
+	{
+		u_int  count ;
+		u_int  sum ;
+
+		for( sum = 0 , count = 1 ; count < win->num_of_children ; count++)
+		{
+			sum += (ACTUAL_WIDTH(win->children[count]) *
+			        ACTUAL_HEIGHT(win->children[count])) ;
+		}
+
+		if( sum < win->disp->width * win->disp->height * 0.9)
+		{
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
 
 /* --- global functions --- */
 
@@ -715,7 +1007,8 @@ x_window_init(
 	u_int  height_inc ,
 	u_int  hmargin ,
 	u_int  vmargin ,
-	int  create_gc
+	int  create_gc ,
+	int  inputtable
 	)
 {
 	memset( win , 0 , sizeof( x_window_t)) ;
@@ -724,6 +1017,7 @@ x_window_init(
 	win->is_scrollable = 1 ;
 
 	win->is_focused = 1 ;
+	win->inputtable = inputtable ;
 	win->is_mapped = 1 ;
 
 	win->create_gc = create_gc ;
@@ -813,15 +1107,17 @@ x_window_ungrab_pointer(
 int
 x_window_set_wall_picture(
 	x_window_t *  win ,
-	Pixmap  pic
+	Pixmap  pic ,
+	int  do_expose
 	)
 {
+	u_int  count ;
 #ifdef  USE_GRF
 	int  ret ;
 
 	if( ( ret = x68k_tvram_set_wall_picture( pic->image , pic->width , pic->height)))
 	{
-		win->wall_picture = 0x1 ;	/* dummy */
+		win->wall_picture = DummyPixmap ;	/* dummy */
 
 		/* Don't set is_scrollable = 0. */
 
@@ -843,27 +1139,38 @@ x_window_set_wall_picture(
 	win->wall_picture = pic ;
 	win->is_scrollable = 0 ;
 
-	clear_margin_area( win) ;
+	if( do_expose)
+	{
+		clear_margin_area( win) ;
 
-	if( win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
-#if  0
-	else
+
+	for( count = 0 ; count < win->num_of_children ; count++)
 	{
-		x_window_clear_all( win) ;
+		x_window_set_wall_picture( win->children[count] , ParentRelative , do_expose) ;
 	}
-#endif
 
 	return  1 ;
 }
 
 int
 x_window_unset_wall_picture(
-	x_window_t *  win
+	x_window_t *  win ,
+	int  do_expose
 	)
 {
+	u_int  count ;
+
 #ifdef  USE_GRF
 	x68k_tvram_set_wall_picture( NULL , 0 , 0) ;
 #endif
@@ -871,18 +1178,26 @@ x_window_unset_wall_picture(
 	win->wall_picture = None ;
 	win->is_scrollable = 1 ;
 
-	clear_margin_area( win) ;
+	if( do_expose)
+	{
+		clear_margin_area( win) ;
 
-	if( win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
-#if  0
-	else
+
+	for( count = 0 ; count < win->num_of_children ; count++)
 	{
-		x_window_clear_all( win) ;
+		x_window_unset_wall_picture( win->children[count] , do_expose) ;
 	}
-#endif
 
 	return  1 ;
 }
@@ -971,13 +1286,36 @@ x_window_add_child(
 	child->parent = win ;
 	child->x = x ;
 	child->y = y ;
-	child->is_mapped = map ;
+	child->is_focused = child->is_mapped = map ;
 
 	win->children[ win->num_of_children ++] = child ;
 
 	return  1 ;
 }
 
+int
+x_window_remove_child(
+	x_window_t *  win ,
+	x_window_t *  child
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		if( win->children[count] == child)
+		{
+			child->parent = NULL ;
+
+			win->children[count] = win->children[--win->num_of_children] ;
+
+			return  1 ;
+		}
+	}
+
+	return  0 ;
+}
+
 x_window_t *
 x_get_root_window(
 	x_window_t *  win
@@ -1032,6 +1370,12 @@ x_window_show(
 
 	win->my_window = win ;	/* dummy */
 
+	if( win->parent && ! win->parent->is_transparent &&
+	    win->parent->wall_picture)
+	{
+		x_window_set_wall_picture( win , ParentRelative , 0) ;
+	}
+
 	/*
 	 * This should be called after Window Manager settings, because
 	 * x_set_{window|icon}_name() can be called in win->window_realized().
@@ -1076,6 +1420,9 @@ x_window_map(
 	if( ! win->is_mapped)
 	{
 		win->is_mapped = 1 ;
+
+		(*win->window_exposed)( win ,
+			0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 		clear_margin_area( win) ;
 	}
 
@@ -1215,15 +1562,25 @@ x_window_move(
 	int  y
 	)
 {
-	int  prev_x ;
-	int  prev_y ;
-
-	prev_x = win->x ;
-	prev_y = win->y ;
+	if( win->x == x && win->y == y)
+	{
+		return  0 ;
+	}
 
 	win->x = x ;
 	win->y = y ;
 
+	if( ! check_child_window_area( x_get_root_window( win)) ||
+	    win->x + ACTUAL_WIDTH(win) > win->disp->width ||
+	    win->y + ACTUAL_HEIGHT(win) > win->disp->height)
+	{
+		/*
+		 * XXX Hack
+		 * (Expect the caller to call x_window_resize() immediately after this.)
+		 */
+		return  1 ;
+	}
+
 	/*
 	 * XXX
 	 * Check if win is input method window or not, because x_window_move()
@@ -1285,8 +1642,25 @@ x_window_clear(
 	}
 	else
 	{
-		return  copy_area( win , win->wall_picture , None ,
-				x , y , width , height , x , y , 1) ;
+		Pixmap  pic ;
+		int  src_x ;
+		int  src_y ;
+
+		if( win->wall_picture == ParentRelative)
+		{
+			src_x = x + win->x ;
+			src_y = y + win->y ;
+			pic = win->parent->wall_picture ;
+		}
+		else
+		{
+			pic = win->wall_picture ;
+			src_x = x ;
+			src_y = y ;
+		}
+
+		return  copy_area( win , pic , None , src_x , src_y ,
+				width , height , x , y , 1) ;
 	}
 
 	return  1 ;
@@ -1379,7 +1753,9 @@ x_window_blank(
 	x_window_t *  win
 	)
 {
-	return  x_window_fill_with( win , &win->fg_color , 0 , 0 , win->width , win->height) ;
+	return  x_window_fill_with( win , &win->fg_color , 0 , 0 ,
+			win->width - RIGHT_MARGIN(win) ,
+			win->height - BOTTOM_MARGIN(win)) ;
 }
 
 int
@@ -1388,6 +1764,11 @@ x_window_update(
 	int  flag
 	)
 {
+	if( ! win->is_mapped)
+	{
+		return  0 ;
+	}
+
 	if( win->update_window)
 	{
 		(*win->update_window)( win, flag) ;
@@ -1403,6 +1784,11 @@ x_window_update_all(
 {
 	u_int  count ;
 
+	if( ! win->is_mapped)
+	{
+		return  0 ;
+	}
+
 	if( ! win->parent)
 	{
 		x_display_reset_cmap() ;
@@ -1553,6 +1939,12 @@ x_window_receive_event(
 			win->button_is_pressing = 1 ;
 			win->prev_button_press_event = event->xbutton ;
 		}
+
+		if( ! win->is_focused && win->inputtable && event->xbutton.button == Button1 &&
+		    ! event->xbutton.state)
+		{
+			x_window_set_input_focus( win) ;
+		}
 	}
 
 	return  1 ;
@@ -1656,6 +2048,12 @@ x_window_get_str(
 			}
 		}
 	}
+	else if( *keysym == XK_Tab && (event->state & ShiftMask))
+	{
+		*keysym = XK_ISO_Left_Tab ;
+
+		return  0 ;
+	}
 
 	/*
 	 * Control + '@'(0x40) ... '_'(0x5f) -> 0x00 ... 0x1f
@@ -1999,7 +2397,14 @@ x_window_set_selection_owner(
 	Time  time
 	)
 {
+#ifdef  __ANDROID__
+	if( win->utf_selection_requested)
+	{
+		(*win->utf_selection_requested)( win , NULL , 0) ;
+	}
+#else
 	win->is_sel_owner = 1 ;
+#endif
 
 	return  1 ;
 }
@@ -2010,6 +2415,10 @@ x_window_xct_selection_request(
 	Time  time
 	)
 {
+#ifdef  __ANDROID__
+	x_display_request_text_selection() ;
+#endif
+
 	return  1 ;
 }
 
@@ -2019,6 +2428,10 @@ x_window_utf_selection_request(
 	Time  time
 	)
 {
+#ifdef  __ANDROID__
+	x_display_request_text_selection() ;
+#endif
+
 	return  1 ;
 }
 
@@ -2042,6 +2455,10 @@ x_window_send_text_selection(
 	Atom  sel_type
 	)
 {
+#ifdef  __ANDROID__
+	x_display_send_text_selection( sel_data , sel_len) ;
+#endif
+
 	return  1 ;
 }
 
@@ -2133,9 +2550,16 @@ x_window_get_mod_meta_mask(
 int
 x_window_bell(
 	x_window_t *  win ,
-	x_bel_mode_t  bel_mode
+	x_bel_mode_t  mode
 	)
 {
+	if( mode & BEL_VISUAL)
+	{
+		x_window_blank( win) ;
+		kik_usleep( 100000) ;	/* 100 mili sec */
+		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+	}
+
 	return  1 ;
 }
 
@@ -2155,6 +2579,19 @@ x_window_translate_coordinates(
 	return  0 ;
 }
 
+void
+x_window_set_input_focus(
+	x_window_t *  win
+	)
+{
+	reset_input_focus( x_get_root_window( win)) ;
+	win->inputtable = win->is_focused = 1 ;
+	if( win->window_focused)
+	{
+		(*win->window_focused)( win) ;
+	}
+}
+
 
 /* for x_display.c */
 
