@@ -157,6 +157,9 @@ typedef LPLOGFONT XFontSet ;
 #ifndef  VK_OEM_1
 #define  VK_OEM_1  0xba
 #endif
+#ifndef  VK_OEM_MINUS
+#define  VK_OEM_MINUS  0xbd
+#endif
 #ifndef  VK_OEM_2
 #define  VK_OEM_2  0xbf
 #endif
@@ -166,6 +169,12 @@ typedef LPLOGFONT XFontSet ;
 #ifndef  VK_OEM_4
 #define  VK_OEM_4  0xdb
 #endif
+#ifndef  VK_OEM_5
+#define  VK_OEM_5  0xdc
+#endif
+#ifndef  VK_OEM_6
+#define  VK_OEM_6  0xdd
+#endif
 #ifndef  VK_OEM_7
 #define  VK_OEM_7  0xde
 #endif
@@ -299,7 +308,7 @@ typedef LPLOGFONT XFontSet ;
 	((ksym) == XK_KP_0 || (XK_KP_1 <= (ksym) && (ksym) <= XK_KP_Divide))
 #define IsModifierKey(ksym)  (0)
 
-#define XK_ISO_Level3_Lock	0xffd9	/* dummy */
+#define XK_ISO_Left_Tab	0xffd9
 
 
 /* XPoint(short x, short y) in Xlib. POINT(long x, long y) in win32. */
@@ -317,6 +326,7 @@ typedef LPLOGFONT XFontSet ;
 
 /* Same as definition in X11/cursorfont.h */
 #define XC_xterm 152
+#define XC_left_ptr 68
 
 /* Same as definition in X11/Xutil.h */
 #define NoValue         0x0000
@@ -363,10 +373,6 @@ typedef LPLOGFONT XFontSet ;
 #endif	/* UTF16_IME_CHAR */
 
 
-#define  XC_sb_v_double_arrow  0
-#define  XC_left_ptr  0
-
-
 int  XParseGeometry( char *  str , int *  x , int *  y ,
 	unsigned int  *  width , unsigned int  *  height) ;
 
