@@ -20,6 +20,42 @@
 #endif
 
 
+/* --- static functions --- */
+
+static size_t
+conv_utf16_to_utf8(
+	u_char *  dst ,
+	size_t  dst_len ,
+	u_char *  src ,
+	size_t  src_len
+	)
+{
+	size_t  conv_len ;
+	mkf_parser_t *  utf16_parser ;
+	mkf_conv_t *  utf8_conv ;
+
+	utf16_parser = mkf_utf16le_parser_new() ;
+	utf8_conv = mkf_utf8_conv_new() ;
+
+	(*utf16_parser->init)( utf16_parser) ;
+	(*utf16_parser->set_str)( utf16_parser , src , src_len) ;
+	(*utf8_conv->init)( utf8_conv) ;
+
+	if( ( conv_len = (*utf8_conv->convert)( utf8_conv , dst ,
+				dst_len , utf16_parser)) == dst_len)
+	{
+		conv_len -- ;
+	}
+
+	dst[conv_len] = '\0' ;
+
+	(*utf16_parser->delete)( utf16_parser) ;
+	(*utf8_conv->delete)( utf8_conv) ;
+
+	return  conv_len ;
+}
+
+
 /* --- global functions --- */
 
 /*
@@ -36,10 +72,9 @@ x_dnd_filter_event(
 	HDROP  drop ;
 	UINT  num ;
 	int  count ;
+	int  do_scp ;
 #ifndef  USE_WIN32API
-	mkf_parser_t *  utf16_parser ;
 	mkf_conv_t *  utf16_conv ;
-	mkf_conv_t *  utf8_conv ;
 	mkf_parser_t *  utf8_parser ;
 #endif
 
@@ -48,14 +83,10 @@ x_dnd_filter_event(
 		return  0 ;
 	}
 
-	if( ! win->utf_selection_notified)
-	{
-		return  1 ;
-	}
+	/* Shift+DnD => SCP */
+	do_scp = (GetKeyState(VK_SHIFT) < 0) ;
 
 #ifndef  USE_WIN32API
-	utf16_parser = mkf_utf16le_parser_new() ;
-	utf8_conv = mkf_utf8_conv_new() ;
 	utf8_parser = mkf_utf8_parser_new() ;
 	utf16_conv = mkf_utf16le_conv_new() ;
 #endif
@@ -64,57 +95,74 @@ x_dnd_filter_event(
 	num = DragQueryFile( drop , 0xffffffff , NULL , 0) ;
 	for( count = 0 ; count < num ; count ++)
 	{
-		WCHAR  path[MAX_PATH] ;
+		WCHAR  utf16_path[MAX_PATH] ;
 		
-		if( ( num = DragQueryFileW( drop , count , path ,
-					sizeof(path) / sizeof(path[0]))) > 0)
+		if( ( num = DragQueryFileW( drop , count , utf16_path ,
+					sizeof(utf16_path) / sizeof(utf16_path[0]))) > 0)
 		{
 			size_t  path_len ;
-			
-		#ifndef  USE_WIN32API
 			u_char  utf8_path[MAX_PATH] ;
-			u_char  posix_path[MAX_PATH] ;
-			
-			(*utf16_parser->init)( utf16_parser) ;
-			(*utf16_parser->set_str)( utf16_parser , (u_char*)path ,
-						num * sizeof(path[0])) ;
-			(*utf8_conv->init)( utf8_conv) ;
 
-			if( ( path_len = (*utf8_conv->convert)( utf8_conv , utf8_path ,
-							sizeof(utf8_path) , utf16_parser)) == 0)
+		#ifdef  USE_WIN32API
+			if( do_scp)
+			{
+				if( win->set_xdnd_config &&
+				    conv_utf16_to_utf8( utf8_path , sizeof(utf8_path) ,
+					utf16_path , num * sizeof(utf16_path[0])) > 0)
+				{
+					(*win->set_xdnd_config)( win , NULL ,
+						"scp" , utf8_path) ;
+				}
+			}
+			else
+			{
+				path_len = num * sizeof(utf16_path[0]) ;
+
+				if( win->utf_selection_notified)
+				{
+					(*win->utf_selection_notified)( win ,
+						(u_char*)utf16_path , path_len) ;
+				}
+			}
+		#else
+			u_char  posix_path[MAX_PATH] ;
+
+			if( conv_utf16_to_utf8( utf8_path , sizeof(utf8_path) ,
+				utf16_path , num * sizeof(utf16_path[0])) == 0)
 			{
 				continue ;
 			}
 
-			if( path_len == sizeof(utf8_path))
-			{
-				path_len -- ;
-			}
-
-			utf8_path[path_len] = '\0' ;
-
 			cygwin_conv_to_posix_path( utf8_path , posix_path) ;
 
-			(*utf8_parser->init)( utf8_parser) ;
-			(*utf8_parser->set_str)( utf8_parser , posix_path , strlen(posix_path)) ;
-			(*utf16_conv->init)( utf16_conv) ;
-
-			if( ( path_len = (*utf16_conv->convert)( utf16_conv , (u_char*)path ,
-						sizeof(path) , utf8_parser)) == 0)
+			if( do_scp)
 			{
-				continue ;
+				if( win->set_xdnd_config)
+				{
+					(*win->set_xdnd_config)( win , NULL ,
+						"scp" , posix_path) ;
+				}
+			}
+			else if( win->utf_selection_notified)
+			{
+				(*utf8_parser->init)( utf8_parser) ;
+				(*utf8_parser->set_str)( utf8_parser , posix_path ,
+					strlen(posix_path) + 1) ;
+				(*utf16_conv->init)( utf16_conv) ;
+
+				if( ( path_len = (*utf16_conv->convert)( utf16_conv ,
+							(u_char*)utf16_path ,
+							sizeof(utf16_path) , utf8_parser)) > 0)
+				{
+					(*win->utf_selection_notified)( win ,
+						(u_char*)utf16_path , path_len) ;
+				}
 			}
-		#else
-			path_len = num * sizeof(path[0]) ;
 		#endif
-		
-			(*win->utf_selection_notified)( win , (u_char*)path , path_len) ;
 		}
 	}
 
 #ifndef  USE_WIN32API
-	(*utf16_parser->delete)( utf16_parser) ;
-	(*utf8_conv->delete)( utf8_conv) ;
 	(*utf8_parser->delete)( utf8_parser) ;
 	(*utf16_conv->delete)( utf16_conv) ;
 #endif
