@@ -24,6 +24,7 @@
 #include  "../x_dnd.h"
 #endif
 
+#include  "x_display.h"		/* x_display_get_cursor */
 #include  "x_gdiobj_pool.h"
 
 
@@ -50,6 +51,14 @@
 	GetSystemMetrics(SM_CYCAPTION)
 #endif
 
+/* win->width is not multiples of (win)->width_inc if window is maximized. */
+#define  RIGHT_MARGIN(win) \
+	((win)->width_inc ? ((win)->width - (win)->min_width) % (win)->width_inc : 0)
+#define  BOTTOM_MARGIN(win) \
+	((win)->height_inc ? ((win)->height - (win)->min_height) % (win)->height_inc : 0)
+
+#define  ParentRelative  (1L)
+
 #if  0
 #define  DEBUG_SCROLLABLE
 #endif
@@ -301,33 +310,38 @@ notify_focus_in_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
-	if( win->is_focused)
+	if( ! win->is_focused)
 	{
-		return ;
-	}
-
-	win->is_focused = 1 ;
-
-	/* If win->wall_picture is set, is_scrollable is always 0. */
-	if( ! win->wall_picture)
-	{
-		if( ! win->parent)
+		if( win->inputtable > 0 ||
+		    win->parent == NULL)	/* check_scrollable checks root->is_focused */
 		{
-			win->is_scrollable = check_scrollable( win) ;
+			win->is_focused = 1 ;
 		}
-		else
+
+		/* If win->wall_picture is set, is_scrollable is always 0. */
+		if( ! win->wall_picture)
 		{
-			win->is_scrollable = win->parent->is_scrollable ;
+			if( ! win->parent)
+			{
+				win->is_scrollable = check_scrollable( win) ;
+			}
+			else
+			{
+				win->is_scrollable = win->parent->is_scrollable ;
+			}
 		}
-	}
 
-	x_xic_set_focus( win) ;
+		if( win->inputtable > 0)
+		{
+			x_xic_set_focus( win) ;
 
-	if( win->window_focused)
-	{
-		(*win->window_focused)( win) ;
+			if( win->window_focused)
+			{
+				(*win->window_focused)( win) ;
+			}
+		}
 	}
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -341,22 +355,20 @@ notify_focus_out_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
-	if( ! win->is_focused)
+	if( win->is_focused)
 	{
-		return ;
-	}
-	
-	win->is_focused = 0 ;
+		win->is_focused = 0 ;
 
-	win->is_scrollable = 0 ;
+		win->is_scrollable = 0 ;
 
-	x_xic_unset_focus( win) ;
+		x_xic_unset_focus( win) ;
 
-	if( win->window_unfocused)
-	{
-		(*win->window_unfocused)( win) ;
+		if( win->window_unfocused)
+		{
+			(*win->window_unfocused)( win) ;
+		}
 	}
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -372,7 +384,12 @@ notify_move_to_children(
 {
 	int  count ;
 
-	if( ! win->parent)
+	
+	if( win->wall_picture)
+	{
+		/* If win->wall_picture is set, is_scrollable is always 0. */
+	}
+	else if( ! win->parent)
 	{
 		int  is_scrollable ;
 
@@ -454,6 +471,7 @@ total_min_width(
 	{
 		if( win->children[count]->is_mapped)
 		{
+			/* XXX */
 			min_width += total_min_width( win->children[count]) ;
 		}
 	}
@@ -475,6 +493,7 @@ total_min_height(
 	{
 		if( win->children[count]->is_mapped)
 		{
+			/* XXX */
 			min_height += total_min_height( win->children[count]) ;
 		}
 	}
@@ -715,31 +734,61 @@ clear_margin_area(
 	x_window_t *  win
 	)
 {
+	u_int  right_margin ;
+	u_int  bottom_margin ;
+	u_int  win_width ;
+	u_int  win_height ;
+
 	if( win->gc->gc == None)
 	{
 		return  0 ;
 	}
 
+	right_margin = RIGHT_MARGIN(win) ;
+	bottom_margin = BOTTOM_MARGIN(win) ;
+	win_width = win->width - right_margin ;
+	win_height = win->height - bottom_margin ;
+
 	if( win->wall_picture)
 	{
-		if( win->hmargin > 0)
+		Pixmap  pic ;
+		int  src_x ;
+		int  src_y ;
+
+		if( win->wall_picture == ParentRelative)
 		{
-			BitBlt( win->gc->gc , 0 , 0 , win->hmargin , ACTUAL_HEIGHT(win) ,
-				win->wall_picture , 0 , 0 , SRCCOPY) ;
-			BitBlt( win->gc->gc , win->width + win->hmargin , 0 ,
-				ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-				win->wall_picture , win->width + win->hmargin , 0 ,
-				SRCCOPY) ;
+			pic = win->parent->wall_picture ;
+			src_x = win->x ;
+			src_y = win->y ;
+		}
+		else
+		{
+			pic = win->wall_picture ;
+			src_x = src_y = 0 ;
 		}
 
-		if( win->vmargin > 0)
+		if( win->hmargin > 0 || right_margin > 0)
+		{
+			BitBlt( win->gc->gc , 0 , 0 ,
+				win->hmargin , ACTUAL_HEIGHT(win) + right_margin ,
+				pic , src_x , src_y , SRCCOPY) ;
+			BitBlt( win->gc->gc , win_width + win->hmargin , 0 ,
+				win->hmargin + right_margin ,
+				ACTUAL_HEIGHT(win) + bottom_margin ,
+				pic ,
+				src_x + win_width + win->hmargin , src_y , SRCCOPY) ;
+		}
+
+		if( win->vmargin > 0 || bottom_margin > 0)
 		{
 			BitBlt( win->gc->gc , win->hmargin , 0 ,
-				win->width + win->hmargin , win->vmargin ,
-				win->wall_picture , win->hmargin , 0 , SRCCOPY) ;
-			BitBlt( win->gc->gc, win->hmargin , win->height + win->vmargin ,
-				win->width + win->hmargin , ACTUAL_HEIGHT(win) ,
-				win->wall_picture , win->hmargin , win->height + win->vmargin ,
+				win_width , win->vmargin ,
+				pic ,
+				src_x + win->hmargin , src_y , SRCCOPY) ;
+			BitBlt( win->gc->gc , win->hmargin , win_height + win->vmargin ,
+				win_width , win->vmargin + bottom_margin ,
+				pic ,
+				src_x + win->hmargin , src_y + win_height + win->vmargin ,
 				SRCCOPY) ;
 		}
 	}
@@ -747,25 +796,29 @@ clear_margin_area(
 	{
 		HBRUSH  brush ;
 		RECT  r ;
+		int  right ;
+		int  bottom ;
 
 		brush = x_acquire_brush( win->bg_color.pixel) ;
+		right = ACTUAL_WIDTH(win) + right_margin ;
+		bottom = ACTUAL_HEIGHT(win) + bottom_margin ;
 
-		if( win->hmargin > 0)
+		if( win->hmargin > 0 || right_margin > 0)
 		{
-			SetRect( &r , 0 , 0 , win->hmargin , ACTUAL_HEIGHT(win)) ;
+			SetRect( &r , 0 , 0 , win->hmargin , bottom) ;
 			FillRect( win->gc->gc , &r , brush) ;
-			SetRect( &r , win->width + win->hmargin , 0 ,
-				ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+			SetRect( &r , win_width + win->hmargin , 0 ,
+				right , bottom) ;
 			FillRect( win->gc->gc , &r , brush) ;
 		}
 
-		if( win->vmargin > 0)
+		if( win->vmargin > 0 || bottom_margin > 0)
 		{
 			SetRect( &r , win->hmargin , 0 ,
-				win->width + win->hmargin , win->vmargin) ;
+				win_width + win->hmargin , win->vmargin) ;
 			FillRect( win->gc->gc , &r , brush) ;
-			SetRect( &r , win->hmargin , win->height + win->vmargin ,
-				win->width + win->hmargin , ACTUAL_HEIGHT(win)) ;
+			SetRect( &r , win->hmargin , win_height + win->vmargin ,
+				win_width + win->hmargin , bottom) ;
 			FillRect( win->gc->gc , &r , brush) ;
 		}
 
@@ -939,6 +992,28 @@ invoke_selection_request(
 	return  0 ;
 }
 
+static void
+reset_input_focus(
+	x_window_t *  win
+	)
+{
+	u_int  count ;
+
+	if( win->inputtable)
+	{
+		win->inputtable = -1 ;
+	}
+	else
+	{
+		win->inputtable = 0 ;
+	}
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		reset_input_focus( win->children[count]) ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -953,7 +1028,8 @@ x_window_init(
 	u_int  height_inc ,
 	u_int  hmargin ,
 	u_int  vmargin ,
-	int  create_gc	/* ignored */
+	int  create_gc ,	/* ignored */
+	int  inputtable
 	)
 {
 	memset( win , 0 , sizeof( x_window_t)) ;
@@ -964,6 +1040,8 @@ x_window_init(
 	/* if visibility is partially obscured , scrollable will be 0. */
 	win->is_scrollable = 1 ;
 
+	win->inputtable = inputtable ;
+
 	/* This flag will map window automatically in x_window_show(). */
 	win->is_mapped = 1 ;
 
@@ -1081,25 +1159,36 @@ x_window_ungrab_pointer(
 int
 x_window_set_wall_picture(
 	x_window_t *  win ,
-	Pixmap  pic
+	Pixmap  pic ,
+	int  do_expose
 	)
 {
+	u_int  count ;
+
 	win->is_scrollable = 0 ;
 	win->wall_picture = pic ;
 
-	if( win->my_window != None)
+	if( win->my_window != None && do_expose)
 	{
 		InvalidateRect( win->my_window, NULL, FALSE) ;
 	}
 
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		x_window_set_wall_picture( win->children[count] , ParentRelative , do_expose) ;
+	}
+
 	return  1 ;
 }
 
 int
 x_window_unset_wall_picture(
-	x_window_t *  win
+	x_window_t *  win ,
+	int  do_expose
 	)
 {
+	u_int  count ;
+
 	win->is_scrollable = check_scrollable( win) ;
 	win->wall_picture = None ;
 
@@ -1108,6 +1197,11 @@ x_window_unset_wall_picture(
 		InvalidateRect( win->my_window, NULL, FALSE) ;
 	}
 
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		x_window_set_wall_picture( win->children[count] , ParentRelative , do_expose) ;
+	}
+
 	return  1 ;
 }
 
@@ -1128,9 +1222,6 @@ x_window_unset_transparent(
 	return  0 ;
 }
 
-/*
- * XXX Cursor is not changeable for now.
- */
 int
 x_window_set_cursor(
 	x_window_t *  win ,
@@ -1221,13 +1312,39 @@ x_window_add_child(
 	child->parent = win ;
 	child->x = x ;
 	child->y = y ;
-	child->is_mapped = map ;
+
+	if( ! ( child->is_mapped = map) && child->inputtable > 0)
+	{
+		child->inputtable = -1 ;
+	}
 
 	win->children[ win->num_of_children ++] = child ;
 
 	return  1 ;
 }
 
+int
+x_window_remove_child(
+	x_window_t *  win ,
+	x_window_t *  child
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		if( win->children[count] == child)
+		{
+			child->parent = NULL ;
+			win->children[count] = win->children[--win->num_of_children] ;
+
+			return  1 ;
+		}
+	}
+
+	return  0 ;
+}
+
 x_window_t *
 x_get_root_window(
 	x_window_t *  win
@@ -1350,6 +1467,12 @@ x_window_show(
 	DragAcceptFiles( win->my_window , TRUE) ;
 #endif
 
+	if( win->parent && ! win->parent->is_transparent &&
+	    win->parent->wall_picture)
+	{
+		x_window_set_wall_picture( win , ParentRelative , 0) ;
+	}
+
 	/*
 	 * This should be called after Window Manager settings, because
 	 * x_set_{window|icon}_name() can be called in win->window_realized().
@@ -1384,10 +1507,20 @@ x_window_show(
 	{
 		ShowWindow( win->my_window , win->cmd_show) ;
 
+		if( win->inputtable > 0)
+		{
+			reset_input_focus( x_get_root_window( win)) ;
+			win->inputtable = 1 ;
+		}
+
 	#if  0
 		x_window_clear_all( win) ;
 	#endif
 	}
+	else
+	{
+		ShowWindow( win->my_window , SW_HIDE) ;
+	}
 
 	if( win->is_transparent)
 	{
@@ -1445,7 +1578,7 @@ x_window_resize(
 	/* Max width of each window is screen width. */
 	if( (flag & LIMIT_RESIZE) && win->disp->width < width)
 	{
-		win->width = win->disp->width ;
+		win->width = win->disp->width - win->hmargin * 2 ;
 	}
 	else
 	{
@@ -1455,7 +1588,7 @@ x_window_resize(
 	/* Max height of each window is screen height. */
 	if( (flag & LIMIT_RESIZE) && win->disp->height < height)
 	{
-		win->height = win->disp->height ;
+		win->height = win->disp->height - win->vmargin * 2 ;
 	}
 	else
 	{
@@ -1565,6 +1698,11 @@ x_window_move(
 	int  y
 	)
 {
+	if( win->x == x && win->y == y)
+	{
+		return  0 ;
+	}
+
 	win->x = x ;
 	win->y = y ;
 
@@ -1645,12 +1783,27 @@ x_window_clear(
 	}
 	else
 	{
-
 		if( win->wall_picture)
 		{
+			Pixmap  pic ;
+			int  src_x ;
+			int  src_y ;
+
+			if( win->wall_picture == ParentRelative)
+			{
+				pic = win->parent->wall_picture ;
+				src_x = win->x ;
+				src_y = win->y ;
+			}
+			else
+			{
+				pic = win->wall_picture ;
+				src_x = src_y = 0 ;
+			}
+
 			BitBlt( win->gc->gc ,
 				r.left , r.top , r.right - r.left , r.bottom - r.top ,
-				win->wall_picture , r.left , r.top , SRCCOPY) ;
+				pic , src_x + r.left , src_y + r.top , SRCCOPY) ;
 		}
 		else
 		{
@@ -1700,11 +1853,12 @@ x_window_fill_with(
 		return  0 ;
 	}
 
-	if( height == 1)
+	if( height == 1 || width == 1)
 	{
 		x_release_pen( x_gc_set_pen( win->gc , x_acquire_pen( color->pixel))) ;
 		MoveToEx( win->gc->gc , win->hmargin + x , win->vmargin + y , NULL) ;
-		LineTo( win->gc->gc , win->hmargin + x + width , win->vmargin + y) ;
+		LineTo( win->gc->gc , win->hmargin + x + (width == 1 ? 0 : width) ,
+		                      win->vmargin + y + (width == 1 ? height : 0)) ;
 	}
 	else
 	{
@@ -1805,14 +1959,7 @@ x_window_update_all(
 {
 	u_int  count ;
 
-	clear_margin_area( win) ;
-
-#if  0
-	if( win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-	}
-#endif
+	InvalidateRect( win->my_window , NULL , FALSE) ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
@@ -2038,9 +2185,8 @@ x_window_receive_event(
 			kev.state = get_key_state() ;
 
 			if( ( 0x30 <= event->wparam && event->wparam <= VK_DIVIDE) ||
-				event->wparam == VK_BACK || event->wparam == VK_TAB ||
-				event->wparam == VK_RETURN || event->wparam == VK_ESCAPE ||
-				event->wparam == VK_SPACE)
+			    event->wparam == VK_BACK || event->wparam == VK_RETURN ||
+			    event->wparam == VK_ESCAPE || event->wparam == VK_SPACE)
 			{
 				kev.ch = event->wparam ;
 
@@ -2069,11 +2215,11 @@ x_window_receive_event(
 					}
 				}
 				else if( ( kev.state & ControlMask) &&
-				         '2' <= kev.ch && kev.ch <= '8')
+				         '0' <= kev.ch && kev.ch <= '9')
 				{
 					/*
 					 * - See x_xic_get_str() in win32/x_xic.c.
-					 * - Control+2-8 (which doesn't cause WM_*_CHAR message.
+					 * - Control+0-9 doesn't cause WM_*_CHAR message.
 					 */
 				}
 				else
@@ -2082,6 +2228,26 @@ x_window_receive_event(
 					break ;
 				}
 			}
+			else if( event->wparam == VK_TAB)
+			{
+				kev.ch = event->wparam ;
+
+				if( kev.state & ShiftMask)
+				{
+					event->wparam = XK_ISO_Left_Tab ;
+					x_xic_filter_event( x_get_root_window( win), event) ;
+				}
+
+				if( kev.state & ControlMask)
+				{
+					/* (Shift+)Control+Tab doesn't cause WM_*_CHAR message. */
+				}
+				else
+				{
+					/* wait for WM_*_CHAR message. */
+					break ;
+				}
+			}
 			else if( event->msg == WM_SYSKEYDOWN)
 			{
 				if( kev.state & ModMask)
@@ -2090,7 +2256,12 @@ x_window_receive_event(
 					 * VK_OEM_XXX doesn't cause WM_*_CHAR message
 					 * in WM_SYSKEYDOWN.
 					 */
-					kev.ch = oem_key_to_char( event->wparam) ;
+					if( ( kev.ch = oem_key_to_char( event->wparam)))
+					{
+						event->wparam = kev.ch ;
+						x_xic_filter_event( x_get_root_window( win) ,
+							event) ;
+					}
 				}
 				else
 				{
@@ -2102,25 +2273,54 @@ x_window_receive_event(
 					break ;
 				}
 			}
+			else if( VK_SHIFT <= event->wparam && event->wparam <= VK_MENU)
+			{
+				/* Don't call key_pressed event */
+				break ;
+			}
 			else
 			{
+				kev.ch = 0 ;
+
 				/*
 				 * - See x_xic_get_str() in win32/x_xic.c.
-				 * - Following keys don't cause WM_*_CHAR message.
 				 * - Ctrl+Alt+key(AltGr+key) can cause WM_*_CHAR message later.
 				 */
 				if( ! ( kev.state & ModMask) &&
-				    ( kev.state & ControlMask) &&
-				    ( event->wparam == VK_OEM_2 ||	/* Ctl+/ */
-				      event->wparam == VK_OEM_3 ||	/* Ctl+@ */
-				      event->wparam == VK_OEM_7 ||	/* Ctl+^ */
-				      event->wparam == VK_OEM_102))	/* Ctl+_ */
+				    ( kev.state & ControlMask))
 				{
-					kev.ch = oem_key_to_char( event->wparam) ;
-				}
-				else
-				{
-					kev.ch = 0 ;
+					if( ( kev.ch = oem_key_to_char( event->wparam)))
+					{
+						/*
+						 * VK_OEM_1 <= event->wparam &&
+						 * event->wparam <= VK_OEM_102
+						 */
+						int  orig_wparam ;
+
+						orig_wparam = event->wparam ;
+						event->wparam = kev.ch ;
+						x_xic_filter_event( x_get_root_window( win) ,
+							event) ;
+
+						if( orig_wparam == VK_OEM_4 ||	/* Ctl+[ */
+						    orig_wparam == VK_OEM_5 ||	/* Ctl+\ */
+						    orig_wparam == VK_OEM_6 ||	/* Ctl+] */
+						    ( orig_wparam == VK_OEM_102 && /* Ctl+_ */
+						      ! (kev.state & ShiftMask)) ||
+						    ( orig_wparam == VK_OEM_MINUS && /* Ctrl+- */
+						      (kev.state & ShiftMask)))
+						{
+							break ;
+						}
+						else
+						{
+							/*
+							 * keys except VK_OEM_{4|5|6} and
+							 * Shift+VK_OEM_102 don't
+							 * cause WM_*_CHAR message.
+							 */
+						}
+					}
 				}
 			}
 
@@ -2146,10 +2346,11 @@ x_window_receive_event(
 
 			kev.state = get_key_state() ;
 
-			if( ( kev.state & (ControlMask|ModMask)) && event->wparam > 0x20)
+			if( ( kev.state & (ControlMask|Mod1Mask)) == (ControlMask|Mod1Mask) &&
+			    0x20 < event->wparam && event->wparam != 0x7f)
 			{
-				/* Ctrl+Alt+key => AltGr+key */
-				kev.state &= ~(ControlMask|ModMask) ;
+				/* AltGr+key sends Ctrl+Alt+char */
+				kev.state &= ~(ControlMask|Mod1Mask) ;
 			}
 
 		#if  0
@@ -2178,14 +2379,8 @@ x_window_receive_event(
 		kik_debug_printf( "FOCUS IN %p\n" , win->my_window) ;
 	#endif
 
-	#if  0
-		/* root window event only */
-		if( win->parent == NULL)
-	#endif
-		{
-			urgent_bell( win , 0) ;
-			notify_focus_in_to_children( win) ;
-		}
+		urgent_bell( win , 0) ;
+		notify_focus_in_to_children( x_get_root_window( win)) ;
 
 		break ;
 
@@ -2194,13 +2389,7 @@ x_window_receive_event(
 		kik_debug_printf( "FOCUS OUT %p\n" , win->my_window) ;
 	#endif
 
-	#if  0
-		/* root window event only */
-		if( win->parent == NULL)
-	#endif
-		{
-			notify_focus_out_to_children( win) ;
-		}
+		notify_focus_out_to_children( x_get_root_window( win)) ;
 
 		break ;
 
@@ -2379,6 +2568,12 @@ x_window_receive_event(
 					bev.state, bev.x, bev.y) ;
 			#endif
 			}
+
+			if( ! win->is_focused && win->inputtable &&
+			    bev.button == Button1 && ! bev.state)
+			{
+				x_window_set_input_focus( win) ;
+			}
 		}
 
 		return  1 ;
@@ -2463,7 +2658,27 @@ x_window_receive_event(
 		}
 		
 		return  1 ;
-		
+
+	case  WM_SETCURSOR:
+		{
+			Cursor  cursor ;
+
+			if( win->cursor_shape == XC_nil)
+			{
+				SetCursor( NULL) ;
+			}
+			else if( ( cursor = x_display_get_cursor( win->disp , win->cursor_shape)))
+			{
+				SetCursor( cursor) ;
+			}
+			else
+			{
+				break ;
+			}
+
+			return  1 ;
+		}
+
 	case  WM_RENDERALLFORMATS:
 		OpenClipboard( win->my_window) ;
 		EmptyClipboard() ;
@@ -2537,58 +2752,44 @@ x_window_receive_event(
 			u_int  height ;
 			u_int  min_width ;
 			u_int  min_height ;
-			u_int  old_width ;
-			u_int  old_height ;
 
 			width = LOWORD(event->lparam) ;
 			height = HIWORD(event->lparam) ;
 
 			min_width = total_min_width(win) ;
 			min_height = total_min_height(win) ;
-			old_width = ACTUAL_WIDTH(win) ;
-			old_height = ACTUAL_HEIGHT(win) ;
-
-		#if  0
-			kik_debug_printf( "WM_SIZE resized from %d %d to %d %d\n" ,
-				old_width, old_height, width, height) ;
-		#endif
 		
-			if( width < min_width || height < min_height)
+			if( width < min_width + win->hmargin * 2 ||
+			    height < min_height + win->vmargin * 2)
 			{
 				x_window_resize( win,
-					K_MAX(min_width,width) - win->hmargin * 2,
-					K_MAX(min_height,height) - win->vmargin * 2,
+					K_MAX(min_width + win->hmargin * 2 , width) -
+						win->hmargin * 2 ,
+					K_MAX(min_height + win->vmargin * 2 , height) -
+						win->vmargin * 2 ,
 					NOTIFY_TO_MYSELF) ;
 			}
-			else if( width != old_width || height != old_height)
+			else if( width != ACTUAL_WIDTH(win) ||
+			         height != ACTUAL_HEIGHT(win))
 			{
 				u_int  width_surplus ;
 				u_int  height_surplus ;
 
-				if( width >= old_width)
+				if( IsZoomed( x_get_root_window( win)->my_window))
 				{
-					width_surplus = (width - old_width) %
-							total_width_inc(win) ;
+					width_surplus = height_surplus = 0 ;
 				}
 				else
 				{
-					width_surplus = total_width_inc(win) -
-							( (old_width - width) %
-							  total_width_inc(win) ) ;
-				}
+					width_surplus = (width - min_width -
+							 win->hmargin * 2) %
+							total_width_inc(win) ;
 
-				if( height >= old_height)
-				{
-					height_surplus = (height - old_height) %
+					height_surplus = (height - min_height -
+							  win->vmargin * 2) %
 							total_height_inc(win) ;
 				}
-				else
-				{
-					height_surplus = total_height_inc(win) -
-							( (old_height - height) %
-							  total_height_inc(win) ) ;
-				}
-				
+
 				win->width = width - win->hmargin * 2 ;
 				win->height = height - win->vmargin * 2 ;
 
@@ -2817,11 +3018,28 @@ x_window_copy_area(
 	int  dst_y	/* >= 0 */
 	)
 {
+#ifndef  DONT_OPTIMIZE_DRAWING_PICTURE
+	int  tmp_gc ;
+#endif
+
 	if( dst_x >= win->width || dst_y >= win->height)
 	{
 		return  0 ;
 	}
 
+	/* for xterm_show_picture() */
+#ifndef  DONT_OPTIMIZE_DRAWING_PICTURE
+	if( win->gc->gc == None)
+	{
+		x_set_gc( win->gc , GetDC( win->my_window)) ;
+		tmp_gc = 1 ;
+	}
+	else
+	{
+		tmp_gc = 0 ;
+	}
+#endif
+
 	if( dst_x + width > win->width)
 	{
 		width = win->width - dst_x ;
@@ -2844,6 +3062,14 @@ x_window_copy_area(
 			width , height , src , src_x , src_y , SRCCOPY) ;
 	}
 
+#ifndef  DONT_OPTIMIZE_DRAWING_PICTURE
+	if( tmp_gc)
+	{
+		ReleaseDC( win->my_window , win->gc->gc) ;
+		x_set_gc( win->gc , None) ;
+	}
+#endif
+
 	return  1 ;
 }
 
@@ -3403,6 +3629,16 @@ x_window_translate_coordinates(
 	return  0 ;
 }
 
+void
+x_window_set_input_focus(
+	x_window_t *  win
+	)
+{
+	reset_input_focus( x_get_root_window( win)) ;
+	win->inputtable = 1 ;
+	SetFocus( win->my_window) ;
+}
+
 #ifdef  DEBUG
 void
 x_window_dump_children(
