@@ -42,6 +42,12 @@ get_spot(
 	spot->x = x + win->hmargin ;
 	spot->y = y + win->vmargin ;
 
+	if( win->parent && GetFocus() == x_get_root_window( win)->my_window)
+	{
+		spot->x += win->x ;
+		spot->y += win->y ;
+	}
+
 	return  1 ;
 }
 
@@ -141,7 +147,7 @@ x_xic_font_set_changed(
 	x_window_t *  win
 	)
 {
-	if( HAS_XIM_LISTENER(win,get_fontset))
+	if( win->xic && HAS_XIM_LISTENER(win,get_fontset))
 	{
 		if( ImmSetCompositionFont( win->xic->ic ,
 			(*win->xim_listener->get_fontset)( win->xim_listener->self)))
@@ -184,8 +190,6 @@ x_xic_set_spot(
 		return  0 ;
 	}
 
-	MapWindowPoints( win->my_window, x_get_root_window( win)->my_window, &spot, 1) ;
-
 	cf.ptCurrentPos = spot ;
 	cf.dwStyle = CFS_FORCE_POSITION ;
 
@@ -204,6 +208,11 @@ x_xic_get_str(
 {
 	size_t  len ;
 
+	if( win->xic == NULL)
+	{
+		goto  zero_return ;
+	}
+
 	*keysym = win->xic->prev_keydown_wparam ;
 
 	if( 'a' <= *keysym && *keysym <= VK_F24)
@@ -222,9 +231,13 @@ x_xic_get_str(
 	{
 		goto  zero_return ;
 	}
+	else if( (event->state & ShiftMask) && *keysym == XK_ISO_Left_Tab)
+	{
+		goto  zero_return ;
+	}
 	else if( event->state & ControlMask)
 	{
-		if( event->ch == '2' || event->ch == ' ')
+		if( event->ch == '2' || event->ch == ' ' || event->ch == '@')
 		{
 			event->ch = 0 ;
 		}
@@ -237,9 +250,10 @@ x_xic_get_str(
 		{
 			event->ch = 0x7f ;
 		}
-		else if( event->ch == '@')
+		else if( event->ch == '0' || event->ch == '1' || event->ch == '9')
 		{
-			event->ch = 0x0 ;
+			/* For modifyOtherKeys */
+			goto  zero_return ;
 		}
 		else if( event->ch == '^')
 		{
@@ -278,10 +292,13 @@ x_xic_get_str(
 #endif
 
 	/* wparam doesn't tell upper case from lower case. */
-	if( 'a' <= event->ch && event->ch <= 'z')
+	if( 'A' <= *keysym && *keysym <= 'Z')
 	{
-		/* Upper to Lower case */
-		*keysym += 0x20 ;
+		if( event->ch < 'A' || 'Z' < event->ch)
+		{
+			/* Upper to Lower case */
+			*keysym += 0x20 ;
+		}
 	}
 
 	*parser = win->xic->parser ;
