@@ -47,8 +47,8 @@ int  x_load_rgb_xcolor( x_display_t *  disp , x_color_t *  xcolor ,
 
 int  x_unload_xcolor( x_display_t *  disp , x_color_t *  xcolor) ;
 
-int  x_get_xcolor_rgb( u_int8_t *  red , u_int8_t *  green , u_int8_t *  blue , u_int8_t *  alpha ,
-	x_color_t *  xcolor) ;
+int  x_get_xcolor_rgba( u_int8_t *  red , u_int8_t *  green , u_int8_t *  blue ,
+	u_int8_t *  alpha , x_color_t *  xcolor) ;
 
 int  x_xcolor_fade( x_display_t * , x_color_t *  xcolor , u_int  fade_ratio) ;
 
