@@ -24,7 +24,7 @@ typedef struct  x_color_manager
 		x_color_t  xcolor ;
 		char *  name ;
 
-	} sys_colors[6] ;
+	} sys_colors[9] ;
 
 	u_int8_t  alpha ;
 	int8_t  is_reversed ;
@@ -35,7 +35,8 @@ typedef struct  x_color_manager
 x_color_manager_t *  x_color_manager_new( x_display_t *  disp ,
 	char *  fg_color , char *  bg_color ,
 	char *  cursor_fg_color , char *  cursor_bg_color ,
-	char *  bd_color , char *  ul_color) ;
+	char *  bd_color , char *  it_color , char *  ul_color ,
+	char *  bl_color , char *  co_color) ;
 
 int  x_color_manager_delete( x_color_manager_t *  color_man) ;
 
@@ -47,9 +48,8 @@ int  x_color_manager_set_cursor_fg_color( x_color_manager_t *  color_man , char
 
 int  x_color_manager_set_cursor_bg_color( x_color_manager_t *  color_man , char *  name) ;
 
-int  x_color_manager_set_bd_color( x_color_manager_t *  color_man , char *  name) ;
-
-int  x_color_manager_set_ul_color( x_color_manager_t *  color_man , char *  name) ;
+int  x_color_manager_set_alt_color( x_color_manager_t *  color_man ,
+	ml_color_t  color , char *  name) ;
 
 char *  x_color_manager_get_fg_color( x_color_manager_t *  color_man) ;
 
@@ -59,9 +59,7 @@ char *  x_color_manager_get_cursor_fg_color( x_color_manager_t *  color_man) ;
 
 char *  x_color_manager_get_cursor_bg_color( x_color_manager_t *  color_man) ;
 
-char *  x_color_manager_get_bd_color( x_color_manager_t *  color_man) ;
-
-char *  x_color_manager_get_ul_color( x_color_manager_t *  color_man) ;
+char *  x_color_manager_get_alt_color( x_color_manager_t *  color_man , ml_color_t  color) ;
 
 x_color_t *  x_get_xcolor( x_color_manager_t *  color_man , ml_color_t  color) ;
 
@@ -77,10 +75,6 @@ int  x_color_manager_adjust_cursor_fg_color( x_color_manager_t *  color_man) ;
 
 int  x_color_manager_adjust_cursor_bg_color( x_color_manager_t *  color_man) ;
 
-int  x_color_manager_adjust_bd_color( x_color_manager_t *  color_man) ;
-
-int  x_color_manager_adjust_ul_color( x_color_manager_t *  color_man) ;
-
 int  x_color_manager_reload( x_color_manager_t *  color_man) ;
 
 int  x_change_true_transbg_alpha( x_color_manager_t *  color_man , u_int8_t  alpha) ;
