@@ -123,8 +123,10 @@ draw_picture(
 	u_int  src_height ;
 	u_int  dst_width ;
 	int  need_clear ;
+	int  is_end ;
 
 	cur_pic = NULL ;
+	is_end = 0 ;
 
 	for( count = 0 ; count < num_of_glyphs ; count++)
 	{
@@ -180,6 +182,8 @@ draw_picture(
 			{
 				continue ;
 			}
+
+			is_end = 1 ;
 		}
 
 		if( need_clear > 0)
@@ -201,6 +205,11 @@ draw_picture(
 				src_x , src_y , src_width , src_height , dst_x , dst_y) ;
 		}
 
+		if( is_end)
+		{
+			return  1 ;
+		}
+
 		dst_x += dst_width ;
 
 	new_picture:
@@ -237,6 +246,21 @@ draw_picture(
 		}
 	}
 
+	if( need_clear > 0)
+	{
+		x_window_clear( window , dst_x , dst_y , dst_width , line_height) ;
+	}
+
+#ifdef  __DEBUG
+	kik_debug_printf(
+		"Drawing picture at %d %d (pix %p mask %p x %d y %d w %d h %d)\n" ,
+		dst_x , dst_y , cur_pic->pixmap , cur_pic->mask ,
+		src_x , src_y , src_width , src_height) ;
+#endif
+
+	x_window_copy_area( window , cur_pic->pixmap , cur_pic->mask ,
+		src_x , src_y , src_width , src_height , dst_x , dst_y) ;
+
 	return  1 ;
 }
 #endif
@@ -1202,12 +1226,18 @@ xcore_draw_str(
 			}
 
 			if(
-			#ifndef  USE_FRAMEBUFFER
-			    ( x_window_has_wall_picture( window) &&
-			           bg_color == ML_BG_COLOR) ||
+		#ifdef  USE_FRAMEBUFFER
+			#ifdef  USE_FREETYPE
+			    xfont->is_proportional ||	/* ISCII */
 			#endif
-				bottom_margin + top_margin > 0 /* == line space XXX */ ||
-				draw_alone || state == 3)
+			    /* draw_alone || */ /* draw_alone is always false on framebuffer. */
+		#else
+			    ( x_window_has_wall_picture( window) &&
+			      bg_color == ML_BG_COLOR) ||
+			    draw_alone ||
+		#endif
+			    bottom_margin + top_margin > 0 /* == line space XXX */ ||
+			    state == 3)
 			{
 				if( bg_color == ML_BG_COLOR)
 				{
@@ -1216,10 +1246,6 @@ xcore_draw_str(
 				}
 				else
 				{
-				#ifdef  __DEBUG
-					kik_debug_printf( KIK_DEBUG_TAG "prop font is used.\n") ;
-				#endif
-
 					x_window_fill_with( window , bg_xcolor ,
 						x , y , current_width - x , height) ;
 				}
