@@ -11,6 +11,7 @@
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
 #include  <kiklib/kik_conf_io.h>
 #include  <kiklib/kik_file.h>
+#include  <kiklib/kik_debug.h>
 
 #include  <ml_char.h>
 #include  <ml_char_encoding.h>	/* ml_parse_unicode_area */
@@ -159,8 +160,13 @@ static cs_table_t  cs_table[] =
 static x_font_config_t **  font_configs ;
 static u_int  num_of_configs ;
 
+#ifdef  __ANDROID__
+static u_int  min_font_size = 10 ;
+static u_int  max_font_size = 40 ;
+#else
 static u_int  min_font_size = 6 ;
 static u_int  max_font_size = 30 ;
+#endif
 
 /*
  * These will be leaked unless operate_custom_cache( ... , 1 [remove]) deletes them.
@@ -173,9 +179,8 @@ static u_int  num_of_customs ;
 
 /* --- static functions --- */
 
-#ifdef  DEBUG
-static void  TEST_write_conf(void) ;
-static void  TEST_create_value(void) ;
+#ifdef  KIK_DEBUG
+static void  TEST_font_config(void) ;
 #endif
 
 
@@ -1259,11 +1264,6 @@ x_set_font_size_range(
 	min_font_size = min_fsize ;
 	max_font_size = max_fsize ;
 
-#ifdef  DEBUG
-	TEST_write_conf() ;
-	TEST_create_value() ;
-#endif
-	
 	return  1 ;
 }
 
@@ -1288,6 +1288,8 @@ x_acquire_font_config(
 	x_font_config_t *  font_config ;
 	void *  p ;
 
+	KIK_TESTIT_ONCE(font_config, ()) ;
+
 	if( ( font_config = find_font_config( type_engine , font_present)))
 	{
 		font_config->ref_count ++ ;
@@ -2105,7 +2107,11 @@ x_get_charset_name(
 	return  NULL ;
 }
 
-#ifdef  DEBUG
+
+#ifdef  KIK_DEBUG
+
+#include  <assert.h>
+
 static void
 TEST_write_conf(void)
 {
@@ -2131,6 +2137,7 @@ TEST_write_conf(void)
 		{
 			kik_debug_printf( KIK_DEBUG_TAG " Test failed(write conf(%s) => %s).\n" ,
 				data[count].data1 , p) ;
+			abort() ;
 		}
 	}
 }
@@ -2182,6 +2189,7 @@ TEST_create_value(void)
 				kik_debug_printf( KIK_DEBUG_TAG
 					" Test failed(create_value(%s + %s) => %s).\n" ,
 					data[count].data1 , data[count].data2 , ret) ;
+				abort() ;
 			}
 		}
 		else if( strcmp( data[count].ow_result , ret) != 0)
@@ -2189,6 +2197,7 @@ TEST_create_value(void)
 			kik_debug_printf( KIK_DEBUG_TAG
 				" Test failed(create_value(%s + %s) => %s).\n" ,
 				data[count].data1 , data[count].data2 , ret) ;
+			abort() ;
 		}
 		else if( is_changed != data[count].ow_changed)
 		{
@@ -2196,6 +2205,7 @@ TEST_create_value(void)
 				" Test failed(create_value(%s + %s) => %s : %s ?).\n" ,
 				data[count].data1 , data[count].data2 , ret ,
 				is_changed ? "is changed" : "is not changed") ;
+			abort() ;
 		}
 
 		free( ret) ;
@@ -2211,6 +2221,7 @@ TEST_create_value(void)
 				kik_debug_printf( KIK_DEBUG_TAG
 					" Test failed(create_value(%s - %s) => %s).\n" ,
 					data[count].data1 , data[count].data2 , ret) ;
+				abort() ;
 			}
 		}
 		else if( strcmp( data[count].rm_result , ret) != 0)
@@ -2218,6 +2229,7 @@ TEST_create_value(void)
 			kik_debug_printf( KIK_DEBUG_TAG
 				" Test failed(create_value(%s - %s) => %s).\n" ,
 				data[count].data1 , data[count].data2 , ret) ;
+			abort() ;
 		}
 		else if( is_changed != data[count].rm_changed)
 		{
@@ -2225,9 +2237,40 @@ TEST_create_value(void)
 				" Test failed(create_value(%s - %s) => %s : %s ?).\n" ,
 				data[count].data1 , data[count].data2 , ret ,
 				is_changed ? "is changed" : "is not changed") ;
+			abort() ;
 		}
 
 		free( ret) ;
 	}
 }
+
+static void
+TEST_font_config(void)
+{
+#if ! defined(USE_FRAMEBUFFER) && ! defined(USE_WIN32GUI)
+	x_font_config_t *  font_config ;
+	char *  value ;
+
+	font_config = x_font_config_new( TYPE_XCORE , 0) ;
+	x_customize_font_name( font_config , ISO8859_1_R , "-hoge-medium-r-fuga-" , 12) ;
+
+	value = x_get_config_font_name( font_config , 12 , ISO8859_1_R|FONT_BOLD) ;
+	assert( strcmp( "-hoge-bold-r-fuga-" , value) == 0) ;
+	free( value) ;
+
+	value = x_get_config_font_name( font_config , 12 , ISO8859_1_R|FONT_ITALIC) ;
+	assert( strcmp( "-hoge-medium-i-fuga-" , value) == 0) ;
+	free( value) ;
+
+	value = x_get_config_font_name( font_config , 12 , ISO8859_1_R|FONT_BOLD|FONT_ITALIC) ;
+	assert( strcmp( "-hoge-bold-i-fuga-" , value) == 0) ;
+	free( value) ;
+
+	x_font_config_delete( font_config) ;
+
+	TEST_create_value() ;
+	TEST_write_conf() ;
+#endif
+}
+
 #endif
