@@ -8,6 +8,7 @@
 #include  <kiklib/kik_mem.h>	/* malloc/realloc */
 #include  <kiklib/kik_str.h>	/* kik_str_to_uint */
 #include  <kiklib/kik_locale.h>	/* kik_get_lang */
+#include  <kiklib/kik_args.h>
 
 #include  "ml_term_manager.h"
 
@@ -64,12 +65,12 @@ x_prepare_for_main_config(
 		"override redirect [false]") ;
 	kik_conf_add_opt( conf , '*' , "type" , 0 , "type_engine" ,
 		"type engine "
-	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
-		"[xcore]"
-	#elif  ! defined(USE_TYPE_XFT) && defined(USE_TYPE_CAIRO)
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+		"[xft]"
+	#elif  ! defined(USE_TYPE_XCORE) && defined(USE_TYPE_CAIRO)
 		"[cairo]"
 	#else
-		"[xft]"
+		"[xcore]"
 	#endif
 		) ;
 #endif	/* USE_WIN32GUI/USE_FRAMEBUFFER */
@@ -102,13 +103,9 @@ x_prepare_for_main_config(
 #endif
 	kik_conf_add_opt( conf , 'B' , "sbbg" , 0 , "sb_bg_color" , 
 		"scrollbar background color") ;
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
-	kik_conf_add_opt( conf , 'C' , "ind" , 1 , "use_ind" , 
-		"use indic (ligature text) [false]") ;
-#endif
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
-	kik_conf_add_opt( conf , 'D' , "bi" , 1 , "use_bidi" ,
-		"use bidi (bi-directional text) [true]") ;
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+	kik_conf_add_opt( conf , 'C' , "ctl" , 1 , "use_ctl" ,
+		"use complex text layouting [true]") ;
 #endif
 	kik_conf_add_opt( conf , 'E' , "km" , 0 , "encoding" ,
 		"character encoding (AUTO/ISO-8859-*/EUC-*/UTF-8/...) [AUTO]") ;
@@ -137,7 +134,7 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , 'O' , "sbmod" , 0 , "scrollbar_mode" ,
 		"scrollbar mode (none/left/right) [none]") ;
 	kik_conf_add_opt( conf , 'P' , "clip" , 1 , "use_clipboard" ,
-		"use CLIPBOARD (not only PRIMARY) selection [false]") ;
+		"use CLIPBOARD (not only PRIMARY) selection [true]") ;
 	kik_conf_add_opt( conf , 'Q' , "vcur" , 1 , "use_vertical_cursor" ,
 		"rearrange cursor key for vertical mode [false]") ;
 	kik_conf_add_opt( conf , 'S' , "sbview" , 0 , "scrollbar_view_name" , 
@@ -184,8 +181,12 @@ x_prepare_for_main_config(
 		"use extended scroll shortcut keys [false]") ;
 	kik_conf_add_opt( conf , 'r' , "fade" , 0 , "fade_ratio" , 
 		"fade ratio in percent when window unfocued [100]") ;
-	kik_conf_add_opt( conf , 's' , "sb" , 1 , "use_scrollbar" , 
+	kik_conf_add_opt( conf , 's' , "mdi" , 1 , "use_mdi" ,
+		"use multiple document interface [true]") ;
+#if  1
+	kik_conf_add_opt( conf , '\0' , "sb" , 1 , "use_scrollbar" ,
 		"use scrollbar [true]") ;
+#endif
 	kik_conf_add_opt( conf , 't' , "transbg" , 1 , "use_transbg" , 
 		"use transparent background [false]") ;
 	kik_conf_add_opt( conf , 'u' , "onlyucsfont" , 1 , "only_use_unicode_font" ,
@@ -480,10 +481,10 @@ x_main_config_init(
 		}
 	}
 
-#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
-	main_config->type_engine = TYPE_XCORE ;
-#elif   defined(USE_TYPE_XFT)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	main_config->type_engine = TYPE_XFT ;
+#elif   defined(USE_TYPE_XCORE)
+	main_config->type_engine = TYPE_XCORE ;
 #else
 	main_config->type_engine = TYPE_CAIRO ;
 #endif
@@ -762,14 +763,11 @@ x_main_config_init(
 		}
 	}
 
-	main_config->use_scrollbar = 1 ;
+	main_config->use_mdi = 1 ;
 
-	if( ( value = kik_conf_get_value( conf , "use_scrollbar")))
+	if( ( value = kik_conf_get_value( conf , "use_mdi")))
 	{
-		if( strcmp( value , "false") == 0)
-		{
-			main_config->use_scrollbar = 0 ;
-		}
+		main_config->use_mdi = (strcmp( value , "true") == 0) ;
 	}
 
 	if( ( value = kik_conf_get_value( conf , "scrollbar_mode")))
@@ -778,7 +776,18 @@ x_main_config_init(
 	}
 	else
 	{
-		main_config->sb_mode = SBM_LEFT ;
+		/* XXX Backward compatibility with 3.4.5 or before */
+	#if  1
+		if( ( value = kik_conf_get_value( conf , "use_scrollbar")))
+		{
+			main_config->sb_mode =
+				(strcmp( value , "true") == 0 ? SBM_LEFT : SBM_NONE) ;
+		}
+		else
+	#endif
+		{
+			main_config->sb_mode = SBM_LEFT ;
+		}
 	}
 
 	if( ( value = kik_conf_get_value( conf , "iso88591_font_for_usascii")))
@@ -826,6 +835,12 @@ x_main_config_init(
 			}
 		}
 	}
+#ifdef  __ANDROID__
+	else
+	{
+		main_config->unicode_policy = ONLY_USE_UNICODE_FONT ;
+	}
+#endif
 
 	if( ( value = kik_conf_get_value( conf , "box_drawing_font")))
 	{
@@ -838,6 +853,12 @@ x_main_config_init(
 			main_config->unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT ;
 		}
 	}
+#ifdef  __ANDROID__
+	else
+	{
+		main_config->unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT ;
+	}
+#endif
 
 	if( ( value = kik_conf_get_value( conf , "receive_string_via_ucs")))
 	{
@@ -1004,29 +1025,16 @@ x_main_config_init(
 		main_config->encoding = ML_ISO8859_1 ;
 	}
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
-	if( ( value = kik_conf_get_value( conf , "use_ind")))
-	{
-		if( strcmp( value , "true") == 0)
-		{
-			main_config->use_ind = 1 ;
-		}
-	}
-#endif
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI) || defined(USE_IND)
+	main_config->use_ctl = 1 ;
 
-#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
-	if( ( value = kik_conf_get_value( conf , "use_bidi")))
+	if( ( value = kik_conf_get_value( conf , "use_ctl")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , "false") == 0)
 		{
-			main_config->use_bidi = 1 ;
-			main_config->use_ind = 0 ;
+			main_config->use_ctl = 0 ;
 		}
 	}
-	else if( ! main_config->use_ind)
-	{
-		main_config->use_bidi = 1 ;
-	}
 #endif
 
 	main_config->bidi_mode = BIDI_NORMAL_MODE ;
@@ -1364,7 +1372,10 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_clipboard")))
 	{
-		x_set_use_clipboard_selection( strcmp( value , "true") == 0) ;
+		if( strcmp( value , "false") == 0)
+		{
+			x_set_use_clipboard_selection( 0) ;
+		}
 	}
 
 	if( ! ( value = kik_conf_get_value( conf , "auto_restart")) ||
@@ -1461,20 +1472,37 @@ x_main_config_init(
 	}
 #endif
 
-	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , "true") == 0)
+	if( ( value = kik_conf_get_value( conf , "exec_cmd")))
 	{
-		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))))
+		if( strcmp( value , "true") == 0)
 		{
-			/*
-			 * !! Notice !!
-			 * cmd_path and strings in cmd_argv vector should be allocated
-			 * by the caller.
-			 */
-			 
-			main_config->cmd_path = argv[0] ;
-			
-			memcpy( &main_config->cmd_argv[0] , argv , sizeof( char*) * argc) ;
-			main_config->cmd_argv[argc] = NULL ;
+			if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))))
+			{
+				/*
+				 * !! Notice !!
+				 * cmd_path and strings in cmd_argv vector should be allocated
+				 * by the caller.
+				 */
+
+				main_config->cmd_path = argv[0] ;
+
+				memcpy( &main_config->cmd_argv[0] , argv , sizeof( char*) * argc) ;
+				main_config->cmd_argv[argc] = NULL ;
+			}
+		}
+		else
+		{
+			u_int  argc ;
+
+			argc = kik_count_char_in_str( value , ' ') + 1 ;
+
+			if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1) +
+							strlen( value) + 1)))
+			{
+				value = strcpy( main_config->cmd_argv + argc + 1 , value) ;
+				_kik_arg_str_to_array( main_config->cmd_argv , &argc , value) ;
+				main_config->cmd_path = main_config->cmd_argv[0] ;
+			}
 		}
 	}
 
