@@ -10,7 +10,7 @@
 #include  <kiklib/kik_conf.h>
 #include  <ml_term.h>
 
-#include  "x_sb_screen.h"
+#include  "x_layout.h"
 
 
 typedef struct x_main_config
@@ -90,10 +90,9 @@ typedef struct x_main_config
 	u_int8_t  fade_ratio ;
 	u_int8_t  line_space ;
 	u_int8_t  letter_space ;
-	int8_t  use_scrollbar ;
+	int8_t  use_mdi ;
 	int8_t  use_login_shell ;
-	int8_t  use_bidi ;
-	int8_t  use_ind ;
+	int8_t  use_ctl ;
 	int8_t  big5_buggy ;
 	int8_t  iso88591_font_for_usascii ;
 	int8_t  receive_string_via_ucs ;
