@@ -24,12 +24,12 @@
  * Threading is not supported for 8 or less bpp framebuffer imaging
  * because of x_display_enable_to_change_cmap() and x_display_set_cmap().
  */
-#if ! defined(__CYGWIN__) || ! defined(USE_WIN32GUI)
-#undef  HAVE_PTHREAD
+#if  (defined(__CYGWIN__) && defined(USE_WIN32GUI)) || defined(__ANDROID__)
+#ifndef  HAVE_PTHREAD
+#define  HAVE_PTHREAD
 #endif
-
-#ifdef  X_PROTOCOL
-#undef  HAVE_WINDOWS_H
+#else
+#undef  HAVE_PTHREAD
 #endif
 
 #if  ! defined(USE_WIN32API) && defined(HAVE_PTHREAD)
@@ -40,6 +40,10 @@
 #endif
 #endif
 
+#ifdef  X_PROTOCOL
+#undef  HAVE_WINDOWS_H
+#endif
+
 #include  "x_imagelib.h"
 
 
@@ -464,7 +468,14 @@ cleanup_inline_pictures(
 
 		memset( flags , 0 , num_of_inline_pics) ;
 
-		beg = - ml_term_get_num_of_logged_lines( term) ;
+		/*
+		 * Inline pictures in back logs except recent MAX_INLINE_PICTURES*2 lines
+		 * are deleted in line_scrolled_out() in x_screen.c.
+		 */
+		if( ( beg = -ml_term_get_num_of_logged_lines( term)) < INLINEPIC_AVAIL_ROW)
+		{
+			beg = INLINEPIC_AVAIL_ROW ;
+		}
 		end = ml_term_get_rows( term) ;
 		restore_alt_edit = 0 ;
 
@@ -679,7 +690,7 @@ load_file_async(
 static int
 ensure_inline_picture(
 	x_display_t *  disp ,
-	char *  file_path ,
+	const char *  file_path ,
 	u_int *  width ,	/* can be 0 */
 	u_int *  height ,	/* can be 0 */
 	u_int  col_width ,
@@ -875,11 +886,12 @@ x_acquire_bg_picture(
 	char *  file_path		/* "root" means transparency. */
 	)
 {
-	u_int  count ;
 	x_picture_t **  p ;
 
 	if( strcmp( file_path , "root") != 0) /* Transparent background is not cached. */
 	{
+		u_int  count ;
+
 		for( count = 0 ; count < num_of_pics ; count++)
 		{
 			if( strcmp( file_path , pics[count]->file_path) == 0 &&
@@ -1055,7 +1067,8 @@ x_load_inline_picture(
 	inline_pic_args_t *  args ;
 
 	/* XXX Don't reuse ~/.mlterm/[pty name].six, [pty name].rgs and anim-*.gif */
-	if( ! strstr( file_path , "mlterm/") || strstr( file_path , "mlterm/macro"))
+	if( ! strstr( file_path , "mlterm/") || strstr( file_path , "mlterm/macro") ||
+	    strstr( file_path , "mlterm/emoji/"))
 	{
 		for( idx = 0 ; idx < num_of_inline_pics ; idx++)
 		{
@@ -1171,7 +1184,18 @@ x_load_inline_picture(
 	{
 		int  ret ;
 
+	#ifdef  BUILTIN_IMAGELIB
 		ret = load_file(args) ;
+	#else
+		struct stat  st ;
+
+		ret =  (
+		#if  ! defined(HAVE_PTHREAD) && ! defined(USE_WIN32API)
+			strstr( file_path , "://") ||
+		#endif
+			stat( file_path , &st) == 0) && load_file(args) ;
+	#endif
+
 		free( args) ;
 
 		if( ret)
@@ -1379,4 +1403,38 @@ x_animate_inline_pictures(
 	return  wait ;
 }
 
+int
+x_load_tmp_picture(
+	x_display_t *  disp ,
+	char *  file_path ,
+	Pixmap *  pixmap ,
+	PixmapMask *  mask ,
+	u_int *  width ,
+	u_int *  height
+	)
+{
+	*width = *height = 0 ;
+
+	if( x_imagelib_load_file( disp , file_path , NULL , pixmap , mask ,
+			width , height))
+	{
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
+void
+x_delete_tmp_picture(
+	x_display_t *  disp ,
+	Pixmap  pixmap ,
+	PixmapMask  mask
+	)
+{
+	x_delete_image( disp->display , pixmap) ;
+	x_delete_mask( disp->display , mask) ;
+}
+
 #endif	/* NO_IMAGE */
