@@ -73,6 +73,7 @@ typedef struct x_inline_picture
 
 #define  MAX_INLINE_PICTURES  (1 << PICTURE_ID_BITS)
 #define  MAKE_INLINEPIC_POS(col , row , num_of_rows)  ((col) * (num_of_rows) + (row))
+#define  INLINEPIC_AVAIL_ROW  -(MAX_INLINE_PICTURES * 2)
 
 #ifdef  NO_IMAGE
 
@@ -88,6 +89,9 @@ typedef struct x_inline_picture
 
 #else
 
+/* defined in c_sixel.c */
+u_int32_t *  x_set_custom_sixel_palette( u_int32_t *  palette) ;
+
 int  x_picture_display_opened( Display *  display) ;
 
 int  x_picture_display_closed( Display *  display) ;
@@ -113,6 +117,11 @@ int  x_add_frame_to_animation( int  prev_idx , int  next_idx) ;
 
 int  x_animate_inline_pictures( ml_term_t *  term) ;
 
+int  x_load_tmp_picture( x_display_t *  disp , char *  file_path , Pixmap *  pixmap ,
+	PixmapMask *  mask , u_int *  width , u_int *  height) ;
+
+void  x_delete_tmp_picture( x_display_t *  disp , Pixmap  pixmap , PixmapMask  mask) ;
+
 #endif
 
 #define  x_picture_modifier_is_normal(pic_mod) (x_picture_modifiers_equal((pic_mod), NULL))
