@@ -14,7 +14,6 @@
 #include  "x_window.h"
 #include  "x_selection.h"
 #include  "x_shortcut.h"
-#include  "x_termcap.h"
 #include  "x_mod_meta_mode.h"
 #include  "x_bel_mode.h"
 #include  "x_sb_mode.h"
@@ -29,7 +28,11 @@ typedef struct x_system_event_listener
 	void *  self ;
 
 	void  (*open_screen)( void * , x_screen_ptr_t) ;
-	void  (*close_screen)( void * , x_screen_ptr_t) ;
+	void  (*split_screen)( void * , x_screen_ptr_t , int , const char *) ;
+	int  (*close_screen)( void * , x_screen_ptr_t , int) ;
+	int  (*next_screen)( void * , x_screen_ptr_t) ;
+	int  (*prev_screen)( void * , x_screen_ptr_t) ;
+	int  (*resize_screen)( void * , x_screen_ptr_t , int , int) ;
 	void  (*open_pty)( void * , x_screen_ptr_t , char *) ;
 	void  (*next_pty)( void * , x_screen_ptr_t) ;
 	void  (*prev_pty)( void * , x_screen_ptr_t) ;
@@ -93,7 +96,6 @@ typedef struct  x_screen
 	x_im_event_listener_t  im_listener ;
 
 	x_shortcut_t *  shortcut ;
-	x_termcap_entry_t *  termcap ;
 
 	char *  input_method ;
 	x_im_t *  im ;
@@ -133,7 +135,8 @@ typedef struct  x_screen
 
 	int16_t  prev_inline_pic ;
 
-	char  prev_mouse_report_seq[5] ;
+	u_int16_t  prev_mouse_report_col ;
+	u_int16_t  prev_mouse_report_row ;
 
 	u_int8_t  fade_ratio ;
 	u_int8_t  line_space ;
@@ -162,16 +165,15 @@ void  x_set_im_cursor_color( char *  color) ;
 #endif
 
 x_screen_t *  x_screen_new( ml_term_t *  term , x_font_manager_t *  font_man ,
-	x_color_manager_t *  color_man , x_termcap_entry_t *  termcap ,
-	u_int  brightness , u_int contrast , u_int gamma , u_int  alpha ,
-	u_int  fade_ratio , x_shortcut_t *  shortcut ,
+	x_color_manager_t *  color_man , u_int  brightness , u_int contrast ,
+	u_int gamma , u_int  alpha , u_int  fade_ratio , x_shortcut_t *  shortcut ,
 	u_int  screen_width_ratio , u_int  screen_height_ratio ,
 	char *  mod_meta_key , x_mod_meta_mode_t  mod_meta_mode ,
 	x_bel_mode_t  bel_mode , int  receive_string_via_ucs , char *  pic_file_path ,
 	int  use_transbg , int  use_vertical_cursor , int  big5_buggy ,
 	int  use_extended_scroll_shortcut , int  borderless , u_int  line_space ,
 	char *  input_method , int  allow_osc52 , int  blink_cursor ,
-	int  hmargin , int  vmargin , int  hide_underline) ;
+	u_int  hmargin , u_int  vmargin , int  hide_underline) ;
 
 int  x_screen_delete( x_screen_t *  screen) ;
 
@@ -213,6 +215,10 @@ int  x_screen_set_config( x_screen_t *  screen , char *  dev , char *  key , cha
 
 int  x_screen_reset_view( x_screen_t *  screen) ;
 
+#if  defined(USE_FRAMEBUFFER) && (defined(__NetBSD__) || defined(__OpenBSD__))
+void  x_screen_reload_color_cache( x_screen_t *  screen , int  do_unload) ;
+#endif
+
 
 x_picture_modifier_t *  x_screen_get_picture_modifier( x_screen_t *  screen) ;
 
