@@ -27,9 +27,8 @@
 #include  <ml_term_manager.h>
 #include  <ml_char_encoding.h>
 
-#include  "x_sb_screen.h"
+#include  "x_layout.h"
 #include  "x_display.h"
-#include  "x_termcap.h"
 
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 #include  "x_connect_dialog.h"
@@ -62,7 +61,6 @@ static x_system_event_listener_t  system_listener ;
 static x_main_config_t  main_config ;
 
 static x_shortcut_t  shortcut ;
-static x_termcap_t  termcap ;
 
 
 /* --- static functions --- */
@@ -128,16 +126,14 @@ create_term_intern(void)
 {
 	ml_term_t *  term ;
 	
-	if( ( term = ml_create_term( main_config.cols , main_config.rows ,
+	if( ( term = ml_create_term( main_config.term_type , main_config.cols , main_config.rows ,
 			main_config.tab_size , main_config.num_of_log_lines ,
 			main_config.encoding , main_config.is_auto_encoding ,
 			main_config.use_auto_detect , main_config.logging_vt_seq ,
 			main_config.unicode_policy , main_config.col_size_of_width_a ,
 			main_config.use_char_combining , main_config.use_multi_col_char ,
-			main_config.use_bidi , main_config.bidi_mode ,
-			main_config.bidi_separators , main_config.use_ind ,
-			x_termcap_get_bool_field(
-				x_termcap_get_entry( &termcap , main_config.term_type) , ML_BCE) ,
+			main_config.use_ctl , main_config.bidi_mode ,
+			main_config.bidi_separators ,
 			main_config.use_dynamic_comb , main_config.bs_mode ,
 			main_config.vertical_mode , main_config.use_local_echo ,
 			main_config.title , main_config.icon_name ,
@@ -165,7 +161,9 @@ open_pty_intern(
 	char *  cmd_path ,
 	char **  cmd_argv ,
 	char *  display ,
-	Window  window
+	Window  window ,
+	u_int  width_pix ,
+	u_int  height_pix
 	)
 {
 	char *  env[6] ;	/* MLTERM,TERM,WINDOWID,DISPLAY,COLORFGBG,NULL */
@@ -379,11 +377,11 @@ open_pty_intern(
 	ret = ml_term_open_pty( term , cmd_path , cmd_argv , env , uri ? uri : display ,
 			main_config.work_dir , pass ,
 		#ifdef  USE_LIBSSH2
-			main_config.public_key , main_config.private_key
+			main_config.public_key , main_config.private_key ,
 		#else
-			NULL , NULL
+			NULL , NULL ,
 		#endif
-			) ;
+			width_pix , height_pix) ;
 
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	if( uri)
@@ -414,7 +412,8 @@ get_picture_data(
 	void *  p ,
 	char *  file_path ,
 	int *  num_of_cols ,	/* can be 0 */
-	int *  num_of_rows	/* can be 0 */
+	int *  num_of_rows ,	/* can be 0 */
+	u_int32_t **  sixel_palette
 	)
 {
 	ml_char_t *  data ;
@@ -427,7 +426,7 @@ get_picture_data(
 		screens[0]->term = p ;		/* XXX */
 		data = (*screens[0]->xterm_listener.get_picture_data)(
 				screens[0]->xterm_listener.self ,
-				file_path , num_of_cols , num_of_rows) ;
+				file_path , num_of_cols , num_of_rows , sixel_palette) ;
 		screens[0]->term = orig_term ;
 	}
 	else
@@ -474,7 +473,42 @@ detach_screen(
 #endif
 
 
-static int
+#ifdef  USE_WIN32GUI
+static void
+close_screen_win32(
+	x_screen_t *  screen
+	)
+{
+	int  is_orphan ;
+
+	if( X_SCREEN_TO_LAYOUT(screen) &&
+	    x_layout_remove_child( X_SCREEN_TO_LAYOUT(screen) , screen))
+	{
+		is_orphan = 1 ;
+	}
+	else
+	{
+		is_orphan = 0 ;
+
+		/*
+		 * XXX Hack
+		 * In case SendMessage(WM_CLOSE) causes WM_KILLFOCUS
+		 * and operates screen->term which was already deleted.
+		 * (see window_unfocused())
+		 */
+		screen->window.window_unfocused = NULL ;
+	}
+
+	SendMessage( x_get_root_window( &screen->window)->my_window , WM_CLOSE , 0 , 0) ;
+
+	if( is_orphan && screen->window.window_deleted)
+	{
+		(*screen->window.window_deleted)( &screen->window) ;
+	}
+}
+#endif
+
+static void
 close_screen_intern(
 	x_screen_t *  screen
 	)
@@ -482,6 +516,11 @@ close_screen_intern(
 	x_window_t *  root ;
 	x_display_t *  disp ;
 
+	if( X_SCREEN_TO_LAYOUT(screen))
+	{
+		x_layout_remove_child( X_SCREEN_TO_LAYOUT(screen) , screen) ;
+	}
+
 	x_screen_detach( screen) ;
 	x_font_manager_delete( screen->font_man) ;
 	x_color_manager_delete( screen->color_man) ;
@@ -489,25 +528,27 @@ close_screen_intern(
 	root = x_get_root_window( &screen->window) ;
 	disp = root->disp ;
 
-	x_display_remove_root( disp, root) ;
-
-	if( disp->num_of_roots == 0)
+	if( ! x_display_remove_root( disp , root))
+	{
+		x_window_unmap( root) ;
+		x_window_final( root) ;
+	}
+	else if( disp->num_of_roots == 0)
 	{
 		x_display_close( disp) ;
 	}
-
-	return  1 ;
 }
 
 static x_screen_t *
 open_screen_intern(
-	char *  pty
+	ml_term_t *  term ,
+	x_layout_t *  layout ,
+	int  horizontal ,
+	const char *  sep
 	)
 {
-	ml_term_t *  term ;
 	x_display_t *  disp ;
 	x_screen_t *  screen ;
-	x_sb_screen_t *  sb_screen ;
 	x_font_manager_t *  font_man ;
 	x_color_manager_t *  color_man ;
 	x_window_t *  root ;
@@ -521,7 +562,6 @@ open_screen_intern(
 	font_man = NULL ;
 	color_man = NULL ;
 	screen = NULL ;
-	sb_screen = NULL ;
 	root = NULL ;
 
 	if( MAX_SCREENS <= num_of_screens)
@@ -529,21 +569,10 @@ open_screen_intern(
 		return  NULL ;
 	}
 
-	if( pty)
+	if( ! term)
 	{
-		if( ( term = ml_get_detached_term( pty)) == NULL)
-		{
-			return  NULL ;
-		}
-	}
-	else
-	{
-	#ifdef  __ANDROID__
-		if( ( term = ml_get_detached_term( NULL)) == NULL &&
-			( term = create_term_intern()) == NULL)
-	#else
-		if( ( term = create_term_intern()) == NULL)
-	#endif
+		if( ( ! layout || ( term = ml_get_detached_term( NULL)) == NULL) &&
+		    ( term = create_term_intern()) == NULL)
 		{
 			return  NULL ;
 		}
@@ -605,7 +634,6 @@ open_screen_intern(
 	}
 
 	if( ( screen = x_screen_new( term , font_man , color_man ,
-			x_termcap_get_entry( &termcap , main_config.term_type) ,
 			main_config.brightness , main_config.contrast , main_config.gamma ,
 			main_config.alpha , main_config.fade_ratio , &shortcut ,
 			main_config.screen_width_ratio , main_config.screen_height_ratio ,
@@ -631,27 +659,41 @@ open_screen_intern(
 
 	x_set_system_listener( screen , &system_listener) ;
 
-	if( main_config.use_scrollbar &&
-	    ( sb_screen = x_sb_screen_new( screen ,
-				main_config.scrollbar_view_name ,
-				main_config.sb_fg_color , main_config.sb_bg_color ,
-				main_config.sb_mode)))
+	if( layout)
 	{
-		root = &sb_screen->window ;
+		if( ! x_layout_add_child( layout , screen , horizontal , sep))
+		{
+			layout = NULL ;
+
+			goto  error ;
+		}
+
+		root = &layout->window ;
 	}
 	else
 	{
-		root = &screen->window ;
-	}
+		if( main_config.use_mdi &&
+		    ( layout = x_layout_new( screen ,
+					main_config.scrollbar_view_name ,
+					main_config.sb_fg_color , main_config.sb_bg_color ,
+					main_config.sb_mode)))
+		{
+			root = &layout->window ;
+		}
+		else
+		{
+			root = &screen->window ;
+		}
 
-	if( ! x_display_show_root( disp, root, main_config.x, main_config.y,
-		main_config.geom_hint, main_config.app_name , main_config.parent_window))
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " x_display_show_root() failed.\n") ;
-	#endif
-	
-		goto  error ;
+		if( ! x_display_show_root( disp, root, main_config.x, main_config.y,
+			main_config.geom_hint, main_config.app_name , main_config.parent_window))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " x_display_show_root() failed.\n") ;
+		#endif
+
+			goto  error ;
+		}
 	}
 
 	if( ( p = realloc( screens , sizeof( x_screen_t*) * (num_of_screens + 1))) == NULL)
@@ -671,7 +713,7 @@ open_screen_intern(
 	 * New screen is successfully created here except ml_pty.
 	 */
 
-	if( pty)
+	if( ml_term_pty_is_opened( term))
 	{
 	#if  0
 		/* mlclient /dev/... -e foo */
@@ -692,21 +734,15 @@ open_screen_intern(
 	else
 	{
 		if( ! open_pty_intern( term , main_config.cmd_path , main_config.cmd_argv ,
-			DisplayString( disp->display) , root->my_window))
+			DisplayString( disp->display) , root->my_window ,
+			screen->window.width , screen->window.height))
 		{
 			x_screen_detach( screen) ;
 			ml_destroy_term( term) ;
 
 		#ifdef  USE_WIN32GUI
-			/*
-			 * XXX Hack
-			 * In case SendMessage(WM_CLOSE) causes WM_KILLFOCUS
-			 * and operates screen->term which was already deleted.
-			 * (see window_unfocused())
-			 */
 			screens[num_of_screens++] = screen ;
-			screen->window.window_unfocused = NULL ;
-			SendMessage( root->my_window , WM_CLOSE , 0 , 0) ;
+			close_screen_win32( screen) ;
 		#else
 			close_screen_intern( screen) ;
 		#endif
@@ -748,9 +784,9 @@ error:
 			x_screen_delete( screen) ;
 		}
 
-		if( sb_screen)
+		if( layout)
 		{
-			x_sb_screen_delete( sb_screen) ;
+			x_layout_delete( layout) ;
 		}
 	}
 
@@ -829,9 +865,13 @@ open_pty(
 	}
 	else
 	{
+		ml_char_encoding_t  encoding ;
 	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 		char *  default_server ;
 		int  show_dialog ;
+		char *  new_cmd_line ;
+		char *  cmd_path ;
+		char **  cmd_argv ;
 	#endif
 		int  ret ;
 
@@ -840,6 +880,8 @@ open_pty(
 			return ;
 		}
 
+		encoding = main_config.encoding ;
+		main_config.encoding = ml_term_get_encoding( screen->term) ;
 	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 		default_server = main_config.default_server ;
 		main_config.default_server = ml_term_get_uri( screen->term) ;
@@ -849,16 +891,36 @@ open_pty(
 		 */
 		show_dialog = main_config.show_dialog ;
 		main_config.show_dialog = 0 ;
+
+		if( ( new_cmd_line = ml_term_get_cmd_line( screen->term)) &&
+		    ( new_cmd_line = kik_str_alloca_dup( new_cmd_line)))
+		{
+			int  argc ;
+
+			cmd_path = main_config.cmd_path ;
+			cmd_argv = main_config.cmd_argv ;
+
+			main_config.cmd_argv = kik_arg_str_to_array( &argc , new_cmd_line) ;
+			main_config.cmd_path = main_config.cmd_argv[0] ;
+		}
 	#endif
 
 		ret = open_pty_intern( new , main_config.cmd_path ,
 			main_config.cmd_argv ,
 			DisplayString( screen->window.disp->display) ,
-			x_get_root_window( &screen->window)->my_window) ;
+			x_get_root_window( &screen->window)->my_window ,
+			screen->window.width , screen->window.height) ;
 
+		main_config.encoding = encoding ;
 	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 		main_config.default_server = default_server ;
 		main_config.show_dialog = show_dialog ;
+
+		if( new_cmd_line)
+		{
+			main_config.cmd_path = cmd_path ;
+			main_config.cmd_argv = cmd_argv ;
+		}
 	#endif
 
 		if( ! ret)
@@ -973,17 +1035,9 @@ pty_closed(
 			#ifdef  __DEBUG
 				kik_debug_printf( " no detached term. closing screen.\n") ;
 			#endif
-			
+
 			#ifdef  USE_WIN32GUI
-				/*
-				 * XXX Hack
-				 * In case SendMessage(WM_CLOSE) causes WM_KILLFOCUS
-				 * and operates screen->term which was already deleted.
-				 * (see window_unfocused())
-				 */
-				screen->window.window_unfocused = NULL ;
-				SendMessage( x_get_root_window( &screen->window)->my_window,
-					WM_CLOSE, 0, 0) ;
+				close_screen_win32( screen) ;
 			#else
 				screens[count] = screens[--num_of_screens] ;
 				close_screen_intern( screen) ;
@@ -1004,19 +1058,27 @@ pty_closed(
 }
 
 static void
-open_screen(
-	void *  p ,
-	x_screen_t *  cur_screen	/* Screen which triggers this event. */
+open_or_split_screen(
+	x_screen_t *  cur_screen ,	/* Screen which triggers this event. */
+	x_layout_t *  layout ,
+	int  horizontal ,
+	const char *  sep
 	)
 {
 	char *  disp_name ;
+	ml_char_encoding_t  encoding ;
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	char *  default_server ;
 	int  show_dialog ;
+	char *  new_cmd_line ;
+	char *  cmd_path ;
+	char **  cmd_argv ;
 #endif
 
 	disp_name = main_config.disp_name ;
 	main_config.disp_name = cur_screen->window.disp->name ;
+	encoding = main_config.encoding ;
+	main_config.encoding = ml_term_get_encoding( cur_screen->term) ;
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	default_server = main_config.default_server ;
 	main_config.default_server = ml_term_get_uri( cur_screen->term) ;
@@ -1026,9 +1088,21 @@ open_screen(
 	 */
 	show_dialog = main_config.show_dialog ;
 	main_config.show_dialog = 0 ;
+
+	if( ( new_cmd_line = ml_term_get_cmd_line( cur_screen->term)) &&
+	    ( new_cmd_line = kik_str_alloca_dup( new_cmd_line)))
+	{
+		int  argc ;
+
+		cmd_path = main_config.cmd_path ;
+		cmd_argv = main_config.cmd_argv ;
+
+		main_config.cmd_argv = kik_arg_str_to_array( &argc , new_cmd_line) ;
+		main_config.cmd_path = main_config.cmd_argv[0] ;
+	}
 #endif
 
-	if( ! open_screen_intern( NULL))
+	if( ! open_screen_intern( NULL , layout , horizontal , sep))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " open_screen_intern failed.\n") ;
@@ -1036,20 +1110,58 @@ open_screen(
 	}
 
 	main_config.disp_name = disp_name ;
+	main_config.encoding = encoding ;
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	main_config.default_server = default_server ;
 	main_config.show_dialog = show_dialog ;
+
+	if( new_cmd_line)
+	{
+		main_config.cmd_path = cmd_path ;
+		main_config.cmd_argv = cmd_argv ;
+	}
 #endif
 }
 
 static void
-close_screen(
+open_screen(
 	void *  p ,
 	x_screen_t *  screen	/* Screen which triggers this event. */
 	)
+{
+	open_or_split_screen( screen , NULL , 0 , 0) ;
+}
+
+static void
+split_screen(
+	void *  p ,
+	x_screen_t *  screen ,	/* Screen which triggers this event. */
+	int  horizontal ,
+	const char *  sep
+	)
+{
+	if( X_SCREEN_TO_LAYOUT(screen))
+	{
+		open_or_split_screen( screen , X_SCREEN_TO_LAYOUT(screen) , horizontal , sep) ;
+	}
+}
+
+static int
+close_screen(
+	void *  p ,
+	x_screen_t *  screen ,	/* Screen which triggers this event. */
+	int  force
+	)
 {
 	u_int  count ;
-	
+
+	if( ! force &&
+	    ( ! X_SCREEN_TO_LAYOUT(screen) ||
+	      x_layout_has_one_child( X_SCREEN_TO_LAYOUT(screen))))
+	{
+		return  0 ;
+	}
+
 	for( count = 0 ; count < num_of_screens ; count ++)
 	{
 		u_int  idx ;
@@ -1067,7 +1179,59 @@ close_screen(
 		idx = count / MSU ;	/* count / 8 */
 		dead_mask[idx] |= (1 << (count - MSU * idx)) ;
 
-		return ;
+		break ;
+	}
+
+	return  1 ;
+}
+
+static int
+next_screen(
+	void *  self ,
+	x_screen_t *  screen
+	)
+{
+	if( X_SCREEN_TO_LAYOUT(screen))
+	{
+		return  x_layout_switch_screen( X_SCREEN_TO_LAYOUT(screen) , 0) ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
+static int
+prev_screen(
+	void *  self ,
+	x_screen_t *  screen
+	)
+{
+	if( X_SCREEN_TO_LAYOUT(screen))
+	{
+		return  x_layout_switch_screen( X_SCREEN_TO_LAYOUT(screen) , 1) ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
+static int
+resize_screen(
+	void *  self ,
+	x_screen_t *  screen ,
+	int  horizontal ,
+	int  step
+	)
+{
+	if( X_SCREEN_TO_LAYOUT(screen))
+	{
+		return  x_layout_resize( X_SCREEN_TO_LAYOUT(screen) , screen , horizontal , step) ;
+	}
+	else
+	{
+		return  0 ;
 	}
 }
 
@@ -1117,7 +1281,7 @@ mlclient(
 		num = ml_get_all_terms( &terms) ;
 		for( count = 0 ; count < num ; count ++)
 		{
-			fprintf( fp , "%s" , ml_term_get_slave_name( terms[count])) ;
+			fprintf( fp , "#%s" , ml_term_get_slave_name( terms[count])) ;
 			if( ml_term_window_name( terms[count]))
 			{
 				fprintf( fp , "(whose title is %s)" ,
@@ -1188,7 +1352,10 @@ mlclient(
 		}
 		else
 		{
-			if( ! open_screen_intern( pty))
+			ml_term_t *  term = NULL ;
+
+			if( ( pty && ! ( term = ml_get_detached_term( pty))) ||
+			    ! open_screen_intern( term , NULL , 0 , 0))
 			{
 			#ifdef  DEBUG
 				kik_warn_printf( KIK_DEBUG_TAG " open_screen_intern() failed.\n") ;
@@ -1284,15 +1451,6 @@ x_screen_manager_init(
 	}
 #endif
 
-	if( ! x_termcap_init( &termcap))
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " x_termcap_init failed.\n") ;
-	#endif
-	
-		return  0 ;
-	}
-
 	if( *main_config.disp_name)
 	{
 		/*
@@ -1319,7 +1477,11 @@ x_screen_manager_init(
 #else
 	system_listener.open_screen = open_screen ;
 #endif
+	system_listener.split_screen = split_screen ;
 	system_listener.close_screen = close_screen ;
+	system_listener.next_screen = next_screen ;
+	system_listener.prev_screen = prev_screen ;
+	system_listener.resize_screen = resize_screen ;
 	system_listener.open_pty = open_pty ;
 	system_listener.next_pty = next_pty ;
 	system_listener.prev_pty = prev_pty ;
@@ -1360,7 +1522,6 @@ x_screen_manager_final(void)
 
 	ml_color_config_final() ;
 	x_shortcut_final( &shortcut) ;
-	x_termcap_final( &termcap) ;
 
 	return  1 ;
 }
@@ -1405,12 +1566,16 @@ x_screen_manager_startup(void)
 	{
 		/* reload ~/.mlterm/main. */
 		config_saved() ;
+		x_shortcut_final( &shortcut) ;
+		x_shortcut_init( &shortcut) ;
+		ml_color_config_final() ;
+		ml_color_config_init() ;
 	}
 #endif
 
 	for( count = 0 ; count < num_of_startup_screens ; count ++)
 	{
-		if( ! open_screen_intern( NULL))
+		if( ! open_screen_intern( ml_get_detached_term( NULL) , NULL , 0 , 0))
 		{
 		#ifdef  DEBUG
 			kik_warn_printf( KIK_DEBUG_TAG " open_screen_intern() failed.\n") ;
