@@ -807,7 +807,6 @@ x_scrollbar_init(
 	x_picture_modifier_t *  pic_mod
 	)
 {
-	char *  default_view_name = "simple" ;
 	u_int  width ;
 
 	/* dynamically allocated */
@@ -822,7 +821,7 @@ x_scrollbar_init(
 	}
 	else
 	{
-		sb->view_name = strdup( default_view_name) ;
+		sb->view_name = strdup( "simple") ;
 	}
 
 	if( sb->view_name == NULL)
@@ -908,7 +907,7 @@ view_created:
 	sb->is_pressing_up_button = 0 ;
 	sb->is_pressing_down_button = 0 ;
 
-	if( ! x_window_init( &sb->window , width , height , width , 0 , 0 , 0 , 0 , 0 , 0))
+	if( ! x_window_init( &sb->window , width , height , width , 0 , 0 , 0 , 0 , 0 , 0 , 0))
 	{
 		goto  error ;
 	}
@@ -944,15 +943,7 @@ view_created:
 		x_window_set_transparent( &sb->window , pic_mod) ;
 	}
 
-	/* default scrollbar cursor */
-	if( strcmp( sb->view_name , default_view_name) == 0)
-	{
-		x_window_set_cursor( &sb->window , XC_sb_v_double_arrow) ;
-	}
-	else
-	{
-		x_window_set_cursor( &sb->window , XC_left_ptr) ;
-	}
+	x_window_set_cursor( &sb->window , XC_left_ptr) ;
 
 	/*
 	 * event callbacks.
