@@ -49,6 +49,13 @@ static key_func_table_t  key_func_table[] =
 	{ "OPEN_PTY" , OPEN_PTY , } ,
 	{ "NEXT_PTY" , NEXT_PTY , } ,
 	{ "PREV_PTY" , PREV_PTY , } ,
+	{ "VSPLIT_SCREEN" , VSPLIT_SCREEN , } ,
+	{ "HSPLIT_SCREEN" , HSPLIT_SCREEN , } ,
+	{ "NEXT_SCREEN" , NEXT_SCREEN , } ,
+	{ "PREV_SCREEN" , PREV_SCREEN , } ,
+	{ "CLOSE_SCREEN" , CLOSE_SCREEN , } ,
+	{ "HEXPAND_SCREEN" , HEXPAND_SCREEN , } ,
+	{ "VEXPAND_SCREEN" , VEXPAND_SCREEN , } ,
 	{ "PAGE_UP" , PAGE_UP , } ,
 	{ "PAGE_DOWN" , PAGE_DOWN , } ,
 	{ "SCROLL_UP" , SCROLL_UP , } ,
@@ -135,7 +142,7 @@ x_shortcut_init(
 
 		/* EXT_KBD(obsolete) */
 		{ 0 , 0 , 0 , } ,
-		
+
 		/* OPEN_SCREEN */
 		{ XK_F1 , ControlMask , 1 , } ,
 
@@ -148,6 +155,27 @@ x_shortcut_init(
 		/* PREV_PTY */
 		{ XK_F4 , ControlMask , 1 , } ,
 
+		/* HSPLIT_SCREEN */
+		{ XK_F1 , ShiftMask , 1 , } ,
+
+		/* VSPLIT_SCREEN */
+		{ XK_F2 , ShiftMask , 1 , } ,
+
+		/* NEXT_SCREEN */
+		{ XK_F3 , ShiftMask , 1 , } ,
+
+		/* PREV_SCREEN */
+		{ XK_F4 , ShiftMask , 1 , } ,
+
+		/* CLOSE_SCREEN */
+		{ XK_F5 , ShiftMask , 1 , } ,
+
+		/* HEXPAND_SCREEN */
+		{ XK_F6 , ShiftMask , 1 , } ,
+
+		/* VEXPAND_SCREEN */
+		{ XK_F7 , ShiftMask , 1 , } ,
+
 		/* PAGE_UP(compatible with kterm) */
 		{ XK_Prior , ShiftMask , 1 , } ,
 
@@ -242,26 +270,15 @@ x_shortcut_match(
 	{
 		return  0 ;
 	}
-	
-	if( shortcut->map[func].state != 0)
-	{
-		/* ingoring except these masks */
-		state &= (ModMask|ControlMask|ShiftMask|button_mask) ;
-		
-		if( ((shortcut->map[func].state & ModMask) == ModMask) &&
-		     (state & ModMask))
-		{
-			/* all ModNMasks are set. */
-			state |= ModMask ;
-		}
 
-		if( state != shortcut->map[func].state)
-		{
-			return  0 ;
-		}
-	}
+	/* ingoring except these masks */
+	state &= (ModMask|ControlMask|ShiftMask|button_mask) ;
 
-	if( shortcut->map[func].ksym == ksym)
+	if( shortcut->map[func].ksym == ksym &&
+	    shortcut->map[func].state ==
+	      ( state |
+	        ( (state & ModMask) &&
+	          (shortcut->map[func].state & ModMask) == ModMask ? ModMask : 0)) )
 	{
 		return  1 ;
 	}
@@ -330,7 +347,8 @@ x_shortcut_parse(
 		{
 			state |= ShiftMask ;
 		}
-		else if( strcmp( key , "Mod") == 0)
+		else if( strcmp( key , "Mod") == 0 ||
+		         strcmp( key , "Alt") == 0)
 		{
 			state |= ModMask ;
 		}
