@@ -22,6 +22,13 @@ typedef enum  x_key_func
 	OPEN_PTY ,
 	NEXT_PTY ,
 	PREV_PTY ,
+	HSPLIT_SCREEN ,
+	VSPLIT_SCREEN ,
+	NEXT_SCREEN ,
+	PREV_SCREEN ,
+	CLOSE_SCREEN ,
+	HEXPAND_SCREEN ,
+	VEXPAND_SCREEN ,
 	PAGE_UP ,
 	PAGE_DOWN ,
 	SCROLL_UP ,
