@@ -96,8 +96,8 @@ typedef struct  x_window
 
 	long  event_mask ;
 
-	int  x ;
-	int  y ;
+	int  x ;			/* relative to a root window. */
+	int  y ;			/* relative to a root window. */
 	u_int  width ;
 	u_int  height ;
 	u_int  min_width ;
@@ -150,11 +150,13 @@ typedef struct  x_window
 #else
 	int8_t  wall_picture_is_set ;	/* Actually set picture (including transparency) or not. */
 	int8_t  wait_copy_area_response ;	/* Used for XCopyArea() */
+	int8_t  configure_root ;
 #endif
 	int8_t  is_sel_owner ;
 	int8_t  is_transparent ;
 	int8_t  is_scrollable ;
 	int8_t  is_focused ;
+	int8_t  inputtable ;	/* 1: focused, -1: unfocused */
 	int8_t  is_mapped ;
 	int8_t  create_gc ;
 
@@ -198,7 +200,7 @@ typedef struct  x_window
 int  x_window_init( x_window_t *  win ,
 	u_int  width , u_int  height , u_int  min_width , u_int  min_height ,
 	u_int  width_inc , u_int  height_inc , u_int  hmargin , u_int  vmargin ,
-	int  create_gc) ;
+	int  create_gc , int  inputtable) ;
 
 int  x_window_final( x_window_t *  win) ;
 
@@ -214,9 +216,9 @@ int  x_window_remove_event_mask( x_window_t *  win , long  event_mask) ;
 
 int  x_window_ungrab_pointer( x_window_t *  win) ;
 
-int  x_window_set_wall_picture( x_window_t *  win , Pixmap  pic) ;
+int  x_window_set_wall_picture( x_window_t *  win , Pixmap  pic , int  do_expose) ;
 
-int  x_window_unset_wall_picture( x_window_t *  win) ;
+int  x_window_unset_wall_picture( x_window_t *  win , int  do_expose) ;
 
 #if  defined(USE_WIN32GUI) || defined(USE_FRAMEBUFFER)
 #define  x_window_has_wall_picture( win)  ((win)->wall_picture != None)
@@ -236,6 +238,8 @@ int  x_window_set_bg_color( x_window_t *  win , x_color_t *  bg_color) ;
 
 int  x_window_add_child( x_window_t *  win , x_window_t *  child , int  x , int  y , int  map) ;
 
+int  x_window_remove_child( x_window_t *  win , x_window_t *  child) ;
+
 x_window_t *  x_get_root_window( x_window_t *  win) ;
 
 GC  x_window_get_fg_gc( x_window_t *  win) ;
@@ -408,6 +412,8 @@ int  x_window_bell( x_window_t *  win , x_bel_mode_t  mode) ;
 int  x_window_translate_coordinates( x_window_t *  win, int x, int y,
 	int *  global_x, int *  global_y, Window *  child) ;
 
+void  x_window_set_input_focus( x_window_t *  win) ;
+
 #ifdef  DEBUG
 void  x_window_dump_children( x_window_t *  win) ;
 #endif
