@@ -593,7 +593,7 @@ x_display_get_cursor(
 	
 	/*
 	 * XXX
-	 * cursor[0] == XC_xterm / cursor[1] == XC_sb_v_double_arrow / cursor[2] == XC_left_ptr
+	 * cursor[0] == XC_xterm / cursor[1] == XC_left_ptr / cursor[2] == XC_nil
 	 * Mlterm uses only these shapes.
 	 */
 	 
@@ -601,11 +601,11 @@ x_display_get_cursor(
 	{
 		idx = 0 ;
 	}
-	else if( shape == XC_sb_v_double_arrow)
+	else if( shape == XC_left_ptr)
 	{
 		idx = 1 ;
 	}
-	else if( shape == XC_left_ptr)
+	else if( shape == XC_nil)
 	{
 		idx = 2 ;
 	}
@@ -613,10 +613,28 @@ x_display_get_cursor(
 	{
 		return  None ;
 	}
-	
+
 	if( ! disp->cursors[idx])
 	{
-		disp->cursors[idx] = XCreateFontCursor( disp->display , shape) ;
+		if( idx == 2)
+		{
+			XFontStruct *  font ;
+			XColor  dummy ;
+
+			if( ! ( font = XLoadQueryFont( disp->display , "nil2")))
+			{
+				return  None ;
+			}
+
+			disp->cursors[idx] = XCreateGlyphCursor( disp->display ,
+						font->fid , font->fid ,
+						'X', ' ', &dummy , &dummy) ;
+			XFreeFont( disp->display , font) ;
+		}
+		else
+		{
+			disp->cursors[idx] = XCreateFontCursor( disp->display , shape) ;
+		}
 	}
 
 	return  disp->cursors[idx] ;
