@@ -1256,15 +1256,15 @@ x_font_get_encoding_names(
 	}
 }
 
-/* For mlterm-libvte */
 void
-x_font_use_point_size_for_fc(
+x_font_use_point_size(
 	int  bool
 	)
 {
 	use_point_size_for_fc = bool ;
 }
 
+/* For mlterm-libvte */
 void
 x_font_set_dpi_for_fc(
 	double  dpi
