@@ -40,12 +40,18 @@
 /*
  * 'data' which is malloc'ed for XCreateImage() in pixbuf_to_ximage_truecolor()
  * is free'ed in XDestroyImage().
- * If malloc is replaced kik_mem_malloc in kik_mem.h, kik_mem_free_all() will
- * free 'data' which is already free'ed in XDestroyImage() and
- * segmentation fault error can happen.
  */
-#if  defined(KIK_DEBUG) && defined(malloc)
-#undef malloc
+#ifdef  KIK_DEBUG
+static void
+destroy_image(
+	XImage *  image
+	)
+{
+	kik_mem_remove(image->data , __FILE__ , __LINE__ , __FUNCTION__) ;
+	XDestroyImage(image) ;
+}
+#undef  XDestroyImage
+#define  XDestroyImage(image)  destroy_image(image)
 #endif
 
 #if  1
@@ -85,6 +91,10 @@
 	 ((((g) >> (rgbinfo).g_limit) << (rgbinfo).g_offset) & (rgbinfo).g_mask) | \
 	 ((((b) >> (rgbinfo).b_limit) << (rgbinfo).b_offset) & (rgbinfo).b_mask) )
 
+#if  1
+#define  BUILTIN_SIXEL
+#endif
+
 
 typedef struct  rgb_info
 {
@@ -403,6 +413,145 @@ modify_pixmap(
 }
 
 
+#if  defined(BUILTIN_IMAGELIB) || defined(BUILTIN_SIXEL)
+
+#ifndef  BUILTIN_IMAGELIB
+#define  CARD_HEAD_SIZE  0
+#include  "../../common/c_sixel.c"
+#endif
+
+static int
+load_sixel(
+	x_display_t *  disp ,
+	char *  path ,
+	Pixmap *  pixmap ,
+	Pixmap *  mask ,		/* Can be NULL */
+	u_int *  width ,		/* Can be NULL */
+	u_int *  height			/* Can be NULL */
+	)
+{
+	XImage *  image ;
+	u_int32_t *  data ;
+	u_int32_t *  in ;
+	u_char *  out ;
+	u_int  w ;
+	u_int  h ;
+	u_int  x ;
+	u_int  y ;
+	XVisualInfo *  vinfo ;
+	rgb_info_t  rgbinfo ;
+	int  bytes_per_pixel ;
+	GC  mask_gc ;
+	XGCValues  gcv ;
+
+	if( disp->depth < 16 ||
+	    ! ( data = in = out = load_sixel_from_file( path , &w , &h)))
+	{
+		return  0 ;
+	}
+
+	vinfo = x_display_get_visual_info( disp) ;
+	rgb_info_init( vinfo , &rgbinfo) ;
+	XFree( vinfo) ;
+
+	if( disp->depth == 16)
+	{
+		bytes_per_pixel = 2 ;
+	}
+	else
+	{
+		bytes_per_pixel = 4 ;
+	}
+
+	if( mask)
+	{
+		*mask = None ;
+	}
+
+	for( y = 0 ; y < h ; y++)
+	{
+		for( x = 0 ; x < w ; x++)
+		{
+			u_int32_t  pixel ;
+
+		#ifdef  GDK_PIXBUF_VERSION
+			if( mask && ((u_char*)in)[3] <= 0x7f)
+		#else
+			if( mask && in[0] <= 0x7fffffff)
+		#endif
+			{
+				if( *mask == None)
+				{
+					*mask = XCreatePixmap( disp->display ,
+							x_display_get_group_leader( disp) ,
+							w , h , 1) ;
+					mask_gc = XCreateGC( disp->display , *mask ,
+							0 , &gcv) ;
+
+					XSetForeground( disp->display , mask_gc , 1) ;
+					XFillRectangle( disp->display , *mask , mask_gc ,
+						0 , 0 , w , h) ;
+					XSetForeground( disp->display , mask_gc , 0) ;
+				}
+
+				XDrawPoint( disp->display , *mask , mask_gc , x , y) ;
+			}
+
+		#ifdef  GDK_PIXBUF_VERSION
+			pixel = RGB_TO_PIXEL( ((u_char*)in)[0] , ((u_char*)in)[1] ,
+					((u_char*)in)[2] , rgbinfo) ;
+		#else
+			pixel = RGB_TO_PIXEL( (in[0] >> 16) & 0xff ,
+					(in[0] >> 8) & 0xff , in[0] & 0xff , rgbinfo) ;
+		#endif
+
+			if( bytes_per_pixel == 2)
+			{
+				*((u_int16_t*)out) = pixel ;
+			}
+			else /* if( bytes_per_pixel == 4) */
+			{
+				*((u_int32_t*)out) = (pixel | (in[0] & 0xff000000)) ;
+			}
+
+			in ++ ;
+			out += bytes_per_pixel ;
+		}
+	}
+
+	if( mask && *mask)
+	{
+		XFreeGC( disp->display , mask_gc) ;
+	}
+
+	image = XCreateImage( disp->display , disp->visual ,
+			disp->depth , ZPixmap , 0 , data , w , h ,
+			/* in case depth isn't multiple of 8 */
+			bytes_per_pixel * 8 ,
+			w * bytes_per_pixel) ;
+
+	*pixmap = XCreatePixmap( disp->display , x_display_get_group_leader( disp) ,
+			w , h , disp->depth) ;
+
+	XPutImage( disp->display , *pixmap , disp->gc->gc , image ,
+			0 , 0 , 0 , 0 , w , h) ;
+	XDestroyImage( image) ;
+
+	if( width)
+	{
+		*width = w ;
+	}
+	if( height)
+	{
+		*height = h ;
+	}
+
+	return  1 ;
+}
+
+#endif	/* BUILTIN_SIXEL */
+
+
 #ifdef  BUILTIN_IMAGELIB
 
 /* create GdkPixbuf from the specified file path.
@@ -1782,6 +1931,16 @@ x_imagelib_load_file(
 		dst_height = *height ;
 	}
 
+#if  defined(BUILTIN_IMAGELIB) || defined(BUILTIN_SIXEL)
+	if( ! cardinal &&
+	    strcasecmp( path + strlen(path) - 4 , ".six") == 0 &&
+	    dst_width == 0 && dst_height == 0 &&
+	    load_sixel( disp , path , pixmap , mask , width , height))
+	{
+		return  1 ;
+	}
+#endif
+
 #ifdef  BUILTIN_IMAGELIB
 
 	if( path)
