@@ -68,6 +68,12 @@
 #define  IS_INHERIT_TRANSPARENT(win) \
 		( use_inherit_transparent && x_picture_modifier_is_normal( (win)->pic_mod))
 
+/* win->width is not multiples of (win)->width_inc if window is maximized. */
+#define  RIGHT_MARGIN(win) \
+	((win)->width_inc ? ((win)->width - (win)->min_width) % (win)->width_inc : 0)
+#define  BOTTOM_MARGIN(win) \
+	((win)->height_inc ? ((win)->height - (win)->min_height) % (win)->height_inc : 0)
+
 
 typedef struct {
 	u_int32_t  flags ;
@@ -97,7 +103,7 @@ typedef struct {
 static int  click_interval = 250 ;	/* millisecond, same as xterm. */
 /* ParentRelative isn't used for transparency by default */
 static int  use_inherit_transparent = 0 ;
-static int  use_clipboard = 0 ;
+static int  use_clipboard = 1 ;
 static int  use_urgent_bell = 0 ;
 
 static struct
@@ -166,21 +172,36 @@ clear_margin_area(
 	x_window_t *  win
 	)
 {
+	u_int  right_margin ;
+	u_int  bottom_margin ;
+
+	right_margin = RIGHT_MARGIN(win) ;
+	bottom_margin = BOTTOM_MARGIN(win) ;
+
 	if( win->hmargin > 0)
 	{
 		XClearArea( win->disp->display , win->my_window ,
 			0 , 0 , win->hmargin , ACTUAL_HEIGHT(win) , 0) ;
+	}
+
+	if( win->hmargin + right_margin > 0)
+	{
 		XClearArea( win->disp->display , win->my_window ,
-			win->width + win->hmargin , 0 , win->hmargin , ACTUAL_HEIGHT(win) , 0) ;
+			win->width - right_margin + win->hmargin , 0 ,
+			win->hmargin + right_margin , ACTUAL_HEIGHT(win) , 0) ;
 	}
 
 	if( win->vmargin > 0)
 	{
 		XClearArea( win->disp->display , win->my_window ,
-			win->hmargin , 0 , win->width , win->vmargin , 0) ;
+			win->hmargin , 0 , win->width - right_margin , win->vmargin , 0) ;
+	}
+
+	if( win->vmargin + bottom_margin > 0)
+	{
 		XClearArea( win->disp->display , win->my_window ,
-			win->hmargin , win->height + win->vmargin ,
-			win->width , win->vmargin , 0) ;
+			win->hmargin , win->height - bottom_margin + win->vmargin ,
+			win->width - right_margin , win->vmargin + bottom_margin , 0) ;
 	}
 
 	return  1 ;
@@ -201,7 +222,7 @@ set_transparent_picture(
 	 */
 	win->is_transparent = 0 ;
 
-	if( ! x_window_set_wall_picture( win , pixmap))
+	if( ! x_window_set_wall_picture( win , pixmap , 1))
 	{
 		win->pic_mod = NULL ;
 
@@ -266,7 +287,7 @@ unset_transparent(
 	win->is_transparent = 0 ;
 	win->pic_mod = NULL ;
 
-	return  x_window_unset_wall_picture( win) ;
+	return  x_window_unset_wall_picture( win , 1) ;
 }
 
 static int
@@ -291,7 +312,7 @@ set_transparent(
 		}
 		else
 		{
-			kik_msg_printf( "_X_ROOTPMAP_ID is not found."
+			kik_msg_printf( "_XROOTPMAP_ID is not found."
 				" Trying ParentRelative for transparency instead.\n") ;
 
 			if( ! x_picture_modifier_is_normal( win->pic_mod))
@@ -375,7 +396,7 @@ notify_focus_in_to_children(
 {
 	u_int  count ;
 
-	if( ! win->is_focused)
+	if( ! win->is_focused && win->inputtable > 0)
 	{
 		win->is_focused = 1 ;
 
@@ -550,6 +571,7 @@ total_min_width(
 	{
 		if( win->children[count]->is_mapped)
 		{
+			/* XXX */
 			min_width += total_min_width( win->children[count]) ;
 		}
 	}
@@ -571,6 +593,7 @@ total_min_height(
 	{
 		if( win->children[count]->is_mapped)
 		{
+			/* XXX */
 			min_height += total_min_height( win->children[count]) ;
 		}
 	}
@@ -817,6 +840,28 @@ right_shift(
 	return  shift ;
 }
 
+static void
+reset_input_focus(
+	x_window_t *  win
+	)
+{
+	u_int  count ;
+
+	if( win->inputtable)
+	{
+		win->inputtable = -1 ;
+	}
+	else
+	{
+		win->inputtable = 0 ;
+	}
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		reset_input_focus( win->children[count]) ;
+	}
+}
+
 
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE)
 
@@ -985,7 +1030,8 @@ x_window_init(
 	u_int  height_inc ,
 	u_int  hmargin ,
 	u_int  vmargin ,
-	int  create_gc
+	int  create_gc ,
+	int  inputtable
 	)
 {
 	memset( win , 0 , sizeof( x_window_t)) ;
@@ -999,6 +1045,7 @@ x_window_init(
 	/* If wall picture is set, scrollable will be 0. */
 	win->is_scrollable = 1 ;
 
+#if  0
 	/*
 	 * is_focus member shoule be 0 by default in order to call
 	 * XSetICFocus(x_xic_set_focus) in startup FocusIn event.
@@ -1006,6 +1053,9 @@ x_window_init(
 	 * in XFilterEvent.
 	 */
 	win->is_focused = 0 ;
+#endif
+
+	win->inputtable = inputtable ;
 
 	/* This flag will map window automatically in x_window_show() */
 	win->is_mapped = 1 ;
@@ -1190,9 +1240,12 @@ x_window_ungrab_pointer(
 int
 x_window_set_wall_picture(
 	x_window_t *  win ,
-	Pixmap  pic
+	Pixmap  pic ,
+	int  do_expose
 	)
 {
+	u_int  count ;
+
 	if( win->is_transparent)
 	{
 		/*
@@ -1206,27 +1259,38 @@ x_window_set_wall_picture(
 	win->wall_picture_is_set = 1 ;
 	win->is_scrollable = 0 ;
 
-	clear_margin_area( win) ;
+	if( do_expose)
+	{
+		clear_margin_area( win) ;
 
-	if( win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
-#if  0
-	else
+
+	for( count = 0 ; count < win->num_of_children ; count++)
 	{
-		x_window_clear_all( win) ;
+		x_window_set_wall_picture( win->children[count] , ParentRelative , do_expose) ;
 	}
-#endif
 
 	return  1 ;
 }
 
 int
 x_window_unset_wall_picture(
-	x_window_t *  win
+	x_window_t *  win ,
+	int  do_expose
 	)
 {
+	u_int  count ;
+
 	/*
 	 * win->wall_picture_is_set == 0 doesn't mean that following codes
 	 * to disable wall picture is already processed.
@@ -1261,18 +1325,26 @@ x_window_unset_wall_picture(
 	win->wall_picture_is_set = 0 ;
 	win->is_scrollable = 1 ;
 
-	clear_margin_area( win) ;
+	if( do_expose)
+	{
+		clear_margin_area( win) ;
 
-	if( win->window_exposed)
-	{
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
-#if  0
-	else
+
+	for( count = 0 ; count < win->num_of_children ; count++)
 	{
-		x_window_clear_all( win) ;
+		x_window_unset_wall_picture( win->children[count] , do_expose) ;
 	}
-#endif
 
 	return  1 ;
 }
@@ -1364,7 +1436,20 @@ x_window_set_cursor(
 	u_int  cursor_shape
 	)
 {
-	win->cursor_shape = cursor_shape ;
+	if( win->my_window == None)
+	{
+		win->cursor_shape = cursor_shape ;
+	}
+	else
+	{
+		Cursor  cursor ;
+
+		if( ( cursor = x_display_get_cursor( win->disp ,
+				( win->cursor_shape = cursor_shape))))
+		{
+			XDefineCursor( win->disp->display , win->my_window , cursor) ;
+		}
+	}
 
 	return  1 ;
 }
@@ -1434,13 +1519,39 @@ x_window_add_child(
 	child->parent = win ;
 	child->x = x ;
 	child->y = y ;
-	child->is_mapped = map ;
+
+	if( ! ( child->is_mapped = map) && child->inputtable > 0)
+	{
+		child->inputtable = -1 ;
+	}
 
 	win->children[ win->num_of_children ++] = child ;
 
 	return  1 ;
 }
 
+int
+x_window_remove_child(
+	x_window_t *  win ,
+	x_window_t *  child
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < win->num_of_children ; count++)
+	{
+		if( win->children[count] == child)
+		{
+			child->parent = NULL ;
+			win->children[count] = win->children[--win->num_of_children] ;
+
+			return  1 ;
+		}
+	}
+
+	return  0 ;
+}
+
 x_window_t *
 x_get_root_window(
 	x_window_t *  win
@@ -1585,8 +1696,8 @@ x_window_show(
 
 		size_hints.width_inc = total_width_inc( win) ;
 		size_hints.height_inc = total_height_inc( win) ;
-		size_hints.min_width = total_min_width( win) ;
-		size_hints.min_height = total_min_height( win) ;
+		size_hints.min_width = total_min_width( win) + size_hints.width_inc ;
+		size_hints.min_height = total_min_height( win) + size_hints.height_inc ;
 		size_hints.base_width = size_hints.min_width > size_hints.width_inc ?
 					size_hints.min_width - size_hints.width_inc : 0 ;
 		size_hints.base_height = size_hints.min_height > size_hints.height_inc ?
@@ -1657,6 +1768,12 @@ x_window_show(
 		XSetWMProtocols( win->disp->display , win->my_window , protocols , 2) ;
 	}
 
+	if( win->parent && ! win->parent->is_transparent &&
+	    win->parent->wall_picture_is_set)
+	{
+		x_window_set_wall_picture( win , ParentRelative , 0) ;
+	}
+
 	/*
 	 * This should be called after Window Manager settings, because
 	 * x_set_{window|icon}_name() can be called in win->window_realized().
@@ -1699,6 +1816,12 @@ x_window_show(
 	{
 		XMapWindow( win->disp->display , win->my_window) ;
 
+		if( win->inputtable > 0)
+		{
+			reset_input_focus( x_get_root_window( win)) ;
+			win->inputtable = 1 ;
+		}
+
 	#if  0
 		x_window_clear_all( win) ;
 	#endif
@@ -1755,7 +1878,7 @@ x_window_resize(
 	/* Max width of each window is DisplayWidth(). */
 	if( (flag & LIMIT_RESIZE) && win->disp->width < width)
 	{
-		win->width = win->disp->width ;
+		win->width = win->disp->width - win->hmargin * 2 ;
 	}
 	else
 	{
@@ -1765,7 +1888,7 @@ x_window_resize(
 	/* Max height of each window is DisplayHeight(). */
 	if( (flag & LIMIT_RESIZE) && win->disp->height < height)
 	{
-		win->height = win->disp->height ;
+		win->height = win->disp->height - win->vmargin * 2 ;
 	}
 	else
 	{
@@ -1792,6 +1915,11 @@ x_window_resize(
 	}
 #endif
 
+	if( ! win->configure_root && ! (flag & NOTIFY_TO_PARENT) && win->parent)
+	{
+		notify_configure_to_children( win) ;
+	}
+
 	return  1 ;
 }
 
@@ -1843,8 +1971,8 @@ x_window_set_normal_hints(
 	 */
 	size_hints.width_inc = total_width_inc( root) ;
 	size_hints.height_inc = total_height_inc( root) ;
-	size_hints.min_width = total_min_width( root) ;
-	size_hints.min_height = total_min_height( root) ;
+	size_hints.min_width = total_min_width( root) + size_hints.width_inc ;
+	size_hints.min_height = total_min_height( root) + size_hints.height_inc ;
 	size_hints.base_width = size_hints.min_width - size_hints.width_inc ;
 	size_hints.base_height = size_hints.min_height - size_hints.height_inc ;
 	size_hints.flags = PMinSize | PResizeInc | PBaseSize ;
@@ -1898,7 +2026,9 @@ x_window_set_override_redirect(
 		XMapWindow( root->disp->display , root->my_window) ;
 	}
 
-	win->is_focused = 1 ;	/* XXX Always focused not to execute XSetInputFocus(). */
+	reset_input_focus( root) ;
+	/* XXX Always focused not to execute XSetInputFocus(). */
+	win->inputtable = win->is_focused = 1 ;
 
 	return  1 ;
 }
@@ -1957,11 +2087,21 @@ x_window_move(
 	int  y
 	)
 {
+	if( win->x == x && win->y == y)
+	{
+		return  0 ;
+	}
+
 	win->x = x ;
 	win->y = y ;
 
 	XMoveWindow( win->disp->display , win->my_window , x , y) ;
 
+	if( ! win->configure_root && win->parent)
+	{
+		notify_configure_to_children( win) ;
+	}
+
 	return  1 ;
 }
 
@@ -2068,7 +2208,9 @@ x_window_blank(
 	restore_fg_color( win) ;
 
 	XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
-		win->hmargin , win->vmargin , win->width , win->height) ;
+		win->hmargin , win->vmargin ,
+		win->width - RIGHT_MARGIN(win) ,
+		win->height - BOTTOM_MARGIN(win)) ;
 
 	return  1 ;
 }
@@ -2392,11 +2534,10 @@ x_window_receive_event(
 		}
 
 		/* XXX Note that win->is_focused is always true on override redirect mode. */
-		if( ! win->is_focused && event->xbutton.button == Button1 &&
+		if( ! win->is_focused && win->inputtable && event->xbutton.button == Button1 &&
 		    ! event->xbutton.state)
 		{
-			XSetInputFocus( win->disp->display , win->my_window ,
-				RevertToParent , CurrentTime) ;
+			x_window_set_input_focus( win) ;
 		}
 	}
 	else if( event->type == NoExpose)
@@ -2415,9 +2556,9 @@ x_window_receive_event(
 		int  nskip = 0 ;
 	#endif
 
-		/* Optimize redrawing. (Don't check GraphicsExpose) */
+		/* Optimize redrawing. */
 		while( XCheckTypedWindowEvent( win->disp->display , win->my_window ,
-			Expose , &next_ev))
+			event->type , &next_ev))
 		{
 			XEvent  ev ;
 			int  diff ;
@@ -2653,7 +2794,9 @@ x_window_receive_event(
 
 			if( win->window_resized)
 			{
+				win->configure_root = 1 ;
 				(*win->window_resized)( win) ;
+				win->configure_root = 0 ;
 			}
 
 			is_changed = 1 ;
@@ -4256,6 +4399,17 @@ x_window_translate_coordinates(
 	return  1 ;
 }
 
+void
+x_window_set_input_focus(
+	x_window_t *  win
+	)
+{
+	reset_input_focus( x_get_root_window( win)) ;
+	win->inputtable = 1 ;
+	XSetInputFocus( win->disp->display , win->my_window ,
+		RevertToParent , CurrentTime) ;
+}
+
 #ifdef  DEBUG
 void
 x_window_dump_children(
